@@ -80,7 +80,7 @@ public final class ConstantDynamic extends ConstantCP {
      * for use by ConstantInvokeDynamic.
      * @since 6.0
      */
-    public final int getBootstrapMethodAttrIndex() {
+    public int getBootstrapMethodAttrIndex() {
         return super.getClassIndex();  // AKA bootstrap_method_attr_index
     }
 
@@ -88,7 +88,7 @@ public final class ConstantDynamic extends ConstantCP {
      * @return String representation
      */
     @Override
-    public final String toString() {
+    public String toString() {
         return super.toString().replace("class_index", "bootstrap_method_attr_index");
     }
 }
