@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2007, 2023, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2007, 2020, Oracle and/or its affiliates. All rights reserved.
  * ORACLE PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
  */
 /**
@@ -46,7 +46,7 @@ public class UnsyncByteArrayOutputStream extends OutputStream  {
 
     public void write(byte[] arg0) {
         if ((VM_ARRAY_INDEX_MAX_VALUE - pos) < arg0.length) {
-            throw new OutOfMemoryError();
+            throw new OutOfMemoryError("Required length exceeds implementation limit");
         }
         int newPos = pos + arg0.length;
         if (newPos > size) {
@@ -58,7 +58,7 @@ public class UnsyncByteArrayOutputStream extends OutputStream  {
 
     public void write(byte[] arg0, int arg1, int arg2) {
         if ((VM_ARRAY_INDEX_MAX_VALUE - pos) < arg2) {
-            throw new OutOfMemoryError();
+            throw new OutOfMemoryError("Required length exceeds implementation limit");
         }
         int newPos = pos + arg2;
         if (newPos > size) {
@@ -70,7 +70,7 @@ public class UnsyncByteArrayOutputStream extends OutputStream  {
 
     public void write(int arg0) {
         if (VM_ARRAY_INDEX_MAX_VALUE - pos == 0) {
-            throw new OutOfMemoryError();
+            throw new OutOfMemoryError("Required length exceeds implementation limit");
         }
         int newPos = pos + 1;
         if (newPos > size) {
@@ -80,7 +80,7 @@ public class UnsyncByteArrayOutputStream extends OutputStream  {
     }
 
     public byte[] toByteArray() {
-        byte[] result = new byte[pos];
+        byte result[] = new byte[pos];
         System.arraycopy(buf, 0, result, 0, pos);
         return result;
     }
@@ -111,7 +111,7 @@ public class UnsyncByteArrayOutputStream extends OutputStream  {
                 newSize = VM_ARRAY_INDEX_MAX_VALUE;
             }
         }
-        byte[] newBuf = new byte[newSize];
+        byte newBuf[] = new byte[newSize];
         System.arraycopy(buf, 0, newBuf, 0, pos);
         buf = newBuf;
         size = newSize;
