@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2005, 2018, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2005, 2019, Oracle and/or its affiliates. All rights reserved.
  * ORACLE PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
  *
  *
@@ -882,7 +882,7 @@ public class JavacFiler implements Filer, Closeable {
          */
         if ((typeName != null)) {
             if (!(fileObject instanceof JavaFileObject))
-                throw new AssertionError("JavaFileOject not found for " + fileObject);
+                throw new AssertionError("JavaFileObject not found for " + fileObject);
             JavaFileObject javaFileObject = (JavaFileObject)fileObject;
             switch(javaFileObject.getKind()) {
             case SOURCE:
