@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 1994, 2008, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 1994, 2019, Oracle and/or its affiliates. All rights reserved.
  * ORACLE PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
  *
  *
@@ -28,8 +28,8 @@ package java.lang;
 /**
  * Thrown to indicate that a thread is not in an appropriate state
  * for the requested operation. See, for example, the
- * <code>suspend</code> and <code>resume</code> methods in class
- * <code>Thread</code>.
+ * {@code suspend} and {@code resume} methods in class
+ * {@code Thread}.
  *
  * @author  unascribed
  * @see     java.lang.Thread#resume()
@@ -37,10 +37,11 @@ package java.lang;
  * @since   1.0
  */
 public class IllegalThreadStateException extends IllegalArgumentException {
+    @java.io.Serial
     private static final long serialVersionUID = -7626246362397460174L;
 
     /**
-     * Constructs an <code>IllegalThreadStateException</code> with no
+     * Constructs an {@code IllegalThreadStateException} with no
      * detail message.
      */
     public IllegalThreadStateException() {
@@ -48,7 +49,7 @@ public class IllegalThreadStateException extends IllegalArgumentException {
     }
 
     /**
-     * Constructs an <code>IllegalThreadStateException</code> with the
+     * Constructs an {@code IllegalThreadStateException} with the
      * specified detail message.
      *
      * @param   s   the detail message.
