@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 1999, 2018, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 1999, 2020, Oracle and/or its affiliates. All rights reserved.
  * ORACLE PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
  *
  *
@@ -37,44 +37,39 @@ class VersionProps {
         "java";
 
     private static final String java_version =
-        "13.0.1";
+        "15.0.2";
 
     private static final String java_version_date =
-        "2019-10-15";
+        "2021-01-19";
 
+    // This field is read by HotSpot
     private static final String java_runtime_name =
         "Java(TM) SE Runtime Environment";
 
+    // This field is read by HotSpot
     private static final String java_runtime_version =
-        "13.0.1+9";
+        "15.0.2+7-27";
 
     private static final String VERSION_NUMBER =
-        "13.0.1";
+        "15.0.2";
 
     private static final String VERSION_SPECIFICATION =
-        "13";
+        "15";
 
     private static final String VERSION_BUILD =
-        "9";
+        "7";
 
     private static final String VERSION_PRE =
         "";
 
     private static final String VERSION_OPT =
-        "";
+        "27";
 
     private static final boolean isLTS =
-        "".startsWith("LTS");
+        "27".startsWith("LTS");
 
     private static final String CLASSFILE_MAJOR_MINOR =
-        "57.0";
-
-    private static final String VENDOR_VERSION_STRING =
-        "";
-
-    private static final String vendor_version =
-        (!VENDOR_VERSION_STRING.isEmpty()
-         ? " " + VENDOR_VERSION_STRING : "");
+        "59.0";
 
     private static final String VENDOR =
         "Oracle Corporation";
@@ -82,9 +77,21 @@ class VersionProps {
     private static final String VENDOR_URL =
         "https://java.oracle.com/";
 
-    private static final String VENDOR_URL_BUG =
+    // The remaining VENDOR_* fields must not be final,
+    // so that they can be redefined by jlink plugins
+
+    // This field is read by HotSpot
+    private static String VENDOR_VERSION =
+        "";
+
+    private static String VENDOR_URL_BUG =
         "https://bugreport.java.com/bugreport/";
 
+    // This field is read by HotSpot
+    @SuppressWarnings("unused")
+    private static String VENDOR_URL_VM_BUG =
+        "https://bugreport.java.com/bugreport/crash.jsp";
+
     /**
      * Initialize system properties using build provided values.
      *
@@ -95,8 +102,8 @@ class VersionProps {
         props.put("java.version.date", java_version_date);
         props.put("java.runtime.version", java_runtime_version);
         props.put("java.runtime.name", java_runtime_name);
-        if (!VENDOR_VERSION_STRING.isEmpty())
-            props.put("java.vendor.version", VENDOR_VERSION_STRING);
+        if (!VENDOR_VERSION.isEmpty())
+            props.put("java.vendor.version", VENDOR_VERSION);
 
         props.put("java.class.version", CLASSFILE_MAJOR_MINOR);
 
@@ -216,6 +223,9 @@ class VersionProps {
             jdk_debug_level = jdk_debug_level + " ";
         }
 
+        String vendor_version = (VENDOR_VERSION.isEmpty()
+                                 ? "" : " " + VENDOR_VERSION);
+
         ps.println(java_runtime_name + vendor_version
                    + " (" + jdk_debug_level + "build " + java_runtime_version + ")");
 
