@@ -935,7 +935,7 @@ public class SecureRandom extends java.util.Random {
                 }
             });
 
-        if ((property == null) || (property.length() == 0)) {
+        if (property == null || property.isEmpty()) {
             throw new NoSuchAlgorithmException(
                 "Null/empty securerandom.strongAlgorithms Security Property");
         }
@@ -1017,6 +1017,7 @@ public class SecureRandom extends java.util.Random {
     }
 
     // Declare serialVersionUID to be compatible with JDK1.1
+    @java.io.Serial
     static final long serialVersionUID = 4940670005562187L;
 
     // Retain unused values serialized from JDK1.1
@@ -1027,6 +1028,7 @@ public class SecureRandom extends java.util.Random {
     /**
      * @serial
      */
+    @SuppressWarnings("serial") // Not statically typed as Serializable
     private MessageDigest digest = null;
     /**
      * @serial
