@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2012, 2016, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2012, 2019, Oracle and/or its affiliates. All rights reserved.
  * ORACLE PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
  *
  *
@@ -133,6 +133,7 @@ public final class IsoChronology extends AbstractChronology implements Serializa
     /**
      * Serialization version.
      */
+    @java.io.Serial
     private static final long serialVersionUID = -1440403870442975015L;
 
     private static final long DAYS_0000_TO_1970 = (146097 * 5L) - (30L * 365L + 7L); // taken from LocalDate
@@ -669,7 +670,6 @@ public final class IsoChronology extends AbstractChronology implements Serializa
      * @param years  the number of years, may be negative
      * @param months  the number of years, may be negative
      * @param days  the number of years, may be negative
-     * @return the period in terms of this chronology, not null
      * @return the ISO period, not null
      */
     @Override  // override with covariant return type
@@ -680,7 +680,7 @@ public final class IsoChronology extends AbstractChronology implements Serializa
     //-----------------------------------------------------------------------
     /**
      * Writes the Chronology using a
-     * <a href="../../../serialized-form.html#java.time.chrono.Ser">dedicated serialized form</a>.
+     * <a href="{@docRoot}/serialized-form.html#java.time.chrono.Ser">dedicated serialized form</a>.
      * @serialData
      * <pre>
      *  out.writeByte(1);     // identifies a Chronology
@@ -690,6 +690,7 @@ public final class IsoChronology extends AbstractChronology implements Serializa
      * @return the instance of {@code Ser}, not null
      */
     @Override
+    @java.io.Serial
     Object writeReplace() {
         return super.writeReplace();
     }
@@ -700,6 +701,7 @@ public final class IsoChronology extends AbstractChronology implements Serializa
      * @param s the stream to read
      * @throws InvalidObjectException always
      */
+    @java.io.Serial
     private void readObject(ObjectInputStream s) throws InvalidObjectException {
         throw new InvalidObjectException("Deserialization via serialization delegate");
     }
