@@ -131,7 +131,7 @@ class ZoneName {
         "America/Argentina/Rio_Gallegos", "Argentina", "America/Buenos_Aires",
         "America/Argentina/Salta", "Argentina", "America/Buenos_Aires",
         "America/Argentina/San_Juan", "Argentina", "America/Buenos_Aires",
-        "America/Argentina/San_Luis", "Argentina_Western", "America/Argentina/San_Luis",
+        "America/Argentina/San_Luis", "Argentina", "America/Buenos_Aires",
         "America/Argentina/Tucuman", "Argentina", "America/Buenos_Aires",
         "America/Argentina/Ushuaia", "Argentina", "America/Buenos_Aires",
         "America/Aruba", "Atlantic", "America/Halifax",
@@ -310,7 +310,7 @@ class ZoneName {
         "Asia/Brunei", "Brunei", "Asia/Brunei",
         "Asia/Calcutta", "India", "Asia/Calcutta",
         "Asia/Chita", "Yakutsk", "Asia/Yakutsk",
-        "Asia/Choibalsan", "Choibalsan", "Asia/Choibalsan",
+        "Asia/Choibalsan", "Mongolia", "Asia/Ulaanbaatar",
         "Asia/Chongqing", "China", "Asia/Shanghai",
         "Asia/Chungking", "China", "Asia/Shanghai",
         "Asia/Colombo", "India", "Asia/Calcutta",
@@ -706,6 +706,7 @@ class ZoneName {
         "Europe_Central", "RS", "Europe/Belgrade",
         "Europe_Central", "SE", "Europe/Stockholm",
         "Europe_Central", "SI", "Europe/Ljubljana",
+        "Europe_Central", "SJ", "Arctic/Longyearbyen",
         "Europe_Central", "SK", "Europe/Bratislava",
         "Europe_Central", "SM", "Europe/San_Marino",
         "Europe_Central", "TN", "Africa/Tunis",
@@ -821,6 +822,7 @@ class ZoneName {
         "America/Dominica", "America/Port_of_Spain",
         "America/Ensenada", "America/Tijuana",
         "America/Fort_Wayne", "America/Indiana/Indianapolis",
+        "America/Godthab", "America/Nuuk",
         "America/Grenada", "America/Port_of_Spain",
         "America/Guadeloupe", "America/Port_of_Spain",
         "America/Indianapolis", "America/Indiana/Indianapolis",
@@ -977,7 +979,6 @@ class ZoneName {
         "US/Michigan", "America/Detroit",
         "US/Mountain", "America/Denver",
         "US/Pacific", "America/Los_Angeles",
-        "US/Pacific-New", "America/Los_Angeles",
         "US/Samoa", "Pacific/Pago_Pago",
         "UTC", "Etc/UTC",
         "Universal", "Etc/UTC",
