@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 1995, 2019, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 1995, 2020, Oracle and/or its affiliates. All rights reserved.
  * ORACLE PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
  *
  *
@@ -83,6 +83,7 @@ public class BitSet implements Cloneable, java.io.Serializable {
      * bit position i % 64 (where bit position 0 refers to the least
      * significant bit and 63 refers to the most significant bit).
      */
+    @java.io.Serial
     private static final ObjectStreamField[] serialPersistentFields = {
         new ObjectStreamField("bits", long[].class),
     };
@@ -104,6 +105,7 @@ public class BitSet implements Cloneable, java.io.Serializable {
     private transient boolean sizeIsSticky = false;
 
     /* use serialVersionUID from JDK 1.0.2 for interoperability */
+    @java.io.Serial
     private static final long serialVersionUID = 7997698588986878753L;
 
     /**
@@ -293,7 +295,7 @@ public class BitSet implements Cloneable, java.io.Serializable {
      * @return a byte array containing a little-endian representation
      *         of all the bits in this bit set
      * @since 1.7
-    */
+     */
     public byte[] toByteArray() {
         int n = wordsInUse;
         if (n == 0)
@@ -322,7 +324,7 @@ public class BitSet implements Cloneable, java.io.Serializable {
      * @return a long array containing a little-endian representation
      *         of all the bits in this bit set
      * @since 1.7
-    */
+     */
     public long[] toLongArray() {
         return Arrays.copyOf(words, wordsInUse);
     }
@@ -1052,7 +1054,7 @@ public class BitSet implements Cloneable, java.io.Serializable {
     /**
      * Compares this object against the specified object.
      * The result is {@code true} if and only if the argument is
-     * not {@code null} and is a {@code Bitset} object that has
+     * not {@code null} and is a {@code BitSet} object that has
      * exactly the same set of bits set to {@code true} as this bit
      * set. That is, for every nonnegative {@code int} index {@code k},
      * <pre>((BitSet)obj).get(k) == this.get(k)</pre>
@@ -1124,6 +1126,7 @@ public class BitSet implements Cloneable, java.io.Serializable {
      * Save the state of the {@code BitSet} instance to a stream (i.e.,
      * serialize it).
      */
+    @java.io.Serial
     private void writeObject(ObjectOutputStream s)
         throws IOException {
 
@@ -1141,6 +1144,7 @@ public class BitSet implements Cloneable, java.io.Serializable {
      * Reconstitute the {@code BitSet} instance from a stream (i.e.,
      * deserialize it).
      */
+    @java.io.Serial
     private void readObject(ObjectInputStream s)
         throws IOException, ClassNotFoundException {
 
