@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2016, 2019, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2016, 2020, Oracle and/or its affiliates. All rights reserved.
  * ORACLE PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
  *
  *
@@ -55,10 +55,64 @@ class ImmutableCollections {
      * it needs to vary sufficiently from one run to the next so that iteration order
      * will vary between JVM runs.
      */
-    static final int SALT;
+    private static final long SALT32L;
+
+    /**
+     * For set and map iteration, we will iterate in "reverse" stochastically,
+     * decided at bootstrap time.
+     */
+    private static final boolean REVERSE;
     static {
-        long nt = System.nanoTime();
-        SALT = (int)((nt >>> 32) ^ nt);
+        // to generate a reasonably random and well-mixed SALT, use an arbitrary
+        // value (a slice of pi), multiply with a random seed, then pick
+        // the mid 32-bits from the product. By picking a SALT value in the
+        // [0 ... 0xFFFF_FFFFL == 2^32-1] range, we ensure that for any positive
+        // int N, (SALT32L * N) >> 32 is a number in the [0 ... N-1] range. This
+        // property will be used to avoid more expensive modulo-based
+        // calculations.
+        long color = 0x243F_6A88_85A3_08D3L; // slice of pi
+
+        // When running with -Xshare:dump, the VM will supply a "random" seed that's
+        // derived from the JVM build/version, so can we generate the exact same
+        // CDS archive for the same JDK build. This makes it possible to verify the
+        // consistency of the JDK build.
+        long seed = VM.getRandomSeedForCDSDump();
+        if (seed == 0) {
+          seed = System.nanoTime();
+        }
+        SALT32L = (int)((color * seed) >> 16) & 0xFFFF_FFFFL;
+        // use the lowest bit to determine if we should reverse iteration
+        REVERSE = (SALT32L & 1) == 0;
+    }
+
+    /**
+     * Constants following this might be initialized from the CDS archive via
+     * this array.
+     */
+    private static Object[] archivedObjects;
+
+    private static final Object EMPTY;
+
+    static final ListN<?> EMPTY_LIST;
+
+    static final SetN<?> EMPTY_SET;
+
+    static final MapN<?,?> EMPTY_MAP;
+
+    static {
+        VM.initializeFromArchive(ImmutableCollections.class);
+        if (archivedObjects == null) {
+            EMPTY = new Object();
+            EMPTY_LIST = new ListN<>();
+            EMPTY_SET = new SetN<>();
+            EMPTY_MAP = new MapN<>();
+            archivedObjects = new Object[] { EMPTY, EMPTY_LIST, EMPTY_SET, EMPTY_MAP };
+        } else {
+            EMPTY = archivedObjects[0];
+            EMPTY_LIST = (ListN)archivedObjects[1];
+            EMPTY_SET = (SetN)archivedObjects[2];
+            EMPTY_MAP = (MapN)archivedObjects[3];
+        }
     }
 
     /** No instances. */
@@ -386,11 +440,13 @@ class ImmutableCollections {
         private final E e0;
 
         @Stable
-        private final E e1;
+        private final Object e1;
 
         List12(E e0) {
             this.e0 = Objects.requireNonNull(e0);
-            this.e1 = null;
+            // Use EMPTY as a sentinel for an unused element: not using null
+            // enable constant folding optimizations over single-element lists
+            this.e1 = EMPTY;
         }
 
         List12(E e0, E e1) {
@@ -400,7 +456,7 @@ class ImmutableCollections {
 
         @Override
         public int size() {
-            return e1 != null ? 2 : 1;
+            return e1 != EMPTY ? 2 : 1;
         }
 
         @Override
@@ -409,21 +465,24 @@ class ImmutableCollections {
         }
 
         @Override
+        @SuppressWarnings("unchecked")
         public E get(int index) {
             if (index == 0) {
                 return e0;
-            } else if (index == 1 && e1 != null) {
-                return e1;
+            } else if (index == 1 && e1 != EMPTY) {
+                return (E)e1;
             }
             throw outOfBounds(index);
         }
 
+        @java.io.Serial
         private void readObject(ObjectInputStream in) throws IOException, ClassNotFoundException {
             throw new InvalidObjectException("not serial proxy");
         }
 
+        @java.io.Serial
         private Object writeReplace() {
-            if (e1 == null) {
+            if (e1 == EMPTY) {
                 return new CollSer(CollSer.IMM_LIST, e0);
             } else {
                 return new CollSer(CollSer.IMM_LIST, e0, e1);
@@ -432,7 +491,7 @@ class ImmutableCollections {
 
         @Override
         public Object[] toArray() {
-            if (e1 == null) {
+            if (e1 == EMPTY) {
                 return new Object[] { e0 };
             } else {
                 return new Object[] { e0, e1 };
@@ -442,7 +501,7 @@ class ImmutableCollections {
         @Override
         @SuppressWarnings("unchecked")
         public <T> T[] toArray(T[] a) {
-            int size = e1 == null ? 1 : 2;
+            int size = size();
             T[] array = a.length >= size ? a :
                     (T[])Array.newInstance(a.getClass().getComponentType(), size);
             array[0] = (T)e0;
@@ -459,16 +518,6 @@ class ImmutableCollections {
     static final class ListN<E> extends AbstractImmutableList<E>
             implements Serializable {
 
-        // EMPTY_LIST may be initialized from the CDS archive.
-        static @Stable List<?> EMPTY_LIST;
-
-        static {
-            VM.initializeFromArchive(ListN.class);
-            if (EMPTY_LIST == null) {
-                EMPTY_LIST = new ListN<>();
-            }
-        }
-
         @Stable
         private final E[] elements;
 
@@ -498,10 +547,12 @@ class ImmutableCollections {
             return elements[index];
         }
 
+        @java.io.Serial
         private void readObject(ObjectInputStream in) throws IOException, ClassNotFoundException {
             throw new InvalidObjectException("not serial proxy");
         }
 
+        @java.io.Serial
         private Object writeReplace() {
             return new CollSer(CollSer.IMM_LIST, elements);
         }
@@ -560,13 +611,16 @@ class ImmutableCollections {
             implements Serializable {
 
         @Stable
-        final E e0;
+        private final E e0;
+
         @Stable
-        final E e1;
+        private final Object e1;
 
         Set12(E e0) {
             this.e0 = Objects.requireNonNull(e0);
-            this.e1 = null;
+            // Use EMPTY as a sentinel for an unused element: not using null
+            // enable constant folding optimizations over single-element sets
+            this.e1 = EMPTY;
         }
 
         Set12(E e0, E e1) {
@@ -580,7 +634,7 @@ class ImmutableCollections {
 
         @Override
         public int size() {
-            return (e1 == null) ? 1 : 2;
+            return (e1 == EMPTY) ? 1 : 2;
         }
 
         @Override
@@ -590,18 +644,18 @@ class ImmutableCollections {
 
         @Override
         public boolean contains(Object o) {
-            return o.equals(e0) || o.equals(e1); // implicit nullcheck of o
+            return o.equals(e0) || e1.equals(o); // implicit nullcheck of o
         }
 
         @Override
         public int hashCode() {
-            return e0.hashCode() + (e1 == null ? 0 : e1.hashCode());
+            return e0.hashCode() + (e1 == EMPTY ? 0 : e1.hashCode());
         }
 
         @Override
         public Iterator<E> iterator() {
             return new Iterator<>() {
-                private int idx = size();
+                private int idx = (e1 == EMPTY) ? 1 : 2;
 
                 @Override
                 public boolean hasNext() {
@@ -609,13 +663,14 @@ class ImmutableCollections {
                 }
 
                 @Override
+                @SuppressWarnings("unchecked")
                 public E next() {
                     if (idx == 1) {
                         idx = 0;
-                        return SALT >= 0 || e1 == null ? e0 : e1;
+                        return (REVERSE || e1 == EMPTY) ? e0 : (E)e1;
                     } else if (idx == 2) {
                         idx = 1;
-                        return SALT >= 0 ? e1 : e0;
+                        return REVERSE ? (E)e1 : e0;
                     } else {
                         throw new NoSuchElementException();
                     }
@@ -623,12 +678,14 @@ class ImmutableCollections {
             };
         }
 
+        @java.io.Serial
         private void readObject(ObjectInputStream in) throws IOException, ClassNotFoundException {
             throw new InvalidObjectException("not serial proxy");
         }
 
+        @java.io.Serial
         private Object writeReplace() {
-            if (e1 == null) {
+            if (e1 == EMPTY) {
                 return new CollSer(CollSer.IMM_SET, e0);
             } else {
                 return new CollSer(CollSer.IMM_SET, e0, e1);
@@ -637,9 +694,9 @@ class ImmutableCollections {
 
         @Override
         public Object[] toArray() {
-            if (e1 == null) {
+            if (e1 == EMPTY) {
                 return new Object[] { e0 };
-            } else if (SALT >= 0) {
+            } else if (REVERSE) {
                 return new Object[] { e1, e0 };
             } else {
                 return new Object[] { e0, e1 };
@@ -649,12 +706,12 @@ class ImmutableCollections {
         @Override
         @SuppressWarnings("unchecked")
         public <T> T[] toArray(T[] a) {
-            int size = e1 == null ? 1 : 2;
+            int size = size();
             T[] array = a.length >= size ? a :
                     (T[])Array.newInstance(a.getClass().getComponentType(), size);
             if (size == 1) {
                 array[0] = (T)e0;
-            } else if (SALT >= 0) {
+            } else if (REVERSE) {
                 array[0] = (T)e1;
                 array[1] = (T)e0;
             } else {
@@ -678,18 +735,9 @@ class ImmutableCollections {
     static final class SetN<E> extends AbstractImmutableSet<E>
             implements Serializable {
 
-        // EMPTY_SET may be initialized from the CDS archive.
-        static @Stable Set<?> EMPTY_SET;
-
-        static {
-            VM.initializeFromArchive(SetN.class);
-            if (EMPTY_SET == null) {
-                EMPTY_SET = new SetN<>();
-            }
-        }
-
         @Stable
         final E[] elements;
+
         @Stable
         final int size;
 
@@ -733,10 +781,10 @@ class ImmutableCollections {
             private int idx;
 
             SetNIterator() {
-                remaining = size();
-                if (remaining > 0) {
-                    idx = Math.floorMod(SALT, elements.length);
-                }
+                remaining = size;
+                // pick a starting index in the [0 .. element.length-1] range
+                // randomly based on SALT32L
+                idx = (int) ((SALT32L * elements.length) >>> 32);
             }
 
             @Override
@@ -744,26 +792,25 @@ class ImmutableCollections {
                 return remaining > 0;
             }
 
-            private int nextIndex() {
-                int idx = this.idx;
-                if (SALT >= 0) {
-                    if (++idx >= elements.length) {
-                        idx = 0;
-                    }
-                } else {
-                    if (--idx < 0) {
-                        idx = elements.length - 1;
-                    }
-                }
-                return this.idx = idx;
-            }
-
             @Override
             public E next() {
                 if (remaining > 0) {
                     E element;
-                    // skip null elements
-                    while ((element = elements[nextIndex()]) == null) {}
+                    int idx = this.idx;
+                    int len = elements.length;
+                    // step to the next element; skip null elements
+                    do {
+                        if (REVERSE) {
+                            if (++idx >= len) {
+                                idx = 0;
+                            }
+                        } else {
+                            if (--idx < 0) {
+                                idx = len - 1;
+                            }
+                        }
+                    } while ((element = elements[idx]) == null);
+                    this.idx = idx;
                     remaining--;
                     return element;
                 } else {
@@ -806,10 +853,12 @@ class ImmutableCollections {
             }
         }
 
+        @java.io.Serial
         private void readObject(ObjectInputStream in) throws IOException, ClassNotFoundException {
             throw new InvalidObjectException("not serial proxy");
         }
 
+        @java.io.Serial
         private Object writeReplace() {
             Object[] array = new Object[size];
             int dest = 0;
@@ -863,6 +912,20 @@ class ImmutableCollections {
         @Override public V replace(K key, V value) { throw uoe(); }
         @Override public boolean replace(K key, V oldValue, V newValue) { throw uoe(); }
         @Override public void replaceAll(BiFunction<? super K,? super V,? extends V> f) { throw uoe(); }
+
+        /**
+         * @implNote {@code null} values are disallowed in these immutable maps,
+         * so we can improve upon the default implementation since a
+         * {@code null} return from {@code get(key)} always means the default
+         * value should be returned.
+         */
+        @Override
+        public V getOrDefault(Object key, V defaultValue) {
+            V v;
+            return ((v = get(key)) != null)
+                    ? v
+                    : defaultValue;
+        }
     }
 
     static final class Map1<K,V> extends AbstractImmutableMap<K,V> {
@@ -906,10 +969,12 @@ class ImmutableCollections {
             return false;
         }
 
+        @java.io.Serial
         private void readObject(ObjectInputStream in) throws IOException, ClassNotFoundException {
             throw new InvalidObjectException("not serial proxy");
         }
 
+        @java.io.Serial
         private Object writeReplace() {
             return new CollSer(CollSer.IMM_MAP, k0, v0);
         }
@@ -931,16 +996,6 @@ class ImmutableCollections {
      */
     static final class MapN<K,V> extends AbstractImmutableMap<K,V> {
 
-        // EMPTY_MAP may be initialized from the CDS archive.
-        static @Stable Map<?,?> EMPTY_MAP;
-
-        static {
-            VM.initializeFromArchive(MapN.class);
-            if (EMPTY_MAP == null) {
-                EMPTY_MAP = new MapN<>();
-            }
-        }
-
         @Stable
         final Object[] table; // pairs of key, value
 
@@ -1035,10 +1090,10 @@ class ImmutableCollections {
             private int idx;
 
             MapNIterator() {
-                remaining = size();
-                if (remaining > 0) {
-                    idx = Math.floorMod(SALT, table.length >> 1) << 1;
-                }
+                remaining = size;
+                // pick an even starting index in the [0 .. table.length-1]
+                // range randomly based on SALT32L
+                idx = (int) ((SALT32L * (table.length >> 1)) >>> 32) << 1;
             }
 
             @Override
@@ -1048,7 +1103,7 @@ class ImmutableCollections {
 
             private int nextIndex() {
                 int idx = this.idx;
-                if (SALT >= 0) {
+                if (REVERSE) {
                     if ((idx += 2) >= table.length) {
                         idx = 0;
                     }
@@ -1110,10 +1165,12 @@ class ImmutableCollections {
             }
         }
 
+        @java.io.Serial
         private void readObject(ObjectInputStream in) throws IOException, ClassNotFoundException {
             throw new InvalidObjectException("not serial proxy");
         }
 
+        @java.io.Serial
         private Object writeReplace() {
             Object[] array = new Object[2 * size];
             int len = table.length;
@@ -1138,6 +1195,7 @@ class ImmutableCollections {
  * @since 9
  */
 final class CollSer implements Serializable {
+    @java.io.Serial
     private static final long serialVersionUID = 6309168927139932177L;
 
     static final int IMM_LIST = 1;
@@ -1198,6 +1256,7 @@ final class CollSer implements Serializable {
      * @throws InvalidObjectException if the count is negative
      * @since 9
      */
+    @java.io.Serial
     private void readObject(ObjectInputStream ois) throws IOException, ClassNotFoundException {
         ois.defaultReadObject();
         int len = ois.readInt();
@@ -1227,6 +1286,7 @@ final class CollSer implements Serializable {
      * @throws IOException if an I/O error occurs
      * @since 9
      */
+    @java.io.Serial
     private void writeObject(ObjectOutputStream oos) throws IOException {
         oos.defaultWriteObject();
         oos.writeInt(array.length);
@@ -1253,6 +1313,7 @@ final class CollSer implements Serializable {
      * @throws ObjectStreamException if another serialization error has occurred
      * @since 9
      */
+   @java.io.Serial
     private Object readResolve() throws ObjectStreamException {
         try {
             if (array == null) {
@@ -1268,7 +1329,7 @@ final class CollSer implements Serializable {
                     return Set.of(array);
                 case IMM_MAP:
                     if (array.length == 0) {
-                        return ImmutableCollections.MapN.EMPTY_MAP;
+                        return ImmutableCollections.EMPTY_MAP;
                     } else if (array.length == 2) {
                         return new ImmutableCollections.Map1<>(array[0], array[1]);
                     } else {
