@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 1997, 2015, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 1997, 2018, Oracle and/or its affiliates. All rights reserved.
  * ORACLE PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
  *
  *
@@ -24,15 +24,39 @@
  */
 package javax.swing;
 
-import java.awt.*;
-import java.awt.event.*;
+import java.awt.Component;
+import java.awt.Color;
+import java.awt.Point;
+import java.awt.Dimension;
+import java.awt.Rectangle;
+import java.awt.Font;
+import java.awt.FontMetrics;
+import java.awt.Cursor;
+
+import java.awt.event.MouseEvent;
+import java.awt.event.FocusListener;
+
 import java.beans.JavaBean;
 import java.beans.BeanProperty;
 import java.beans.Transient;
-import java.util.*;
-import javax.swing.event.*;
-import javax.swing.plaf.*;
-import javax.accessibility.*;
+
+import java.util.Locale;
+import java.util.ArrayList;
+
+import javax.swing.event.ChangeListener;
+import javax.swing.event.ChangeEvent;
+
+import javax.swing.plaf.TabbedPaneUI;
+import javax.swing.plaf.UIResource;
+
+import javax.accessibility.AccessibleContext;
+import javax.accessibility.Accessible;
+import javax.accessibility.AccessibleRole;
+import javax.accessibility.AccessibleComponent;
+import javax.accessibility.AccessibleStateSet;
+import javax.accessibility.AccessibleIcon;
+import javax.accessibility.AccessibleSelection;
+import javax.accessibility.AccessibleState;
 
 import sun.swing.SwingUtilities2;
 
@@ -45,7 +69,7 @@ import java.io.IOException;
  * A component that lets the user switch between a group of components by
  * clicking on a tab with a given title and/or icon.
  * For examples and information on using tabbed panes see
- * <a href="http://docs.oracle.com/javase/tutorial/uiswing/components/tabbedpane.html">How to Use Tabbed Panes</a>,
+ * <a href="https://docs.oracle.com/javase/tutorial/uiswing/components/tabbedpane.html">How to Use Tabbed Panes</a>,
  * a section in <em>The Java Tutorial</em>.
  * <p>
  * Tabs/components are added to a <code>TabbedPane</code> object by using the
@@ -93,7 +117,7 @@ import java.io.IOException;
  * future Swing releases. The current serialization support is
  * appropriate for short term storage or RMI between applications running
  * the same version of Swing.  As of 1.4, support for long term storage
- * of all JavaBeans&trade;
+ * of all JavaBeans
  * has been added to the <code>java.beans</code> package.
  * Please see {@link java.beans.XMLEncoder}.
  *
@@ -923,6 +947,13 @@ public class JTabbedPane extends JComponent
         }
     }
 
+    private void clearAccessibleParent(Component c) {
+        AccessibleContext ac = c.getAccessibleContext();
+        if (ac != null) {
+            ac.setAccessibleParent(null);
+        }
+    }
+
     /**
      * Removes the tab at <code>index</code>.
      * After the component associated with <code>index</code> is removed,
@@ -1000,11 +1031,12 @@ public class JTabbedPane extends JComponent
         // container's children array indices, so make sure we
         // remove the correct child!
         if (component != null) {
-            Component components[] = getComponents();
+            Component[] components = getComponents();
             for (int i = components.length; --i >= 0; ) {
                 if (components[i] == component) {
                     super.remove(i);
                     component.setVisible(true);
+                    clearAccessibleParent(component);
                     break;
                 }
             }
@@ -1034,7 +1066,7 @@ public class JTabbedPane extends JComponent
         } else {
             // Container#remove(comp) invokes Container#remove(int)
             // so make sure JTabbedPane#remove(int) isn't called here
-            Component children[] = getComponents();
+            Component[] children = getComponents();
             for (int i=0; i < children.length; i++) {
                 if (component == children[i]) {
                     super.remove(i);
@@ -1548,10 +1580,11 @@ public class JTabbedPane extends JComponent
                 // why not if (page.component.getParent() == this) remove(component)
                 synchronized(getTreeLock()) {
                     int count = getComponentCount();
-                    Component children[] = getComponents();
+                    Component[] children = getComponents();
                     for (int i = 0; i < count; i++) {
                         if (children[i] == page.component) {
                             super.remove(i);
+                            clearAccessibleParent(children[i]);
                         }
                     }
                 }
@@ -1888,7 +1921,7 @@ public class JTabbedPane extends JComponent
      * future Swing releases. The current serialization support is
      * appropriate for short term storage or RMI between applications running
      * the same version of Swing.  As of 1.4, support for long term storage
-     * of all JavaBeans&trade;
+     * of all JavaBeans
      * has been added to the <code>java.beans</code> package.
      * Please see {@link java.beans.XMLEncoder}.
      */
