@@ -55,7 +55,7 @@ import sun.swing.UIAction;
  * <p>
  * For adding specific actions,
  * overide <code>installKeyboardActions</code> to add actions in response to
- * KeyStroke bindings. See the article <a href="http://docs.oracle.com/javase/tutorial/uiswing/misc/keybinding.html">How to Use Key Bindings</a>
+ * KeyStroke bindings. See the article <a href="https://docs.oracle.com/javase/tutorial/uiswing/misc/keybinding.html">How to Use Key Bindings</a>
  *
  * @author Arnaud Weber
  * @author Tom Santos
@@ -1709,8 +1709,20 @@ public class BasicComboBoxUI extends ComboBoxUI {
 
         @Override
         public boolean accept(Object c) {
-            if (getName() == HIDE) {
+            if (getName() == HIDE ) {
                 return (c != null && ((JComboBox)c).isPopupVisible());
+            } else if (getName() == ENTER) {
+                JRootPane root = SwingUtilities.getRootPane((JComboBox)c);
+                if (root != null && (c != null && !((JComboBox)c).isPopupVisible())) {
+                    InputMap im = root.getInputMap(JComponent.WHEN_IN_FOCUSED_WINDOW);
+                    ActionMap am = root.getActionMap();
+                    if (im != null && am != null) {
+                        Object obj = im.get(KeyStroke.getKeyStroke(KeyEvent.VK_ENTER, 0));
+                        if (obj == null) {
+                            return false;
+                        }
+                    }
+                }
             }
             return true;
         }
