@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2016, 2018, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2016, 2020, Oracle and/or its affiliates. All rights reserved.
  * ORACLE PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
  *
  *
@@ -21,18 +21,19 @@
  *
  */
 
+
+
 package jdk.tools.jaotc;
 
 import java.util.ArrayList;
 
+import org.graalvm.compiler.code.CompilationResult;
+import org.graalvm.compiler.hotspot.HotSpotForeignCallLinkage;
+import org.graalvm.compiler.options.OptionValues;
+
 import jdk.tools.jaotc.binformat.BinaryContainer;
 import jdk.tools.jaotc.binformat.CodeContainer;
 import jdk.tools.jaotc.binformat.Symbol;
-import jdk.tools.jaotc.StubInformation;
-import org.graalvm.compiler.code.CompilationResult;
-import org.graalvm.compiler.hotspot.HotSpotForeignCallLinkage;
-
-import jdk.vm.ci.aarch64.AArch64;
 import jdk.vm.ci.amd64.AMD64;
 import jdk.vm.ci.code.TargetDescription;
 import jdk.vm.ci.code.site.Call;
@@ -42,6 +43,8 @@ import jdk.vm.ci.meta.ResolvedJavaMethod;
 
 final class CodeSectionProcessor {
 
+    private final OptionValues optionValues;
+
     private final TargetDescription target;
 
     private final BinaryContainer binaryContainer;
@@ -49,6 +52,7 @@ final class CodeSectionProcessor {
     CodeSectionProcessor(DataBuilder dataBuilder) {
         this.target = dataBuilder.getBackend().getTarget();
         this.binaryContainer = dataBuilder.getBinaryContainer();
+        this.optionValues = dataBuilder.getBackend().getRuntime().getOptions();
     }
 
     /**
@@ -75,7 +79,7 @@ final class CodeSectionProcessor {
                 if (infopoint.reason == InfopointReason.CALL) {
                     final Call callInfopoint = (Call) infopoint;
                     if (callInfopoint.target instanceof HotSpotForeignCallLinkage &&
-                        target.arch instanceof AMD64) {
+                                    target.arch instanceof AMD64) {
                         // TODO 4 is x86 size of relative displacement.
                         // For SPARC need something different.
                         int destOffset = infopoint.pcOffset + callInfopoint.size - 4;
@@ -131,7 +135,7 @@ final class CodeSectionProcessor {
     private StubInformation addCallStub(boolean isVirtualCall) {
         final int startOffset = binaryContainer.getCodeContainer().getByteStreamSize();
         StubInformation stub = new StubInformation(startOffset, isVirtualCall);
-        ELFMacroAssembler masm = ELFMacroAssembler.getELFMacroAssembler(target);
+        ELFMacroAssembler masm = ELFMacroAssembler.getELFMacroAssembler(target, optionValues);
         byte[] code;
         if (isVirtualCall) {
             code = masm.getPLTVirtualEntryCode(stub);
