@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2016, 2018, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2016, 2020, Oracle and/or its affiliates. All rights reserved.
  * ORACLE PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
  *
  *
@@ -35,13 +35,14 @@ import org.graalvm.compiler.asm.amd64.AMD64Address;
 import org.graalvm.compiler.asm.amd64.AMD64MacroAssembler;
 
 import jdk.vm.ci.code.TargetDescription;
+import org.graalvm.compiler.options.OptionValues;
 
 public final class AMD64ELFMacroAssembler extends AMD64MacroAssembler implements ELFMacroAssembler {
 
     private int currentEndOfInstruction;
 
-    public AMD64ELFMacroAssembler(TargetDescription target) {
-        super(target);
+    public AMD64ELFMacroAssembler(TargetDescription target, OptionValues optionValues) {
+        super(target, optionValues);
     }
 
     @Override
