@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2018, 2021, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2018, 2020, Oracle and/or its affiliates. All rights reserved.
  * ORACLE PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
  *
  *
@@ -49,15 +49,21 @@ import java.lang.reflect.Method;
 
 public interface Metrics {
 
-    public static final long LONG_RETVAL_UNLIMITED = -1;
-
     /**
      * Returns an instance of the Metrics class.
      *
      * @return Metrics object or null if not supported on this platform.
      */
     public static Metrics systemMetrics() {
-        return SystemMetrics.instance();
+        try {
+            Class<?> c = Class.forName("jdk.internal.platform.CgroupMetrics");
+            Method m = c.getMethod("getInstance");
+            return (Metrics) m.invoke(null);
+        } catch (ClassNotFoundException e) {
+            return null;
+        } catch (ReflectiveOperationException e) {
+            throw new InternalError(e);
+        }
     }
 
     /**
