@@ -28,7 +28,6 @@ package jdk.internal.platform.cgroupv1;
 public class CgroupV1MemorySubSystemController extends CgroupV1SubsystemController {
 
     private boolean hierarchical;
-    private boolean swapenabled;
 
     public CgroupV1MemorySubSystemController(String root, String mountPoint) {
         super(root, mountPoint);
@@ -42,11 +41,4 @@ public class CgroupV1MemorySubSystemController extends CgroupV1SubsystemControll
         this.hierarchical = hierarchical;
     }
 
-    boolean isSwapEnabled() {
-        return swapenabled;
-    }
-
-    void setSwapEnabled(boolean swapenabled) {
-        this.swapenabled = swapenabled;
-    }
 }
\ No newline at end of file
