@@ -1,6 +1,5 @@
 /*
- * Copyright (c) 2018, 2021, Oracle and/or its affiliates. All rights reserved.
- * Copyright (c) 2022, Red Hat Inc.
+ * Copyright (c) 2018, 2020, Oracle and/or its affiliates. All rights reserved.
  * ORACLE PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
  *
  *
@@ -26,11 +25,14 @@
 
 package jdk.internal.platform.cgroupv1;
 
-import java.util.Map;
+import java.io.IOException;
+import java.nio.file.Path;
+import java.nio.file.Paths;
+import java.util.stream.Stream;
 
-import jdk.internal.platform.CgroupInfo;
 import jdk.internal.platform.CgroupSubsystem;
 import jdk.internal.platform.CgroupSubsystemController;
+import jdk.internal.platform.CgroupUtil;
 import jdk.internal.platform.CgroupV1Metrics;
 
 public class CgroupV1Subsystem implements CgroupSubsystem, CgroupV1Metrics {
@@ -39,122 +41,164 @@ public class CgroupV1Subsystem implements CgroupSubsystem, CgroupV1Metrics {
     private CgroupV1SubsystemController cpuacct;
     private CgroupV1SubsystemController cpuset;
     private CgroupV1SubsystemController blkio;
-    private CgroupV1SubsystemController pids;
+    private boolean activeSubSystems;
 
-    private static volatile CgroupV1Subsystem INSTANCE;
+    private static final CgroupV1Subsystem INSTANCE = initSubSystem();
 
     private static final String PROVIDER_NAME = "cgroupv1";
 
-    private CgroupV1Subsystem() {}
+    private CgroupV1Subsystem() {
+        activeSubSystems = false;
+    }
 
-    /**
-     * Get a singleton instance of CgroupV1Subsystem. Initially, it creates a new
-     * object by retrieving the pre-parsed information from cgroup interface
-     * files from the provided 'infos' map.
-     *
-     * See CgroupSubsystemFactory.determineType() where the actual parsing of
-     * cgroup interface files happens.
-     *
-     * @return A singleton CgroupV1Subsystem instance, never null
-     */
-    public static CgroupV1Subsystem getInstance(Map<String, CgroupInfo> infos) {
-        if (INSTANCE == null) {
-            CgroupV1Subsystem tmpSubsystem = initSubSystem(infos);
-            synchronized (CgroupV1Subsystem.class) {
-                if (INSTANCE == null) {
-                    INSTANCE = tmpSubsystem;
-                }
-            }
-        }
+    public static CgroupV1Subsystem getInstance() {
         return INSTANCE;
     }
 
-    private static CgroupV1Subsystem initSubSystem(Map<String, CgroupInfo> infos) {
+    private static CgroupV1Subsystem initSubSystem() {
         CgroupV1Subsystem subsystem = new CgroupV1Subsystem();
 
-        boolean anyActiveControllers = false;
-        /*
-         * Find the cgroup mount points for subsystem controllers
-         * by looking up relevant data in the infos map
+        /**
+         * Find the cgroup mount points for subsystems
+         * by reading /proc/self/mountinfo
+         *
+         * Example for docker MemorySubSystem subsystem:
+         * 219 214 0:29 /docker/7208cebd00fa5f2e342b1094f7bed87fa25661471a4637118e65f1c995be8a34 /sys/fs/cgroup/MemorySubSystem ro,nosuid,nodev,noexec,relatime - cgroup cgroup rw,MemorySubSystem
+         *
+         * Example for host:
+         * 34 28 0:29 / /sys/fs/cgroup/MemorySubSystem rw,nosuid,nodev,noexec,relatime shared:16 - cgroup cgroup rw,MemorySubSystem
          */
-        for (CgroupInfo info: infos.values()) {
-            switch (info.getName()) {
-            case "memory": {
-                if (info.getMountRoot() != null && info.getMountPoint() != null) {
-                    CgroupV1MemorySubSystemController controller = new CgroupV1MemorySubSystemController(info.getMountRoot(), info.getMountPoint());
-                    controller.setPath(info.getCgroupPath());
-                    boolean isHierarchial = getHierarchical(controller);
-                    controller.setHierarchical(isHierarchial);
-                    boolean isSwapEnabled = getSwapEnabled(controller);
-                    controller.setSwapEnabled(isSwapEnabled);
-                    subsystem.setMemorySubSystem(controller);
-                    anyActiveControllers = true;
-                }
-                break;
-            }
-            case "cpuset": {
-                if (info.getMountRoot() != null && info.getMountPoint() != null) {
-                    CgroupV1SubsystemController controller = new CgroupV1SubsystemController(info.getMountRoot(), info.getMountPoint());
-                    controller.setPath(info.getCgroupPath());
-                    subsystem.setCpuSetController(controller);
-                    anyActiveControllers = true;
-                }
-                break;
-            }
-            case "cpuacct": {
-                if (info.getMountRoot() != null && info.getMountPoint() != null) {
-                    CgroupV1SubsystemController controller = new CgroupV1SubsystemController(info.getMountRoot(), info.getMountPoint());
-                    controller.setPath(info.getCgroupPath());
-                    subsystem.setCpuAcctController(controller);
-                    anyActiveControllers = true;
-                }
-                break;
-            }
-            case "cpu": {
-                if (info.getMountRoot() != null && info.getMountPoint() != null) {
-                    CgroupV1SubsystemController controller = new CgroupV1SubsystemController(info.getMountRoot(), info.getMountPoint());
-                    controller.setPath(info.getCgroupPath());
-                    subsystem.setCpuController(controller);
-                    anyActiveControllers = true;
-                }
-                break;
-            }
-            case "blkio": {
-                if (info.getMountRoot() != null && info.getMountPoint() != null) {
-                    CgroupV1SubsystemController controller = new CgroupV1SubsystemController(info.getMountRoot(), info.getMountPoint());
-                    controller.setPath(info.getCgroupPath());
-                    subsystem.setBlkIOController(controller);
-                    anyActiveControllers = true;
-                }
-                break;
-            }
-            case "pids": {
-                if (info.getMountRoot() != null && info.getMountPoint() != null) {
-                    CgroupV1SubsystemController controller = new CgroupV1SubsystemController(info.getMountRoot(), info.getMountPoint());
-                    controller.setPath(info.getCgroupPath());
-                    subsystem.setPidsController(controller);
-                    anyActiveControllers = true;
-                }
-                break;
-            }
-            default:
-                throw new AssertionError("Unrecognized controller in infos: " + info.getName());
-            }
+        try (Stream<String> lines =
+                CgroupUtil.readFilePrivileged(Paths.get("/proc/self/mountinfo"))) {
+
+            lines.filter(line -> line.contains(" - cgroup "))
+                 .map(line -> line.split(" "))
+                 .forEach(entry -> createSubSystemController(subsystem, entry));
+
+        } catch (IOException e) {
+            return null;
+        }
+
+        /**
+         * Read /proc/self/cgroup and map host mount point to
+         * local one via /proc/self/mountinfo content above
+         *
+         * Docker example:
+         * 5:memory:/docker/6558aed8fc662b194323ceab5b964f69cf36b3e8af877a14b80256e93aecb044
+         *
+         * Host example:
+         * 5:memory:/user.slice
+         *
+         * Construct a path to the process specific memory and cpuset
+         * cgroup directory.
+         *
+         * For a container running under Docker from memory example above
+         * the paths would be:
+         *
+         * /sys/fs/cgroup/memory
+         *
+         * For a Host from memory example above the path would be:
+         *
+         * /sys/fs/cgroup/memory/user.slice
+         *
+         */
+        try (Stream<String> lines =
+                CgroupUtil.readFilePrivileged(Paths.get("/proc/self/cgroup"))) {
+
+            lines.map(line -> line.split(":"))
+                 .filter(line -> (line.length >= 3))
+                 .forEach(line -> setSubSystemControllerPath(subsystem, line));
+
+        } catch (IOException e) {
+            return null;
         }
 
         // Return Metrics object if we found any subsystems.
-        if (anyActiveControllers) {
+        if (subsystem.activeSubSystems()) {
             return subsystem;
         }
 
         return null;
     }
 
-    private static boolean getSwapEnabled(CgroupV1MemorySubSystemController controller) {
-        long memswBytes = getLongValue(controller, "memory.memsw.limit_in_bytes");
-        long swappiness = getLongValue(controller, "memory.swappiness");
-        return (memswBytes > 0 && swappiness > 0);
-     }
+    /**
+     * createSubSystem objects and initialize mount points
+     */
+    private static void createSubSystemController(CgroupV1Subsystem subsystem, String[] mountentry) {
+        if (mountentry.length < 5) return;
+
+        Path p = Paths.get(mountentry[4]);
+        String[] subsystemNames = p.getFileName().toString().split(",");
+
+        for (String subsystemName: subsystemNames) {
+            switch (subsystemName) {
+                case "memory":
+                    subsystem.setMemorySubSystem(new CgroupV1MemorySubSystemController(mountentry[3], mountentry[4]));
+                    break;
+                case "cpuset":
+                    subsystem.setCpuSetController(new CgroupV1SubsystemController(mountentry[3], mountentry[4]));
+                    break;
+                case "cpuacct":
+                    subsystem.setCpuAcctController(new CgroupV1SubsystemController(mountentry[3], mountentry[4]));
+                    break;
+                case "cpu":
+                    subsystem.setCpuController(new CgroupV1SubsystemController(mountentry[3], mountentry[4]));
+                    break;
+                case "blkio":
+                    subsystem.setBlkIOController(new CgroupV1SubsystemController(mountentry[3], mountentry[4]));
+                    break;
+                default:
+                    // Ignore subsystems that we don't support
+                    break;
+            }
+        }
+    }
+
+    /**
+     * setSubSystemPath based on the contents of /proc/self/cgroup
+     */
+    private static void setSubSystemControllerPath(CgroupV1Subsystem subsystem, String[] entry) {
+        String controllerName = entry[1];
+        String base = entry[2];
+
+        if (controllerName != null && base != null) {
+            for (String cName: controllerName.split(",")) {
+                switch (cName) {
+                    case "memory":
+                        setPath(subsystem, subsystem.memoryController(), base);
+                        break;
+                    case "cpuset":
+                        setPath(subsystem, subsystem.cpuSetController(), base);
+                        break;
+                    case "cpu":
+                        setPath(subsystem, subsystem.cpuController(), base);
+                        break;
+                    case "cpuacct":
+                        setPath(subsystem, subsystem.cpuAcctController(), base);
+                        break;
+                    case "blkio":
+                        setPath(subsystem, subsystem.blkIOController(), base);
+                        break;
+                    // Ignore subsystems that we don't support
+                    default:
+                        break;
+                }
+            }
+        }
+
+    }
+
+    private static void setPath(CgroupV1Subsystem subsystem, CgroupV1SubsystemController controller, String base) {
+        if (controller != null) {
+            controller.setPath(base);
+            if (controller instanceof CgroupV1MemorySubSystemController) {
+                CgroupV1MemorySubSystemController memorySubSystem = (CgroupV1MemorySubSystemController)controller;
+                boolean isHierarchial = getHierarchical(memorySubSystem);
+                memorySubSystem.setHierarchical(isHierarchial);
+            }
+            subsystem.setActiveSubSystems();
+        }
+    }
 
 
     private static boolean getHierarchical(CgroupV1MemorySubSystemController controller) {
@@ -162,6 +206,14 @@ public class CgroupV1Subsystem implements CgroupSubsystem, CgroupV1Metrics {
         return hierarchical > 0;
     }
 
+    private void setActiveSubSystems() {
+        activeSubSystems = true;
+    }
+
+    private boolean activeSubSystems() {
+        return activeSubSystems;
+    }
+
     private void setMemorySubSystem(CgroupV1MemorySubSystemController memory) {
         this.memory = memory;
     }
@@ -182,8 +234,24 @@ public class CgroupV1Subsystem implements CgroupSubsystem, CgroupV1Metrics {
         this.blkio = blkio;
     }
 
-    private void setPidsController(CgroupV1SubsystemController pids) {
-        this.pids = pids;
+    private CgroupV1SubsystemController memoryController() {
+        return memory;
+    }
+
+    private CgroupV1SubsystemController cpuController() {
+        return cpu;
+    }
+
+    private CgroupV1SubsystemController cpuAcctController() {
+        return cpuacct;
+    }
+
+    private CgroupV1SubsystemController cpuSetController() {
+        return cpuset;
+    }
+
+    private CgroupV1SubsystemController blkIOController() {
+        return blkio;
     }
 
     private static long getLongValue(CgroupSubsystemController controller,
@@ -334,6 +402,10 @@ public class CgroupV1Subsystem implements CgroupSubsystem, CgroupV1Metrics {
         return getLongValue(memory, "memory.kmem.failcnt");
     }
 
+    public long getKernelMemoryLimit() {
+        return CgroupV1SubsystemController.longValOrUnlimited(getLongValue(memory, "memory.kmem.limit_in_bytes"));
+    }
+
     public long getKernelMemoryMaxUsage() {
         return getLongValue(memory, "memory.kmem.max_usage_in_bytes");
     }
@@ -346,6 +418,10 @@ public class CgroupV1Subsystem implements CgroupSubsystem, CgroupV1Metrics {
         return getLongValue(memory, "memory.kmem.tcp.failcnt");
     }
 
+    public long getTcpMemoryLimit() {
+        return CgroupV1SubsystemController.longValOrUnlimited(getLongValue(memory, "memory.kmem.tcp.limit_in_bytes"));
+    }
+
     public long getTcpMemoryMaxUsage() {
         return getLongValue(memory, "memory.kmem.tcp.max_usage_in_bytes");
     }
@@ -355,16 +431,10 @@ public class CgroupV1Subsystem implements CgroupSubsystem, CgroupV1Metrics {
     }
 
     public long getMemoryAndSwapFailCount() {
-        if (memory != null && !memory.isSwapEnabled()) {
-            return getMemoryFailCount();
-        }
         return getLongValue(memory, "memory.memsw.failcnt");
     }
 
     public long getMemoryAndSwapLimit() {
-        if (memory != null && !memory.isSwapEnabled()) {
-            return getMemoryLimit();
-        }
         long retval = getLongValue(memory, "memory.memsw.limit_in_bytes");
         if (retval > CgroupV1SubsystemController.UNLIMITED_MIN) {
             if (memory.isHierarchical()) {
@@ -380,16 +450,10 @@ public class CgroupV1Subsystem implements CgroupSubsystem, CgroupV1Metrics {
     }
 
     public long getMemoryAndSwapMaxUsage() {
-        if (memory != null && !memory.isSwapEnabled()) {
-            return getMemoryMaxUsage();
-        }
         return getLongValue(memory, "memory.memsw.max_usage_in_bytes");
     }
 
     public long getMemoryAndSwapUsage() {
-        if (memory != null && !memory.isSwapEnabled()) {
-            return getMemoryUsage();
-        }
         return getLongValue(memory, "memory.memsw.usage_in_bytes");
     }
 
@@ -402,17 +466,6 @@ public class CgroupV1Subsystem implements CgroupSubsystem, CgroupV1Metrics {
         return CgroupV1SubsystemController.longValOrUnlimited(getLongValue(memory, "memory.soft_limit_in_bytes"));
     }
 
-    /*****************************************************************
-     *  pids subsystem
-     ****************************************************************/
-    public long getPidsMax() {
-        String pidsMaxStr = CgroupSubsystemController.getStringValue(pids, "pids.max");
-        return CgroupSubsystem.limitFromString(pidsMaxStr);
-    }
-
-    public long getPidsCurrent() {
-        return getLongValue(pids, "pids.current");
-    }
 
     /*****************************************************************
      * BlKIO Subsystem
