@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 1997, 2019, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 1997, 2020, Oracle and/or its affiliates. All rights reserved.
  * ORACLE PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
  *
  *
@@ -35,6 +35,7 @@ import javax.lang.model.element.TypeElement;
 import javax.lang.model.element.VariableElement;
 import javax.lang.model.type.ArrayType;
 import javax.lang.model.type.DeclaredType;
+import javax.lang.model.type.ExecutableType;
 import javax.lang.model.type.TypeMirror;
 import javax.lang.model.type.TypeVariable;
 import javax.lang.model.util.SimpleTypeVisitor9;
@@ -58,10 +59,6 @@ import static jdk.javadoc.internal.doclets.formats.html.LinkInfoImpl.Kind.RECEIV
  *  If you write code that depends on this, you do so at your own risk.
  *  This code and its internal interfaces are subject to change or
  *  deletion without notice.</b>
- *
- * @author Robert Field
- * @author Atul M Dambalkar
- * @author Bhavesh Patel (Modified)
  */
 public abstract class AbstractExecutableMemberWriter extends AbstractMemberWriter {
 
@@ -85,9 +82,6 @@ public abstract class AbstractExecutableMemberWriter extends AbstractMemberWrite
         return writer.getTypeParameterLinks(linkInfo);
     }
 
-    /**
-     * {@inheritDoc}
-     */
     @Override
     protected Content getDeprecatedLink(Element member) {
         Content deprecatedLinkContent = new ContentBuilder();
@@ -96,7 +90,7 @@ public abstract class AbstractExecutableMemberWriter extends AbstractMemberWrite
             deprecatedLinkContent.add(".");
             deprecatedLinkContent.add(member.getSimpleName());
         }
-        String signature = utils.flatSignature((ExecutableElement) member);
+        String signature = utils.flatSignature((ExecutableElement) member, typeElement);
         if (signature.length() > 2) {
             deprecatedLinkContent.add(Entity.ZERO_WIDTH_SPACE);
         }
@@ -118,8 +112,7 @@ public abstract class AbstractExecutableMemberWriter extends AbstractMemberWrite
             Content tdSummary) {
         ExecutableElement ee = (ExecutableElement)member;
         Content memberLink = HtmlTree.SPAN(HtmlStyle.memberNameLink,
-                writer.getDocLink(context, te, ee,
-                name(ee), false));
+                writer.getDocLink(context, te, ee, name(ee), false));
         Content code = HtmlTree.CODE(memberLink);
         addParameters(ee, code);
         tdSummary.add(code);
@@ -145,10 +138,10 @@ public abstract class AbstractExecutableMemberWriter extends AbstractMemberWrite
      * @param isVarArg true if this is a link to var arg.
      * @param tree the content tree to which the parameter information will be added.
      */
-    protected void addParam(ExecutableElement member, VariableElement param,
+    protected void addParam(ExecutableElement member, VariableElement param, TypeMirror paramType,
             boolean isVarArg, Content tree) {
         Content link = writer.getLink(new LinkInfoImpl(configuration, EXECUTABLE_MEMBER_PARAM,
-                param.asType()).varargs(isVarArg));
+                paramType).varargs(isVarArg));
         tree.add(link);
         if(name(param).length() > 0) {
             tree.add(Entity.NO_BREAK_SPACE);
@@ -212,9 +205,11 @@ public abstract class AbstractExecutableMemberWriter extends AbstractMemberWrite
             sep = "," + DocletConstants.NL;
         }
         int paramstart;
+        ExecutableType instMeth = utils.asInstantiatedMethodType(typeElement, member);
         for (paramstart = 0; paramstart < parameters.size(); paramstart++) {
             paramTree.add(sep);
             VariableElement param = parameters.get(paramstart);
+            TypeMirror paramType = instMeth.getParameterTypes().get(paramstart);
 
             if (param.getKind() != ElementKind.INSTANCE_INIT) {
                 if (includeAnnotations) {
@@ -224,7 +219,7 @@ public abstract class AbstractExecutableMemberWriter extends AbstractMemberWrite
                         paramTree.add(DocletConstants.NL);
                     }
                 }
-                addParam(member, param,
+                addParam(member, param, paramType,
                     (paramstart == parameters.size() - 1) && member.isVarArgs(), paramTree);
                 break;
             }
@@ -241,7 +236,8 @@ public abstract class AbstractExecutableMemberWriter extends AbstractMemberWrite
                     paramTree.add(DocletConstants.NL);
                 }
             }
-            addParam(member, parameters.get(i), (i == parameters.size() - 1) && member.isVarArgs(),
+            addParam(member, parameters.get(i), instMeth.getParameterTypes().get(i),
+                    (i == parameters.size() - 1) && member.isVarArgs(),
                     paramTree);
         }
 
@@ -255,12 +251,12 @@ public abstract class AbstractExecutableMemberWriter extends AbstractMemberWrite
      * @return the content tree containing the exceptions information.
      */
     protected Content getExceptions(ExecutableElement member) {
-        List<? extends TypeMirror> exceptions = member.getThrownTypes();
+        List<? extends TypeMirror> exceptions = utils.asInstantiatedMethodType(typeElement, member).getThrownTypes();
         Content htmltree = new ContentBuilder();
         if (!exceptions.isEmpty()) {
             Content link = writer.getLink(new LinkInfoImpl(configuration, MEMBER, exceptions.get(0)));
             htmltree.add(link);
-            for(int i = 1; i < exceptions.size(); i++) {
+            for (int i = 1; i < exceptions.size(); i++) {
                 htmltree.add(",");
                 htmltree.add(DocletConstants.NL);
                 Content exceptionLink = writer.getLink(new LinkInfoImpl(configuration, MEMBER,
