@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 1998, 2019, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 1998, 2020, Oracle and/or its affiliates. All rights reserved.
  * ORACLE PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
  *
  *
@@ -151,19 +151,28 @@ public class Contents {
     public final Content packageLabel;
     public final Content package_;
     public final Content packagesLabel;
+    public final Content parameters;
     public final Content properties;
     public final Content propertyLabel;
     public final Content propertyDetailsLabel;
     public final Content propertySummaryLabel;
-    public final Content seeLabel;
+    public final Content record;
+    public final Content recordComponents;
+    public final Content referencedIn;
+    public final Content returns;
+    public final Content seeAlso;
     public final Content serializedForm;
     public final Content servicesLabel;
     public final Content specifiedByLabel;
     public final Content subclassesLabel;
     public final Content subinterfacesLabel;
     public final Content summaryLabel;
+    public final Content systemPropertiesLabel;
+    public final Content systemPropertiesSummaryLabel;
+    public final Content throws_;
     public final Content treeLabel;
     public final Content typeLabel;
+    public final Content typeParameters;
     public final Content useLabel;
     public final Content valueLabel;
 
@@ -178,7 +187,7 @@ public class Contents {
      * resources used to look up resource keys, and other details.
      */
     Contents(HtmlConfiguration configuration) {
-        this.resources = configuration.getResources();
+        this.resources = configuration.getDocResources();
 
         allClassesLabel = getNonBreakContent("doclet.All_Classes");
         allImplementedInterfacesLabel = getContent("doclet.All_Implemented_Interfaces");
@@ -278,19 +287,28 @@ public class Contents {
         packageLabel = getContent("doclet.Package");
         package_ = getContent("doclet.package");
         packagesLabel = getContent("doclet.Packages");
+        parameters = getContent("doclet.Parameters");
         properties = getContent("doclet.Properties");
         propertyLabel = getContent("doclet.Property");
         propertyDetailsLabel = getContent("doclet.Property_Detail");
         propertySummaryLabel = getContent("doclet.Property_Summary");
-        seeLabel = getContent("doclet.See");
+        record = getContent("doclet.Record");
+        recordComponents = getContent("doclet.RecordComponents");
+        referencedIn = getContent("doclet.ReferencedIn");
+        returns = getContent("doclet.Returns");
+        seeAlso = getContent("doclet.See_Also");
         serializedForm = getContent("doclet.Serialized_Form");
         servicesLabel = getContent("doclet.Services");
         specifiedByLabel = getContent("doclet.Specified_By");
         subclassesLabel = getContent("doclet.Subclasses");
         subinterfacesLabel = getContent("doclet.Subinterfaces");
         summaryLabel = getContent("doclet.Summary");
+        systemPropertiesLabel = getContent("doclet.systemProperties");
+        systemPropertiesSummaryLabel = getContent("doclet.systemPropertiesSummary");
+        throws_ = getContent("doclet.Throws");
         treeLabel = getContent("doclet.Tree");
         typeLabel = getContent("doclet.Type");
+        typeParameters = getContent("doclet.TypeParameters");
         useLabel = getContent("doclet.navClassUse");
         valueLabel = getContent("doclet.Value");
 
