@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2003, 2019, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2003, 2020, Oracle and/or its affiliates. All rights reserved.
  * ORACLE PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
  *
  *
@@ -25,23 +25,21 @@
 
 package jdk.javadoc.internal.doclets.formats.html.markup;
 
+import java.io.IOException;
+import java.io.Writer;
 import java.text.SimpleDateFormat;
 import java.util.ArrayList;
 import java.util.Arrays;
-import java.util.Calendar;
 import java.util.Collections;
 import java.util.Date;
-import java.util.GregorianCalendar;
 import java.util.List;
-import java.util.TimeZone;
 
 import jdk.javadoc.internal.doclets.toolkit.Content;
-import jdk.javadoc.internal.doclets.toolkit.util.DocFile;
 import jdk.javadoc.internal.doclets.toolkit.util.DocPath;
 import jdk.javadoc.internal.doclets.toolkit.util.DocPaths;
 
 /**
- * A builder for HTML HEAD elements.
+ * An HTML {@code <head>} element.
  *
  * Many methods return the current object, to facilitate fluent builder-style usage.
  *
@@ -50,8 +48,9 @@ import jdk.javadoc.internal.doclets.toolkit.util.DocPaths;
  *  This code and its internal interfaces are subject to change or
  *  deletion without notice.</b>
  */
-public class Head {
-    private final String docletVersion;
+public class Head extends Content {
+    private final Runtime.Version docletVersion;
+    private final Date generatedDate;
     private final DocPath pathToRoot;
     private String title;
     private String charset;
@@ -68,8 +67,6 @@ public class Head {
     private boolean addDefaultScript = true;
     private DocPath canonicalLink;
 
-    private static final Calendar calendar = new GregorianCalendar(TimeZone.getDefault());
-
     /**
      * Creates a {@code Head} object, for a given file and HTML version.
      * The file is used to help determine the relative paths to stylesheet and script files.
@@ -77,10 +74,11 @@ public class Head {
      * recording the time the file was created.
      * The doclet version should also be provided for recording in the file.
      * @param path the path for the file that will include this HEAD element
-     * @param docletVersion a string identifying the doclet version
+     * @param docletVersion the doclet version
      */
-    public Head(DocPath path, String docletVersion) {
+    public Head(DocPath path, Runtime.Version docletVersion, Date generatedDate) {
         this.docletVersion = docletVersion;
+        this.generatedDate = generatedDate;
         pathToRoot = path.parent().invert();
         keywords = new ArrayList<>();
         scripts = new ArrayList<>();
@@ -148,7 +146,7 @@ public class Head {
      * @param timestamp true if timestamps should be be added.
      * @return this object
      */
-    // For temporary backwards compatibiility, if this method is not called,
+    // For temporary backwards compatibility, if this method is not called,
     // no 'Generated by javadoc' comment will be added.
     public Head setTimestamp(boolean timestamp) {
         showTimestamp = timestamp;
@@ -215,7 +213,7 @@ public class Head {
      * Specifies a value for a
      * <a href="https://en.wikipedia.org/wiki/Canonical_link_element">canonical link</a>
      * in the {@code <head>} element.
-     * @param link
+     * @param link the value for the canonical link
      */
     public void setCanonicalLink(DocPath link) {
         this.canonicalLink = link;
@@ -232,16 +230,31 @@ public class Head {
         return this;
     }
 
+    /**
+     * {@inheritDoc}
+     *
+     * @implSpec This implementation always returns {@code false}.
+     *
+     * @return {@code false}
+     */
+    @Override
+    public boolean isEmpty() {
+        return false;
+    }
+
+    @Override
+    public boolean write(Writer out, boolean atNewline) throws IOException {
+        return toContent().write(out, atNewline);
+    }
+
     /**
      * Returns the HTML for the HEAD element.
      *
      * @return the HTML
      */
-    public Content toContent() {
-        Date now = showTimestamp ? calendar.getTime() : null;
-
-        HtmlTree tree = new HtmlTree(HtmlTag.HEAD);
-        tree.add(getGeneratedBy(showTimestamp, now));
+    private Content toContent() {
+        HtmlTree tree = new HtmlTree(TagName.HEAD);
+        tree.add(getGeneratedBy(showTimestamp, generatedDate));
         tree.add(HtmlTree.TITLE(title));
 
         if (charset != null) { // compatibility; should this be allowed?
@@ -250,7 +263,7 @@ public class Head {
 
         if (showTimestamp) {
             SimpleDateFormat dateFormat = new SimpleDateFormat("yyyy-MM-dd");
-            tree.add(HtmlTree.META("dc.created", dateFormat.format(now)));
+            tree.add(HtmlTree.META("dc.created", dateFormat.format(generatedDate)));
         }
 
         if (description != null) {
@@ -266,7 +279,7 @@ public class Head {
         }
 
         if (canonicalLink != null) {
-            HtmlTree link = new HtmlTree(HtmlTag.LINK);
+            HtmlTree link = new HtmlTree(TagName.LINK);
             link.put(HtmlAttr.REL, "canonical");
             link.put(HtmlAttr.HREF, canonicalLink.getPath());
             tree.add(link);
@@ -282,7 +295,7 @@ public class Head {
     private Comment getGeneratedBy(boolean timestamp, Date now) {
         String text = "Generated by javadoc"; // marker string, deliberately not localized
         if (timestamp) {
-            text += " ("+ docletVersion + ") on " + now;
+            text += " ("+ docletVersion.feature() + ") on " + now;
         }
         return new Comment(text);
     }
@@ -298,7 +311,9 @@ public class Head {
         }
 
         if (index) {
-            addStylesheet(tree, DocPaths.JQUERY_FILES.resolve(DocPaths.JQUERY_STYLESHEET_FILE));
+            // The order of the addStylesheet(...) calls is important
+            addStylesheet(tree, DocPaths.JQUERY_FILES.resolve(DocPaths.JQUERY_UI_CSS));
+            addStylesheet(tree, DocPaths.JQUERY_OVERRIDES_CSS);
         }
     }
 
@@ -317,15 +332,10 @@ public class Head {
                 mainBodyScript.append("var pathtoroot = ")
                         .appendStringLiteral(ptrPath + "/")
                         .append(";\n")
-                        .append("loadScripts(document, \'script\');");
+                        .append("loadScripts(document, 'script');");
             }
-            addJQueryFile(tree, DocPaths.JSZIP_MIN);
-            addJQueryFile(tree, DocPaths.JSZIPUTILS_MIN);
-            tree.add(new RawHtml("<!--[if IE]>"));
-            addJQueryFile(tree, DocPaths.JSZIPUTILS_IE_MIN);
-            tree.add(new RawHtml("<![endif]-->"));
-            addJQueryFile(tree, DocPaths.JQUERY_JS_3_4);
             addJQueryFile(tree, DocPaths.JQUERY_JS);
+            addJQueryFile(tree, DocPaths.JQUERY_UI_JS);
         }
         for (Script script : scripts) {
             tree.add(script.asContent());
