@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2010, 2019, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2010, 2020, Oracle and/or its affiliates. All rights reserved.
  * ORACLE PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
  *
  *
@@ -38,8 +38,6 @@ import jdk.javadoc.internal.doclets.toolkit.util.DocletConstants;
  *  If you write code that depends on this, you do so at your own risk.
  *  This code and its internal interfaces are subject to change or
  *  deletion without notice.</b>
- *
- * @author Bhavesh Patel
  */
 public class StringContent extends Content {
 
@@ -62,17 +60,6 @@ public class StringContent extends Content {
         Entity.escapeHtmlChars(initialContent, stringContent);
     }
 
-    /**
-     * This method is not supported by the class.
-     *
-     * @param content content that needs to be added
-     * @throws UnsupportedOperationException always
-     */
-    @Override
-    public void add(Content content) {
-        throw new UnsupportedOperationException();
-    }
-
     /**
      * Adds content for the StringContent object.  The method escapes
      * HTML characters for the string content that is added.
@@ -80,13 +67,11 @@ public class StringContent extends Content {
      * @param strContent string content to be added
      */
     @Override
-    public void add(CharSequence strContent) {
+    public StringContent add(CharSequence strContent) {
         Entity.escapeHtmlChars(strContent, stringContent);
+        return this;
     }
 
-    /**
-     * {@inheritDoc}
-     */
     @Override
     public boolean isEmpty() {
         return (stringContent.length() == 0);
@@ -97,17 +82,11 @@ public class StringContent extends Content {
         return RawHtml.charCount(stringContent.toString());
     }
 
-    /**
-     * {@inheritDoc}
-     */
     @Override
     public String toString() {
         return stringContent.toString();
     }
 
-    /**
-     * {@inheritDoc}
-     */
     @Override
     public boolean write(Writer out, boolean atNewline) throws IOException {
         String s = stringContent.toString();
