@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2003, 2016, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2003, 2019, Oracle and/or its affiliates. All rights reserved.
  * ORACLE PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
  *
  *
@@ -35,32 +35,25 @@ import javax.lang.model.element.TypeElement;
  *  If you write code that depends on this, you do so at your own risk.
  *  This code and its internal interfaces are subject to change or
  *  deletion without notice.</b>
- *
- * @author Jamie Ho
- * @author Bhavesh Patel (Modified)
  */
 
-public interface ConstructorWriter {
+public interface ConstructorWriter extends MemberWriter {
 
     /**
      * Get the constructor details tree header.
      *
-     * @param typeElement the class being documented
      * @param memberDetailsTree the content tree representing member details
      * @return content tree for the constructor details header
      */
-    public Content getConstructorDetailsTreeHeader(TypeElement typeElement,
-            Content memberDetailsTree);
+    Content getConstructorDetailsTreeHeader(Content memberDetailsTree);
 
     /**
      * Get the constructor documentation tree header.
      *
      * @param constructor the constructor being documented
-     * @param constructorDetailsTree the content tree representing constructor details
      * @return content tree for the constructor documentation header
      */
-    public Content getConstructorDocTreeHeader(ExecutableElement constructor,
-            Content constructorDetailsTree);
+    Content getConstructorDocTreeHeader(ExecutableElement constructor);
 
     /**
      * Get the signature for the given constructor.
@@ -68,7 +61,7 @@ public interface ConstructorWriter {
      * @param constructor the constructor being documented
      * @return content tree for the constructor signature
      */
-    public Content getSignature(ExecutableElement constructor);
+    Content getSignature(ExecutableElement constructor);
 
     /**
      * Add the deprecated output for the given constructor.
@@ -76,7 +69,7 @@ public interface ConstructorWriter {
      * @param constructor the constructor being documented
      * @param constructorDocTree content tree to which the deprecated information will be added
      */
-    public void addDeprecated(ExecutableElement constructor, Content constructorDocTree);
+    void addDeprecated(ExecutableElement constructor, Content constructorDocTree);
 
     /**
      * Add the comments for the given constructor.
@@ -84,7 +77,7 @@ public interface ConstructorWriter {
      * @param constructor the constructor being documented
      * @param constructorDocTree the content tree to which the comments will be added
      */
-    public void addComments(ExecutableElement constructor, Content constructorDocTree);
+    void addComments(ExecutableElement constructor, Content constructorDocTree);
 
     /**
      * Add the tags for the given constructor.
@@ -92,7 +85,7 @@ public interface ConstructorWriter {
      * @param constructor the constructor being documented
      * @param constructorDocTree the content tree to which the tags will be added
      */
-    public void addTags(ExecutableElement constructor, Content constructorDocTree);
+    void addTags(ExecutableElement constructor, Content constructorDocTree);
 
     /**
      * Get the constructor details tree.
@@ -101,27 +94,19 @@ public interface ConstructorWriter {
      * @param memberDetailsTree the content tree representing member details
      * @return content tree for the constructor details
      */
-    public Content getConstructorDetails(Content memberDetailsTreeHeader, Content memberDetailsTree);
-
-    /**
-     * Get the constructor documentation.
-     *
-     * @param constructorDocTree the content tree representing constructor documentation
-     * @return content tree for the constructor documentation
-     */
-    public Content getConstructorDoc(Content constructorDocTree);
+    Content getConstructorDetails(Content memberDetailsTreeHeader, Content memberDetailsTree);
 
     /**
      * Let the writer know whether a non public constructor was found.
      *
      * @param foundNonPubConstructor true if we found a non public constructor.
      */
-    public void setFoundNonPubConstructor(boolean foundNonPubConstructor);
+    void setFoundNonPubConstructor(boolean foundNonPubConstructor);
 
     /**
      * Gets the member header tree.
      *
      * @return a content tree for the member header
      */
-    public Content getMemberTreeHeader();
+    Content getMemberTreeHeader();
 }
