@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2003, 2018, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2003, 2019, Oracle and/or its affiliates. All rights reserved.
  * ORACLE PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
  *
  *
@@ -30,10 +30,9 @@ import java.util.EnumSet;
 import javax.lang.model.element.Element;
 
 import com.sun.source.doctree.DocTree;
+import jdk.javadoc.doclet.Taglet.Location;
 import jdk.javadoc.internal.doclets.toolkit.Content;
 
-import static com.sun.source.doctree.DocTree.Kind.CODE;
-
 /**
  * An inline Taglet used to denote literal code fragments.
  * The enclosed text is interpreted as not containing HTML markup or
@@ -50,14 +49,12 @@ import static com.sun.source.doctree.DocTree.Kind.CODE;
  *  If you write code that depends on this, you do so at your own risk.
  *  This code and its internal interfaces are subject to change or
  *  deletion without notice.</b>
- *
- * @author Scott Seligman
  */
 
 public class CodeTaglet extends BaseTaglet {
 
     CodeTaglet() {
-        super(CODE.tagName, true, EnumSet.allOf(Site.class));
+        super(DocTree.Kind.CODE, true, EnumSet.allOf(Location.class));
     }
 
     @Override
