@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2001, 2016, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2001, 2019, Oracle and/or its affiliates. All rights reserved.
  * ORACLE PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
  *
  *
@@ -25,13 +25,13 @@
 
 package jdk.javadoc.internal.doclets.toolkit.taglets;
 
+import java.util.EnumSet;
 import javax.lang.model.element.Element;
 
 import com.sun.source.doctree.DocTree;
+import jdk.javadoc.doclet.Taglet.Location;
 import jdk.javadoc.internal.doclets.toolkit.Content;
 
-import static com.sun.source.doctree.DocTree.Kind.*;
-
 /**
  * An inline Taglet representing {&#064;docRoot}.  This taglet is
  * used to get the relative path to the document's root output
@@ -41,30 +41,18 @@ import static com.sun.source.doctree.DocTree.Kind.*;
  *  If you write code that depends on this, you do so at your own risk.
  *  This code and its internal interfaces are subject to change or
  *  deletion without notice.</b>
- *
- * @author Jamie Ho
- * @author Doug Kramer
  */
 
-public class DocRootTaglet extends BaseInlineTaglet {
-
+public class DocRootTaglet extends BaseTaglet {
 
     /**
      * Construct a new DocRootTaglet.
      */
     public DocRootTaglet() {
-        name = DOC_ROOT.tagName;
+        super(DocTree.Kind.DOC_ROOT, true, EnumSet.allOf(Location.class));
     }
 
-    /**
-     * Given a <code>Doc</code> object, check if it holds any tags of
-     * this type.  If it does, return the string representing the output.
-     * If it does not, return null.
-     * @param holder
-     * @param tag a tag representing the custom tag.
-     * @param writer a {@link TagletWriter} Taglet writer.
-     * @return the string representation of this <code>Tag</code>.
-     */
+    @Override
     public Content getTagletOutput(Element holder, DocTree tag, TagletWriter writer) {
         return writer.getDocRootOutput();
     }
