@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2015, 2016, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2015, 2019, Oracle and/or its affiliates. All rights reserved.
  * ORACLE PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
  *
  *
@@ -25,30 +25,26 @@
 
 package jdk.javadoc.internal.doclets.toolkit.taglets;
 
-import com.sun.source.doctree.DocTree;
-import java.util.Map;
+import java.util.EnumSet;
 import javax.lang.model.element.Element;
+
+import com.sun.source.doctree.DocTree;
+import jdk.javadoc.doclet.Taglet.Location;
 import jdk.javadoc.internal.doclets.toolkit.Content;
 
 /**
  * An inline Taglet used to index word or a phrase.
  * The enclosed text is interpreted as not containing HTML markup or
  * nested javadoc tags.
- *
- * @author Bhavesh Patel
  */
 
-public class IndexTaglet extends BaseInlineTaglet {
+public class IndexTaglet extends BaseTaglet {
 
-    private static final String NAME = "index";
-
-    public String getName() {
-        return NAME;
+    IndexTaglet() {
+        super(DocTree.Kind.INDEX, true, EnumSet.allOf(Location.class));
     }
 
-    /**
-     * {@inheritDoc}
-     */
+    @Override
     public Content getTagletOutput(Element element, DocTree tag, TagletWriter writer) {
         return writer.indexTagOutput(element, tag);
     }
