@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2003, 2016, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2003, 2019, Oracle and/or its affiliates. All rights reserved.
  * ORACLE PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
  *
  *
@@ -25,12 +25,12 @@
 
 package jdk.javadoc.internal.doclets.toolkit.taglets;
 
+import java.util.EnumSet;
 import javax.lang.model.element.Element;
 
 import com.sun.source.doctree.DocTree;
+import jdk.javadoc.doclet.Taglet.Location;
 import jdk.javadoc.internal.doclets.toolkit.Content;
-
-import static com.sun.source.doctree.DocTree.Kind.*;
 /**
  * An inline Taglet used to denote literal text.
  * For example, the text:
@@ -42,21 +42,15 @@ import static com.sun.source.doctree.DocTree.Kind.*;
  *  If you write code that depends on this, you do so at your own risk.
  *  This code and its internal interfaces are subject to change or
  *  deletion without notice.</b>
- *
- * @author Scott Seligman
  */
 
-public class LiteralTaglet extends BaseInlineTaglet {
+public class LiteralTaglet extends BaseTaglet {
 
-    private static final String NAME = LITERAL.tagName;
-
-    public String getName() {
-        return NAME;
+    LiteralTaglet() {
+        super(DocTree.Kind.LITERAL, true, EnumSet.allOf(Location.class));
     }
 
-    /**
-     * {@inheritDoc}
-     */
+    @Override
     public Content getTagletOutput(Element e, DocTree tag, TagletWriter writer) {
         return writer.literalTagOutput(e, tag);
     }
