@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2003, 2019, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2003, 2020, Oracle and/or its affiliates. All rights reserved.
  * ORACLE PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
  *
  *
@@ -26,8 +26,9 @@
 package jdk.javadoc.internal.doclets.toolkit.taglets;
 
 import java.util.List;
-
 import javax.lang.model.element.Element;
+import javax.lang.model.element.ElementKind;
+import javax.lang.model.element.TypeElement;
 import javax.lang.model.element.VariableElement;
 import javax.lang.model.type.TypeMirror;
 
@@ -45,8 +46,6 @@ import jdk.javadoc.internal.doclets.toolkit.util.Utils;
  *  If you write code that depends on this, you do so at your own risk.
  *  This code and its internal interfaces are subject to change or
  *  deletion without notice.</b>
- *
- * @author Jamie Ho
  */
 
 public abstract class TagletWriter {
@@ -75,7 +74,7 @@ public abstract class TagletWriter {
     protected abstract Content codeTagOutput(Element element, DocTree tag);
 
     /**
-     * Return the output for a {@index...} tag.
+     * Return the output for a {@code {@index...}} tag.
      *
      * @param tag the tag.
      * @return the output of the taglet.
@@ -108,10 +107,10 @@ public abstract class TagletWriter {
     /**
      * Return the header for the param tags.
      *
-     * @param header the header to display.
-     * @return the header for the param tags.
+     * @param kind the kind of header that is required
+     * @return the header for the param tags
      */
-    protected abstract Content getParamHeader(String header);
+    protected abstract Content getParamHeader(ParamTaglet.ParamKind kind);
 
     /**
      * Return the output for param tags.
@@ -192,9 +191,10 @@ public abstract class TagletWriter {
      *
      * @param element
      * @param throwsTag the throws tag.
+     * @param substituteType instantiated type of a generic type-variable, or null.
      * @return the output of the throws tag.
      */
-    protected abstract Content throwsTagOutput(Element element, DocTree throwsTag);
+    protected abstract Content throwsTagOutput(Element element, DocTree throwsTag, TypeMirror substituteType);
 
     /**
      * Return the output for the throws tag.
@@ -216,27 +216,48 @@ public abstract class TagletWriter {
     protected abstract Content valueTagOutput(VariableElement field,
         String constantVal, boolean includeLink);
 
+    /**
+     * Return the main type element of the current page or null for pages that don't have one.
+     *
+     * @return the type element of the current page or null.
+     */
+    protected abstract TypeElement getCurrentPageElement();
+
     /**
      * Given an output object, append to it the tag documentation for
      * the given member.
      *
      * @param tagletManager the manager that manages the taglets.
-     * @param element the Doc that we are print tags for.
+     * @param element the element that we are print tags for.
      * @param taglets the taglets to print.
      * @param writer the writer that will generate the output strings.
      * @param output the output buffer to store the output in.
      */
-    public static void genTagOutput(TagletManager tagletManager, Element element,
-            List<Taglet> taglets, TagletWriter writer, Content output) {
+    public static void genTagOutput(TagletManager tagletManager,
+                                    Element element,
+                                    List<Taglet> taglets,
+                                    TagletWriter writer,
+                                    Content output)
+    {
         Utils utils = writer.configuration().utils;
         tagletManager.checkTags(element, utils.getBlockTags(element), false);
         tagletManager.checkTags(element, utils.getFullBody(element), true);
         for (Taglet taglet : taglets) {
             if (utils.isTypeElement(element) && taglet instanceof ParamTaglet) {
-                //The type parameters are documented in a special section away
-                //from the tag info, so skip here.
+                // The type parameters and state components are documented in a special
+                // section away from the tag info, so skip here.
                 continue;
             }
+            if (element.getKind() == ElementKind.MODULE && taglet instanceof BaseTaglet) {
+                BaseTaglet t = (BaseTaglet) taglet;
+                switch (t.getTagKind()) {
+                    // @uses and @provides are handled separately, so skip here.
+                    // See ModuleWriterImpl.computeModulesData
+                    case USES:
+                    case PROVIDES:
+                        continue;
+                }
+            }
             if (taglet instanceof DeprecatedTaglet) {
                 //Deprecated information is documented "inline", not in tag info
                 //section.
@@ -252,7 +273,7 @@ public abstract class TagletWriter {
             } catch (UnsupportedTagletOperationException utoe) {
                 //The taglet does not take a member as an argument.  Let's try
                 //a single tag.
-                List<? extends DocTree> tags = utils.getBlockTags(element, taglet.getName());
+                List<? extends DocTree> tags = utils.getBlockTags(element, taglet);
                 if (!tags.isEmpty()) {
                     currentOutput = taglet.getTagletOutput(element, tags.get(0), writer);
                 }
@@ -267,14 +288,18 @@ public abstract class TagletWriter {
      * Given an inline tag, return its output.
      * @param holder
      * @param tagletManager The taglet manager for the current doclet.
-     * @param holderTag The tag this holds this inline tag.  Null if there
-     * is no tag that holds it.
+     * @param holderTag The tag that holds this inline tag, or {@code null} if
+     *                  there is no tag that holds it.
      * @param inlineTag The inline tag to be documented.
      * @param tagletWriter The taglet writer to write the output.
      * @return The output of the inline tag.
      */
-    public static Content getInlineTagOutput(Element holder, TagletManager tagletManager,
-            DocTree holderTag, DocTree inlineTag, TagletWriter tagletWriter) {
+    public static Content getInlineTagOutput(Element holder,
+                                             TagletManager tagletManager,
+                                             DocTree holderTag,
+                                             DocTree inlineTag,
+                                             TagletWriter tagletWriter)
+    {
         List<Taglet> definedTags = tagletManager.getInlineTaglets();
         CommentHelper ch = tagletWriter.configuration().utils.getCommentHelper(holder);
         final String inlineTagName = ch.getTagName(inlineTag);
