@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2003, 2016, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2003, 2019, Oracle and/or its affiliates. All rights reserved.
  * ORACLE PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
  *
  *
@@ -26,14 +26,12 @@
 package jdk.javadoc.internal.doclets.toolkit.util.links;
 
 /**
- * Stores output of a link.
+ *  Stores output of a link.
  *
  *  <p><b>This is NOT part of any supported API.
  *  If you write code that depends on this, you do so at your own risk.
  *  This code and its internal interfaces are subject to change or
  *  deletion without notice.</b>
- *
- * @author Jamie Ho
  */
 public interface LinkOutput {
 
@@ -42,7 +40,7 @@ public interface LinkOutput {
      *
      * @param o the object to append.
      */
-    public void append(Object o);
+    void append(Object o);
 
     /**
      * Insert the given object into the output sequence.
@@ -50,5 +48,5 @@ public interface LinkOutput {
      * @param offset the offset.
      * @param o the object to be inserted.
      */
-    public void insert(int offset, Object o);
+    void insert(int offset, Object o);
 }
