@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 1997, 2018, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 1997, 2020, Oracle and/or its affiliates. All rights reserved.
  * ORACLE PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
  *
  *
@@ -28,28 +28,22 @@ package jdk.javadoc.internal.tool;
 import java.io.File;
 import java.io.IOException;
 import java.io.PrintWriter;
-import java.nio.file.Path;
 import java.text.BreakIterator;
 import java.text.Collator;
 import java.util.ArrayList;
-import java.util.Arrays;
-import java.util.Collection;
 import java.util.Collections;
 import java.util.Comparator;
 import java.util.IllformedLocaleException;
 import java.util.List;
 import java.util.Locale;
-import java.util.MissingResourceException;
 import java.util.Objects;
-import java.util.ResourceBundle;
 import java.util.Set;
+import java.util.function.Supplier;
 import java.util.stream.Collectors;
-import java.util.stream.Stream;
 
 import javax.tools.JavaFileManager;
 import javax.tools.JavaFileObject;
 import javax.tools.StandardJavaFileManager;
-import javax.tools.StandardLocation;
 
 import com.sun.tools.javac.api.JavacTrees;
 import com.sun.tools.javac.file.BaseFileManager;
@@ -57,25 +51,21 @@ import com.sun.tools.javac.file.JavacFileManager;
 import com.sun.tools.javac.jvm.Target;
 import com.sun.tools.javac.main.Arguments;
 import com.sun.tools.javac.main.CommandLine;
-import com.sun.tools.javac.main.OptionHelper;
-import com.sun.tools.javac.main.OptionHelper.GrumpyHelper;
-import com.sun.tools.javac.platform.PlatformDescription;
-import com.sun.tools.javac.platform.PlatformUtils;
 import com.sun.tools.javac.util.ClientCodeException;
 import com.sun.tools.javac.util.Context;
 import com.sun.tools.javac.util.Log;
-import com.sun.tools.javac.util.Log.WriterKind;
-import com.sun.tools.javac.util.Options;
 import com.sun.tools.javac.util.StringUtils;
 
 import jdk.javadoc.doclet.Doclet;
 import jdk.javadoc.doclet.Doclet.Option;
 import jdk.javadoc.doclet.DocletEnvironment;
+import jdk.javadoc.doclet.StandardDoclet;
+import jdk.javadoc.internal.Versions;
 import jdk.javadoc.internal.tool.Main.Result;
+import jdk.javadoc.internal.tool.ToolOptions.ToolOption;
 
 import static javax.tools.DocumentationTool.Location.*;
 
-import static com.sun.tools.javac.main.Option.*;
 import static jdk.javadoc.internal.tool.Main.Result.*;
 
 /**
@@ -86,14 +76,9 @@ import static jdk.javadoc.internal.tool.Main.Result.*;
  *  If you write code that depends on this, you do so at your own risk.
  *  This code and its internal interfaces are subject to change or
  *  deletion without notice.</b>
- *
- * @author Robert Field
- * @author Neal Gafter (rewrite)
  */
-public class Start extends ToolOption.Helper {
+public class Start {
 
-    private static final Class<?> StdDoclet =
-            jdk.javadoc.doclet.StandardDoclet.class;
     /** Context for this invocation. */
     private final Context context;
 
@@ -112,7 +97,6 @@ public class Start extends ToolOption.Helper {
     // used to determine the locale for the messager
     private Locale locale;
 
-
     /**
      * In API mode, exceptions thrown while calling the doclet are
      * propagated using ClientCodeException.
@@ -121,6 +105,8 @@ public class Start extends ToolOption.Helper {
 
     private JavaFileManager fileManager;
 
+    private final ToolOptions options;
+
     Start() {
         this(null, null, null, null, null, null);
     }
@@ -141,6 +127,8 @@ public class Start extends ToolOption.Helper {
         this.classLoader = classLoader;
         this.docletClass = null;
         this.locale = Locale.getDefault();
+
+        options = getToolOptions();
     }
 
     public Start(Context context) {
@@ -150,56 +138,69 @@ public class Start extends ToolOption.Helper {
         this.docletName = null;
         this.classLoader = null;
         this.locale = Locale.getDefault();
-    }
 
-    void initMessager() {
-        if (!apiMode)
-            return;
-        if (messager == null) {
-            Log log = context.get(Log.logKey);
-            if (log instanceof Messager) {
-                messager = (Messager) log;
-            } else {
-                PrintWriter out = context.get(Log.errKey);
-                messager = (out == null)
-                        ? new Messager(context, ProgramName)
-                        : new Messager(context, ProgramName, out, out);
-            }
+        Log log = context.get(Log.logKey);
+        if (log instanceof Messager) {
+            messager = (Messager) log;
+        } else {
+            PrintWriter out = context.get(Log.errKey);
+            messager = (out == null)
+                    ? new Messager(context, ProgramName)
+                    : new Messager(context, ProgramName, out, out);
         }
+
+        options = getToolOptions();
     }
 
-    /**
-     * Usage
-     */
-    @Override
-    void usage() {
-        usage("main.usage", OptionKind.STANDARD, "main.usage.foot");
+    private ToolOptions getToolOptions() {
+        ToolOptions.ShowHelper helper =  new ToolOptions.ShowHelper() {
+            @Override
+            public void usage() {
+                showUsage("main.usage", ToolOption.Kind.STANDARD, "main.usage.foot");
+            }
+
+            @Override
+            public void Xusage() {
+                showUsage("main.Xusage", ToolOption.Kind.EXTENDED, "main.Xusage.foot");
+            }
+
+            @Override
+            public void version() {
+                showVersion("javadoc.version", orDefault(() -> Versions.shortVersionStringOf(toolVersion())));
+            }
+
+            @Override
+            public void fullVersion() {
+                showVersion("javadoc.fullversion", orDefault(() -> Versions.fullVersionStringOf(toolVersion())));
+            }
+
+            private String orDefault(Supplier<String> s) {
+                try {
+                    return s.get();
+                } catch (RuntimeException e) {
+                    assert false : e;
+                    return Log.getLocalizedString("version.not.available");
+                }
+            }
+        };
+        return new ToolOptions(context, messager, helper);
     }
 
-    @Override
-    void Xusage() {
-        usage("main.Xusage", OptionKind.EXTENDED, "main.Xusage.foot");
+    private Runtime.Version toolVersion() {
+        return Versions.javadocVersion();
     }
 
-    @Override
-    void version() {
-        messager.notice("javadoc.version", messager.programName, version("release"));
+    private void showUsage() {
+        showUsage("main.usage", ToolOption.Kind.STANDARD, "main.usage.foot");
     }
 
-    @Override
-    void fullVersion() {
-        messager.notice("javadoc.fullversion", messager.programName, version("full"));
-    }
-
-    private void usage(String headerKey, OptionKind kind, String footerKey) {
+    private void showUsage(String headerKey, ToolOption.Kind kind, String footerKey) {
         messager.notice(headerKey);
         showToolOptions(kind);
 
         // let doclet print usage information
         if (docletClass != null) {
-            String name = doclet.getName();
-            messager.notice("main.doclet.usage.header", name);
-            showDocletOptions(kind == OptionKind.EXTENDED
+            showDocletOptions(kind == ToolOption.Kind.EXTENDED
                     ? Option.Kind.EXTENDED
                     : Option.Kind.STANDARD);
         }
@@ -207,25 +208,11 @@ public class Start extends ToolOption.Helper {
             messager.notice(footerKey);
     }
 
-    private static final String versionRBName = "jdk.javadoc.internal.tool.resources.version";
-    private static ResourceBundle versionRB;
-
-    private static String version(String key) {
-        if (versionRB == null) {
-            try {
-                versionRB = ResourceBundle.getBundle(versionRBName);
-            } catch (MissingResourceException e) {
-                return Log.getLocalizedString("version.not.available");
-            }
-        }
-        try {
-            return versionRB.getString(key);
-        } catch (MissingResourceException e) {
-            return Log.getLocalizedString("version.not.available");
-        }
+    private void showVersion(String labelKey, String value) {
+        messager.notice(labelKey, messager.programName, value);
     }
 
-    void showToolOptions(OptionKind kind) {
+    private void showToolOptions(ToolOption.Kind kind) {
         Comparator<ToolOption> comp = new Comparator<ToolOption>() {
             final Collator collator = Collator.getInstance(Locale.US);
             { collator.setStrength(Collator.PRIMARY); }
@@ -236,26 +223,37 @@ public class Start extends ToolOption.Helper {
             }
         };
 
-        Stream.of(ToolOption.values())
+        options.getSupportedOptions().stream()
                     .filter(opt -> opt.kind == kind)
                     .sorted(comp)
                     .forEach(this::showToolOption);
     }
 
-    void showToolOption(ToolOption option) {
+    private void showToolOption(ToolOption option) {
         List<String> names = option.getNames();
+        String primaryName = option.primaryName;
         String parameters;
-        if (option.hasArg || option.primaryName.endsWith(":")) {
-            String sep = (option == ToolOption.J) || option.primaryName.endsWith(":") ? "" : " ";
+        if (option.hasArg || primaryName.endsWith(":")) {
+            String sep = primaryName.endsWith(":")
+                    || primaryName.equals(ToolOptions.AT)
+                    || primaryName.equals(ToolOptions.J)
+                    ? "" : " ";
             parameters = sep + option.getParameters(messager);
         } else {
             parameters = "";
         }
         String description = option.getDescription(messager);
-        showUsage(names, parameters, description);
+        showOption(names, parameters, description);
     }
 
-    void showDocletOptions(Option.Kind kind) {
+    private void showDocletOptions(Option.Kind kind) {
+        String name = doclet.getName();
+        Set<? extends Option> options = getSupportedOptionsOf(doclet);
+        if (options.isEmpty()) {
+            return;
+        }
+        messager.notice("main.doclet.usage.header", name);
+
         Comparator<Doclet.Option> comp = new Comparator<Doclet.Option>() {
             final Collator collator = Collator.getInstance(Locale.US);
             { collator.setStrength(Collator.PRIMARY); }
@@ -266,33 +264,33 @@ public class Start extends ToolOption.Helper {
             }
         };
 
-        doclet.getSupportedOptions().stream()
+        options.stream()
                 .filter(opt -> opt.getKind() == kind)
                 .sorted(comp)
                 .forEach(this::showDocletOption);
     }
 
-    void showDocletOption(Doclet.Option option) {
+    private void showDocletOption(Doclet.Option option) {
         List<String> names = option.getNames();
         String parameters;
-        String optname = names.get(0);
-        if (option.getArgumentCount() > 0 || optname.endsWith(":")) {
-            String sep = optname.endsWith(":") ? "" : " ";
+        String primaryName = names.get(0);
+        if (option.getArgumentCount() > 0 || primaryName.endsWith(":")) {
+            String sep = primaryName.endsWith(":") ? "" : " ";
             parameters = sep + option.getParameters();
         } else {
             parameters = "";
         }
         String description = option.getDescription();
-        showUsage(names, parameters, description);
+        showOption(names, parameters, description);
     }
 
     // The following constants are intended to format the output to
     // be similar to that of the java launcher: i.e. "java -help".
 
     /** The indent for the option synopsis. */
-    private static final String SMALL_INDENT = "    ";
+    private static final String SMALL_INDENT = " ".repeat(4);
     /** The automatic indent for the description. */
-    private static final String LARGE_INDENT = "                  ";
+    private static final String LARGE_INDENT = " ".repeat(18);
     /** The space allowed for the synopsis, if the description is to be shown on the same line. */
     private static final int DEFAULT_SYNOPSIS_WIDTH = 13;
     /** The nominal maximum line length, when seeing if text will fit on a line. */
@@ -300,7 +298,7 @@ public class Start extends ToolOption.Helper {
     /** The format for a single-line help entry. */
     private static final String COMPACT_FORMAT = SMALL_INDENT + "%-" + DEFAULT_SYNOPSIS_WIDTH + "s %s";
 
-    void showUsage(List<String> names, String parameters, String description) {
+    void showOption(List<String> names, String parameters, String description) {
         String synopses = names.stream()
                 .map(s -> s + parameters)
                 .collect(Collectors.joining(", "));
@@ -340,19 +338,21 @@ public class Start extends ToolOption.Helper {
     @SuppressWarnings("deprecation")
     Result begin(String... argv) {
         // Preprocess @file arguments
+        List<String> allArgs;
         try {
-            argv = CommandLine.parse(argv);
-            return begin(Arrays.asList(argv), Collections.emptySet());
+            allArgs = CommandLine.parse(List.of(argv));
         } catch (IOException e) {
             error("main.cant.read", e.getMessage());
             return ERROR;
         }
+        return begin(allArgs, Collections.emptySet());
     }
 
-    // Called by 199 API.
+    // Called by the JSR 199 API
     public boolean begin(Class<?> docletClass,
-            Iterable<String> options,
-            Iterable<? extends JavaFileObject> fileObjects) {
+                         Iterable<String> options,
+                         Iterable<? extends JavaFileObject> fileObjects)
+    {
         this.docletClass = docletClass;
         List<String> opts = new ArrayList<>();
         for (String opt: options)
@@ -361,7 +361,6 @@ public class Start extends ToolOption.Helper {
         return begin(opts, fileObjects).isOK();
     }
 
-    @SuppressWarnings("removal")
     private Result begin(List<String> options, Iterable<? extends JavaFileObject> fileObjects) {
         fileManager = context.get(JavaFileManager.class);
         if (fileManager == null) {
@@ -372,9 +371,10 @@ public class Start extends ToolOption.Helper {
             }
         }
 
-        // locale, doclet and maybe taglet, needs to be determined first
+        // Perform an initial scan of the options to determine the doclet to be used (if any),
+        // so that it may participate in the main round of option processing.
         try {
-            doclet = preprocess(fileManager, options);
+            doclet = preprocess(options);
         } catch (ToolException te) {
             if (!te.result.isOK()) {
                 if (te.message != null) {
@@ -402,14 +402,14 @@ public class Start extends ToolOption.Helper {
             Throwable t = e.getCause();
             dumpStack(t == null ? e : t);
             return ERROR;
-        } catch (OptionException toe) {
-            if (toe.message != null)
-                messager.printError(toe.message);
+        } catch (OptionException oe) {
+            if (oe.message != null)
+                messager.printError(oe.message);
 
-            toe.m.run();
-            Throwable t = toe.getCause();
-            dumpStack(t == null ? toe : t);
-            return toe.result;
+            oe.m.run();
+            Throwable t = oe.getCause();
+            dumpStack(t == null ? oe : t);
+            return oe.result;
         } catch (ToolException exc) {
             if (exc.message != null) {
                 messager.printError(exc.message);
@@ -440,13 +440,15 @@ public class Start extends ToolOption.Helper {
                     fileManager.close();
                 } catch (IOException ignore) {}
             }
-            boolean haveErrorWarnings = messager.hasErrors()
-                    || (rejectWarnings && messager.hasWarnings());
-            if (!result.isOK() && !haveErrorWarnings) {
+            if (this.options.rejectWarnings() && messager.hasWarnings()) {
+                error("main.warnings.Werror");
+            }
+            boolean haveErrors = messager.hasErrors();
+            if (!result.isOK() && !haveErrors) {
                 // the doclet failed, but nothing reported, flag it!.
                 error("main.unknown.error");
             }
-            if (haveErrorWarnings && result.isOK()) {
+            if (haveErrors && result.isOK()) {
                 result = ERROR;
             }
             messager.printErrorWarningCounts();
@@ -465,7 +467,7 @@ public class Start extends ToolOption.Helper {
     }
 
     private void dumpStack(boolean enabled, Throwable t) {
-        if (t != null && (enabled || dumpOnError)) {
+        if (t != null && (enabled || options.dumpOnError())) {
             t.printStackTrace(System.err);
         }
     }
@@ -473,18 +475,16 @@ public class Start extends ToolOption.Helper {
     /**
      * Main program - internal
      */
-    @SuppressWarnings("unchecked")
     private Result parseAndExecute(List<String> argList, Iterable<? extends JavaFileObject> fileObjects)
-            throws ToolException, OptionException, com.sun.tools.javac.main.Option.InvalidValueException {
-        long tm = System.currentTimeMillis();
+            throws ToolException, OptionException, com.sun.tools.javac.main.Option.InvalidValueException
+    {
+        final long startNanos = System.nanoTime();
 
         List<String> javaNames = new ArrayList<>();
 
-        compOpts = Options.instance(context);
-
         // Make sure no obsolete source/target messages are reported
         try {
-            com.sun.tools.javac.main.Option.XLINT_CUSTOM.process(getOptionHelper(), "-Xlint:-options");
+            options.processCompilerOption(com.sun.tools.javac.main.Option.XLINT_CUSTOM, "-Xlint:-options");
         } catch (com.sun.tools.javac.main.Option.InvalidValueException ignore) {
         }
 
@@ -493,7 +493,12 @@ public class Start extends ToolOption.Helper {
         arguments.allowEmpty();
 
         doclet.init(locale, messager);
-        parseArgs(argList, javaNames);
+        int beforeCount = messager.nerrors;
+        boolean success = parseArgs(argList, javaNames);
+        int afterCount = messager.nerrors;
+        if (!success && beforeCount == afterCount) { // if there were failures but they have not been reported
+            return CMDERR;
+        }
 
         if (!arguments.handleReleaseOptions(extra -> true)) {
             // Arguments does not always increase the error count in the
@@ -514,22 +519,19 @@ public class Start extends ToolOption.Helper {
         }
 
         if (fileManager instanceof BaseFileManager) {
-            ((BaseFileManager) fileManager).handleOptions(fileManagerOpts);
+            ((BaseFileManager) fileManager).handleOptions(options.fileManagerOptions());
         }
 
-        if (fileManager.isSupportedOption(MULTIRELEASE.primaryName) == 1) {
+        String mr = com.sun.tools.javac.main.Option.MULTIRELEASE.primaryName;
+        if (fileManager.isSupportedOption(mr) == 1) {
             Target target = Target.instance(context);
             List<String> list = List.of(target.multiReleaseValue());
-            fileManager.handleOption(MULTIRELEASE.primaryName, list.iterator());
+            fileManager.handleOption(mr, list.iterator());
         }
-        compOpts.notifyListeners();
-        List<String> modules = (List<String>) jdtoolOpts.computeIfAbsent(ToolOption.MODULE,
-                s -> Collections.EMPTY_LIST);
+        options.compilerOptions().notifyListeners();
 
-        if (modules.isEmpty()) {
-            List<String> subpkgs = (List<String>) jdtoolOpts.computeIfAbsent(ToolOption.SUBPACKAGES,
-                    s -> Collections.EMPTY_LIST);
-            if (subpkgs.isEmpty()) {
+        if (options.modules().isEmpty()) {
+            if (options.subpackages().isEmpty()) {
                 if (javaNames.isEmpty() && isEmpty(fileObjects)) {
                     String text = messager.getText("main.No_modules_packages_or_classes_specified");
                     throw new ToolException(CMDERR, text);
@@ -540,14 +542,12 @@ public class Start extends ToolOption.Helper {
         JavadocTool comp = JavadocTool.make0(context);
         if (comp == null) return ABNORMAL;
 
-        DocletEnvironment docEnv = comp.getEnvironment(jdtoolOpts,
-                javaNames,
-                fileObjects);
+        DocletEnvironment docEnv = comp.getEnvironment(options, javaNames, fileObjects);
 
         // release resources
         comp = null;
 
-        if (breakiterator || !locale.getLanguage().equals(Locale.ENGLISH.getLanguage())) {
+        if (options.breakIterator() || !locale.getLanguage().equals(Locale.ENGLISH.getLanguage())) {
             JavacTrees trees = JavacTrees.instance(context);
             trees.setBreakIterator(BreakIterator.getSentenceInstance(locale));
         }
@@ -557,9 +557,9 @@ public class Start extends ToolOption.Helper {
                 : ERROR;
 
         // We're done.
-        if (compOpts.get("-verbose") != null) {
-            tm = System.currentTimeMillis() - tm;
-            messager.notice("main.done_in", Long.toString(tm));
+        if (options.verbose()) {
+            long elapsedMillis = (System.nanoTime() - startNanos) / 1_000_000;
+            messager.notice("main.done_in", Long.toString(elapsedMillis));
         }
 
         return returnStatus;
@@ -581,11 +581,16 @@ public class Start extends ToolOption.Helper {
         return matches(option.getNames(), targ);
     }
 
-    Set<? extends Doclet.Option> docletOptions = null;
-    int handleDocletOptions(int idx, List<String> args, boolean isToolOption)
-            throws OptionException {
+    private Set<? extends Doclet.Option> docletOptions = null;
+
+    /*
+     * Consumes an option along with its arguments. Returns an advanced index
+     * modulo the sign. If the value is negative, it means there was a failure
+     * processing one or more options.
+     */
+    int consumeDocletOption(int idx, List<String> args, boolean isToolOption) throws OptionException {
         if (docletOptions == null) {
-            docletOptions = doclet.getSupportedOptions();
+            docletOptions = getSupportedOptionsOf(doclet);
         }
         String arg = args.get(idx);
         String argBase, argVal;
@@ -597,6 +602,7 @@ public class Start extends ToolOption.Helper {
             argBase = arg;
             argVal = null;
         }
+        int m = 1;
         String text = null;
         for (Doclet.Option opt : docletOptions) {
             if (matches(opt, argBase)) {
@@ -604,54 +610,75 @@ public class Start extends ToolOption.Helper {
                     switch (opt.getArgumentCount()) {
                         case 0:
                             text = messager.getText("main.unnecessary_arg_provided", argBase);
-                            throw new OptionException(ERROR, this::usage, text);
+                            throw new OptionException(ERROR, this::showUsage, text);
                         case 1:
-                            opt.process(arg, Arrays.asList(argVal));
+                            if (!opt.process(arg, Collections.singletonList(argVal))) {
+                                m = -1;
+                            }
                             break;
                         default:
                             text = messager.getText("main.only_one_argument_with_equals", argBase);
-                            throw new OptionException(ERROR, this::usage, text);
+                            throw new OptionException(ERROR, this::showUsage, text);
                     }
                 } else {
-                    if (args.size() - idx -1 < opt.getArgumentCount()) {
+                    if (args.size() - idx - 1 < opt.getArgumentCount()) {
                         text = messager.getText("main.requires_argument", arg);
-                        throw new OptionException(ERROR, this::usage, text);
+                        throw new OptionException(ERROR, this::showUsage, text);
+                    }
+                    if (!opt.process(arg, args.subList(idx + 1, idx + 1 + opt.getArgumentCount()))) {
+                        m = -1;
                     }
-                    opt.process(arg, args.subList(idx + 1, args.size()));
                     idx += opt.getArgumentCount();
                 }
-                return idx;
+                return m * idx;
             }
         }
         // check if arg is accepted by the tool before emitting error
         if (!isToolOption) {
             text = messager.getText("main.invalid_flag", arg);
-            throw new OptionException(ERROR, this::usage, text);
+            throw new OptionException(ERROR, this::showUsage, text);
         }
-        return idx;
+        return m * idx;
     }
 
-    private Doclet preprocess(JavaFileManager jfm,
-            List<String> argv) throws ToolException, OptionException {
+    private static Set<? extends Option> getSupportedOptionsOf(Doclet doclet) {
+        Set<? extends Option> options = doclet.getSupportedOptions();
+        return options == null ? Set.of() : options;
+    }
+
+    /**
+     * Performs an initial pass over the options, primarily to determine
+     * the doclet to be used (if any), so that it may participate in the
+     * main round of option decoding. This avoids having to specify that
+     * the options to specify the doclet should appear before any options
+     * that are handled by the doclet.
+     *
+     * The downside of this initial phase is that we have to skip over
+     * unknown options, and assume that we can reliably detect the options
+     * we need to handle.
+     *
+     * @param argv the arguments to be processed
+     * @return the doclet
+     * @throws ToolException if an error occurs initializing the doclet
+     * @throws OptionException if an error occurs while processing an option
+     */
+    private Doclet preprocess(List<String> argv) throws ToolException, OptionException {
         // doclet specifying arguments
         String userDocletPath = null;
         String userDocletName = null;
 
-        // taglet specifying arguments, since tagletpath is a doclet
-        // functionality, assume they are repeated and inspect all.
-        List<File> userTagletPath = new ArrayList<>();
-        List<String> userTagletNames = new ArrayList<>();
-
         // Step 1: loop through the args, set locale early on, if found.
-        for (int i = 0 ; i < argv.size() ; i++) {
+        for (int i = 0; i < argv.size(); i++) {
             String arg = argv.get(i);
-            if (arg.equals(ToolOption.DUMPONERROR.primaryName)) {
-                dumpOnError = true;
-            } else if (arg.equals(ToolOption.LOCALE.primaryName)) {
+            if (arg.equals(ToolOptions.DUMP_ON_ERROR)) {
+                // although this option is not needed in order to initialize the doclet,
+                // it is helpful if it is set before trying to initialize the doclet
+                options.setDumpOnError(true);
+            } else if (arg.equals(ToolOptions.LOCALE)) {
                 checkOneArg(argv, i++);
                 String lname = argv.get(i);
                 locale = getLocale(lname);
-            } else if (arg.equals(ToolOption.DOCLET.primaryName)) {
+            } else if (arg.equals(ToolOptions.DOCLET)) {
                 checkOneArg(argv, i++);
                 if (userDocletName != null) {
                     if (apiMode) {
@@ -672,26 +699,19 @@ public class Start extends ToolOption.Helper {
                     throw new ToolException(CMDERR, text);
                 }
                 userDocletName = argv.get(i);
-            } else if (arg.equals(ToolOption.DOCLETPATH.primaryName)) {
+            } else if (arg.equals(ToolOptions.DOCLET_PATH)) {
                 checkOneArg(argv, i++);
                 if (userDocletPath == null) {
                     userDocletPath = argv.get(i);
                 } else {
                     userDocletPath += File.pathSeparator + argv.get(i);
                 }
-            } else if ("-taglet".equals(arg)) {
-                userTagletNames.add(argv.get(i + 1));
-            } else if ("-tagletpath".equals(arg)) {
-                for (String pathname : argv.get(i + 1).split(File.pathSeparator)) {
-                    userTagletPath.add(new File(pathname));
-                }
             }
         }
 
-
         // Step 3: doclet name specified ? if so find a ClassLoader,
         // and load it.
-        if(docletClass == null) {
+        if (docletClass == null) {
             if (userDocletName != null) {
                 ClassLoader cl = classLoader;
                 if (cl == null) {
@@ -731,14 +751,12 @@ public class Start extends ToolOption.Helper {
             } else if (docletName != null){
                 docletClass = loadDocletClass(docletName, getClass().getClassLoader());
             } else {
-                docletClass = StdDoclet;
+                docletClass = StandardDoclet.class;
             }
         }
 
-        if (jdk.javadoc.doclet.Doclet.class.isAssignableFrom(docletClass)) {
-            // no need to dispatch to old, safe to init now
-            initMessager();
-            messager.setLocale(locale);
+        if (Doclet.class.isAssignableFrom(docletClass)) {
+            messager.setLocale(Locale.getDefault());  // use default locale for console messages
             try {
                 Object o = docletClass.getConstructor().newInstance();
                 doclet = (Doclet) o;
@@ -768,40 +786,49 @@ public class Start extends ToolOption.Helper {
         }
     }
 
-    private void parseArgs(List<String> args, List<String> javaNames) throws ToolException,
-            OptionException, com.sun.tools.javac.main.Option.InvalidValueException {
-        for (int i = 0 ; i < args.size() ; i++) {
+    private boolean parseArgs(List<String> args, List<String> javaNames)
+            throws OptionException, com.sun.tools.javac.main.Option.InvalidValueException
+    {
+        boolean success = true;
+        for (int i = 0; i < args.size(); i++) {
             String arg = args.get(i);
-            ToolOption o = ToolOption.get(arg);
+            ToolOption o = options.getOption(arg);
             if (o != null) {
                 // handle a doclet argument that may be needed however
                 // don't increment the index, and allow the tool to consume args
-                handleDocletOptions(i, args, true);
+                if (consumeDocletOption(i, args, true) < 0) {
+                    success = false;
+                }
                 if (o.hasArg) {
                     if (arg.startsWith("--") && arg.contains("=")) {
-                        o.process(this, arg.substring(arg.indexOf('=') + 1));
+                        o.process(arg.substring(arg.indexOf('=') + 1));
                     } else {
                         checkOneArg(args, i++);
-                        o.process(this, args.get(i));
+                        o.process(args.get(i));
                     }
                 } else if (o.hasSuffix) {
-                    o.process(this, arg);
+                    o.process(arg);
                 } else {
-                    o.process(this);
+                    o.process();
                 }
             } else if (arg.startsWith("-XD")) {
                 // hidden javac options
                 String s = arg.substring("-XD".length());
                 int eq = s.indexOf('=');
                 String key = (eq < 0) ? s : s.substring(0, eq);
-                String value = (eq < 0) ? s : s.substring(eq+1);
-                compOpts.put(key, value);
+                String value = (eq < 0) ? s : s.substring(eq + 1);
+                options.compilerOptions().put(key, value);
             } else if (arg.startsWith("-")) {
-                i = handleDocletOptions(i, args, false);
+                i = consumeDocletOption(i, args, false);
+                if (i < 0) {
+                    i = -i;
+                    success = false;
+                }
             } else {
                 javaNames.add(arg);
             }
         }
+        return success;
     }
 
     private <T> boolean isEmpty(Iterable<T> iter) {
@@ -815,7 +842,7 @@ public class Start extends ToolOption.Helper {
     private void checkOneArg(List<String> args, int index) throws OptionException {
         if ((index + 1) >= args.size() || args.get(index + 1).startsWith("-d")) {
             String text = messager.getText("main.requires_argument", args.get(index));
-            throw new OptionException(CMDERR, this::usage, text);
+            throw new OptionException(CMDERR, this::showUsage, text);
         }
     }
 
@@ -823,10 +850,6 @@ public class Start extends ToolOption.Helper {
         messager.printErrorUsingKey(key, args);
     }
 
-    void warn(String key, Object... args)  {
-        messager.printWarningUsingKey(key, args);
-    }
-
     /**
      * Get the locale if specified on the command line
      * else return null and if locale option is not used
@@ -846,50 +869,4 @@ public class Start extends ToolOption.Helper {
         }
     }
 
-    /**
-     * Search the locale for specified language, specified country and
-     * specified variant.
-     */
-    private Locale searchLocale(String language, String country,
-                                String variant) {
-        for (Locale loc : Locale.getAvailableLocales()) {
-            if (loc.getLanguage().equals(language) &&
-                (country == null || loc.getCountry().equals(country)) &&
-                (variant == null || loc.getVariant().equals(variant))) {
-                return loc;
-            }
-        }
-        return null;
-    }
-
-    @Override
-    OptionHelper getOptionHelper() {
-        return new GrumpyHelper(messager) {
-            @Override
-            public String get(com.sun.tools.javac.main.Option option) {
-                return compOpts.get(option);
-            }
-
-            @Override
-            public void put(String name, String value) {
-                compOpts.put(name, value);
-            }
-
-            @Override
-            public void remove(String name) {
-                compOpts.remove(name);
-            }
-
-            @Override
-            public boolean handleFileManagerOption(com.sun.tools.javac.main.Option option, String value) {
-                fileManagerOpts.put(option, value);
-                return true;
-            }
-        };
-    }
-
-    @Override
-    String getLocalizedMessage(String msg, Object... args) {
-        return messager.getText(msg, args);
-    }
 }
