@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2015, 2020, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2015, 2019, Oracle and/or its affiliates. All rights reserved.
  * ORACLE PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
  *
  *
@@ -27,16 +27,14 @@ package jdk.security.jarsigner;
 
 import com.sun.jarsigner.ContentSigner;
 import com.sun.jarsigner.ContentSignerParameters;
-import jdk.internal.access.JavaUtilZipFileAccess;
-import jdk.internal.access.SharedSecrets;
 import sun.security.tools.PathList;
 import sun.security.tools.jarsigner.TimestampedSigner;
-import sun.security.util.Event;
 import sun.security.util.ManifestDigester;
 import sun.security.util.SignatureFileVerifier;
 import sun.security.x509.AlgorithmId;
 
 import java.io.*;
+import java.lang.reflect.InvocationTargetException;
 import java.net.SocketTimeoutException;
 import java.net.URI;
 import java.net.URL;
@@ -84,8 +82,6 @@ import java.util.zip.ZipOutputStream;
  */
 public final class JarSigner {
 
-    static final JavaUtilZipFileAccess JUZFA = SharedSecrets.getJavaUtilZipFileAccess();
-
     /**
      * A mutable builder class that can create an immutable {@code JarSigner}
      * from various signing-related parameters.
@@ -504,7 +500,6 @@ public final class JarSigner {
     private final boolean externalSF; // leave the .SF out of the PKCS7 block
     private final String altSignerPath;
     private final String altSigner;
-    private boolean extraAttrsDetected;
 
     private JarSigner(JarSigner.Builder builder) {
 
@@ -847,14 +842,14 @@ public final class JarSigner {
         signer.update(content);
         byte[] signature = signer.sign();
 
-        @SuppressWarnings("deprecation")
+        @SuppressWarnings("removal")
         ContentSigner signingMechanism = null;
         if (altSigner != null) {
             signingMechanism = loadSigningMechanism(altSigner,
                     altSignerPath);
         }
 
-        @SuppressWarnings("deprecation")
+        @SuppressWarnings("removal")
         ContentSignerParameters params =
                 new JarSignerParameters(null, tsaUrl, tSAPolicyID,
                         tSADigestAlg, signature,
@@ -948,12 +943,6 @@ public final class JarSigner {
         ze2.setTime(ze.getTime());
         ze2.setComment(ze.getComment());
         ze2.setExtra(ze.getExtra());
-        int extraAttrs = JUZFA.getExtraAttributes(ze);
-        if (!extraAttrsDetected && extraAttrs != -1) {
-            extraAttrsDetected = true;
-            Event.report(Event.ReporterCategory.ZIPFILEATTRS, "detected");
-        }
-        JUZFA.setExtraAttributes(ze2, extraAttrs);
         if (ze.getMethod() == ZipEntry.STORED) {
             ze2.setSize(ze.getSize());
             ze2.setCrc(ze.getCrc());
@@ -1070,10 +1059,15 @@ public final class JarSigner {
      * Try to load the specified signing mechanism.
      * The URL class loader is used.
      */
-    @SuppressWarnings("deprecation")
+    @SuppressWarnings("removal")
     private ContentSigner loadSigningMechanism(String signerClassName,
                                                String signerClassPath) {
 
+        // If there is no signerClassPath provided, search from here
+        if (signerClassPath == null) {
+            signerClassPath = ".";
+        }
+
         // construct class loader
         String cpString;   // make sure env.class.path defaults to dot
 
@@ -1089,10 +1083,11 @@ public final class JarSigner {
         try {
             // attempt to find signer
             Class<?> signerClass = appClassLoader.loadClass(signerClassName);
-            Object signer = signerClass.newInstance();
+            Object signer = signerClass.getDeclaredConstructor().newInstance();
             return (ContentSigner) signer;
         } catch (ClassNotFoundException|InstantiationException|
-                IllegalAccessException|ClassCastException e) {
+                IllegalAccessException|ClassCastException|
+                NoSuchMethodException| InvocationTargetException e) {
             throw new IllegalArgumentException(
                     "Invalid altSigner or altSignerPath", e);
         }
@@ -1186,7 +1181,7 @@ public final class JarSigner {
         }
 
         // Generates the PKCS#7 content of block file
-        @SuppressWarnings("deprecation")
+        @SuppressWarnings("removal")
         public byte[] generateBlock(ContentSignerParameters params,
                                     boolean externalSF,
                                     ContentSigner signingMechanism)
@@ -1204,7 +1199,7 @@ public final class JarSigner {
         }
     }
 
-    @SuppressWarnings("deprecation")
+    @SuppressWarnings("removal")
     class JarSignerParameters implements ContentSignerParameters {
 
         private String[] args;
