@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2015, 2017, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2015, 2020, Oracle and/or its affiliates. All rights reserved.
  * ORACLE PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
  *
  *
@@ -43,8 +43,6 @@ import java.nio.file.Path;
 import java.nio.file.Paths;
 import java.util.ArrayList;
 import java.util.Arrays;
-import java.util.Collection;
-import java.util.Collections;
 import java.util.Comparator;
 import java.util.Date;
 import java.util.HashMap;
@@ -88,13 +86,12 @@ public class JlinkTask {
     private static final Option<?>[] recognizedOptions = {
         new Option<JlinkTask>(false, (task, opt, arg) -> {
             task.options.help = true;
-        }, "--help", "-h"),
+        }, "--help", "-h", "-?"),
         new Option<JlinkTask>(true, (task, opt, arg) -> {
             // if used multiple times, the last one wins!
             // So, clear previous values, if any.
             task.options.modulePath.clear();
             String[] dirs = arg.split(File.pathSeparator);
-            int i = 0;
             Arrays.stream(dirs)
                   .map(Paths::get)
                   .forEach(task.options.modulePath::add);
@@ -231,7 +228,8 @@ public class JlinkTask {
         try {
             List<String> remaining = optionsHelper.handleOptions(this, args);
             if (remaining.size() > 0 && !options.suggestProviders) {
-                throw taskHelper.newBadArgs("err.orphan.arguments", toString(remaining))
+                throw taskHelper.newBadArgs("err.orphan.arguments",
+                                            remaining.stream().collect(Collectors.joining(" ")))
                                 .showUsage(true);
             }
             if (options.help) {
@@ -659,9 +657,12 @@ public class JlinkTask {
         throws BadArgs
     {
         if (args.size() > 1) {
-            throw taskHelper.newBadArgs("err.orphan.argument",
-                                        toString(args.subList(1, args.size())))
-                            .showUsage(true);
+            List<String> arguments = args.get(0).startsWith("-")
+                                        ? args
+                                        : args.subList(1, args.size());
+            throw taskHelper.newBadArgs("err.invalid.arg.for.option",
+                                        "--suggest-providers",
+                                        arguments.stream().collect(Collectors.joining(" ")));
         }
 
         if (options.bindServices) {
@@ -714,7 +715,7 @@ public class JlinkTask {
                  .forEach(names::remove);
             if (!names.isEmpty()) {
                 log.println(taskHelper.getMessage("warn.provider.notfound",
-                                                  toString(names)));
+                    names.stream().sorted().collect(Collectors.joining(","))));
             }
 
             String msg = String.format("%n%s:", taskHelper.getMessage("suggested.providers.header"));
@@ -722,11 +723,6 @@ public class JlinkTask {
         }
     }
 
-    private static String toString(Collection<String> collection) {
-        return collection.stream().sorted()
-                         .collect(Collectors.joining(","));
-    }
-
     private String getSaveOpts() {
         StringBuilder sb = new StringBuilder();
         sb.append('#').append(new Date()).append("\n");
@@ -737,36 +733,6 @@ public class JlinkTask {
         return sb.toString();
     }
 
-    private static String getBomHeader() {
-        StringBuilder sb = new StringBuilder();
-        sb.append("#").append(new Date()).append("\n");
-        sb.append("#Please DO NOT Modify this file").append("\n");
-        return sb.toString();
-    }
-
-    private String genBOMContent() throws IOException {
-        StringBuilder sb = new StringBuilder();
-        sb.append(getBomHeader());
-        StringBuilder command = new StringBuilder();
-        for (String c : optionsHelper.getInputCommand()) {
-            command.append(c).append(" ");
-        }
-        sb.append("command").append(" = ").append(command);
-        sb.append("\n");
-
-        return sb.toString();
-    }
-
-    private static String genBOMContent(JlinkConfiguration config,
-            PluginsConfiguration plugins)
-            throws IOException {
-        StringBuilder sb = new StringBuilder();
-        sb.append(getBomHeader());
-        sb.append(config);
-        sb.append(plugins);
-        return sb.toString();
-    }
-
     private static class ImageHelper implements ImageProvider {
         final ByteOrder order;
         final Path packagedModulesPath;
