@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2014, 2019, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2014, 2020, Oracle and/or its affiliates. All rights reserved.
  * ORACLE PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
  *
  *
@@ -26,8 +26,8 @@
 /**
  * Defines the foundational APIs of the Java SE Platform.
  *
- * <dl>
- * <dt class="simpleTagLabel" style="font-family:'DejaVu Sans', Arial, Helvetica, sans serif">Providers:</dt>
+ * <dl class="notes">
+ * <dt>Providers:</dt>
  * <dd> The JDK implementation of this module provides an implementation of
  *      the {@index jrt jrt} {@linkplain java.nio.file.spi.FileSystemProvider
  *      file system provider} to enumerate and read the class and resource
@@ -75,8 +75,6 @@
  * @since 9
  */
 module java.base {
-    // source file: file:///scratch/opt/mach5/mesos/work_dir/slaves/fcf4c0c4-d73e-4321-860c-6613427db92b-S2589/frameworks/1735e8a2-a1db-478c-8104-60c8b0af87dd-0196/executors/7e0f51a1-6397-462e-a00e-f30e1df5387c/runs/2786f1af-1296-422d-8542-d335b44e3efc/workspace/open/src/java.base/share/classes/module-info.java
-    //              file:///scratch/opt/mach5/mesos/work_dir/slaves/fcf4c0c4-d73e-4321-860c-6613427db92b-S2589/frameworks/1735e8a2-a1db-478c-8104-60c8b0af87dd-0196/executors/7e0f51a1-6397-462e-a00e-f30e1df5387c/runs/2786f1af-1296-422d-8542-d335b44e3efc/workspace/open/src/java.base/unix/classes/module-info.java.extra
     exports java.io;
     exports java.lang;
     exports java.lang.annotation;
@@ -85,6 +83,7 @@ module java.base {
     exports java.lang.module;
     exports java.lang.ref;
     exports java.lang.reflect;
+    exports java.lang.runtime;
     exports java.math;
     exports java.net;
     exports java.net.spi;
@@ -97,7 +96,6 @@ module java.base {
     exports java.nio.file.attribute;
     exports java.nio.file.spi;
     exports java.security;
-    exports java.security.acl;
     exports java.security.cert;
     exports java.security.interfaces;
     exports java.security.spec;
@@ -130,15 +128,21 @@ module java.base {
     exports javax.security.auth.x500;
     exports javax.security.cert;
     exports com.sun.security.ntlm to java.security.sasl;
-    exports jdk.internal to jdk.jfr;
+    exports jdk.internal to
+        java.compiler,
+        jdk.compiler,
+        jdk.jfr,
+        jdk.jshell;
     exports jdk.internal.access to
         java.desktop,
         java.logging,
         java.management,
         java.naming,
         java.rmi,
+        jdk.incubator.foreign,
         jdk.jlink,
         jdk.net;
+    exports jdk.internal.access.foreign to jdk.incubator.foreign;
     exports jdk.internal.event to jdk.jfr;
     exports jdk.internal.jimage to jdk.jlink;
     exports jdk.internal.jimage.decompressor to jdk.jlink;
@@ -147,7 +151,8 @@ module java.base {
         jdk.jlink;
     exports jdk.internal.loader to
         java.instrument,
-        java.logging;
+        java.logging,
+        java.naming;
     exports jdk.internal.logger to java.logging;
     exports jdk.internal.misc to
         java.desktop,
@@ -161,39 +166,35 @@ module java.base {
         jdk.attach,
         jdk.charsets,
         jdk.compiler,
+        jdk.incubator.foreign,
         jdk.internal.vm.ci,
         jdk.jfr,
         jdk.jshell,
-        jdk.scripting.nashorn,
-        jdk.scripting.nashorn.shell,
+        jdk.nio.mapmode,
         jdk.unsupported;
     exports jdk.internal.module to
         java.instrument,
         java.management.rmi,
+        jdk.incubator.jpackage,
         jdk.jartool,
         jdk.jfr,
         jdk.jlink;
     exports jdk.internal.org.objectweb.asm to
         jdk.jartool,
         jdk.jfr,
-        jdk.jlink,
-        jdk.scripting.nashorn;
-    exports jdk.internal.org.objectweb.asm.commons to
-        jdk.jfr,
-        jdk.scripting.nashorn;
-    exports jdk.internal.org.objectweb.asm.signature to jdk.scripting.nashorn;
+        jdk.jlink;
+    exports jdk.internal.org.objectweb.asm.commons to jdk.jfr;
     exports jdk.internal.org.objectweb.asm.tree to
         jdk.jfr,
         jdk.jlink;
-    exports jdk.internal.org.objectweb.asm.util to
-        jdk.jfr,
-        jdk.scripting.nashorn;
+    exports jdk.internal.org.objectweb.asm.util to jdk.jfr;
     exports jdk.internal.org.xml.sax to jdk.jfr;
     exports jdk.internal.org.xml.sax.helpers to jdk.jfr;
     exports jdk.internal.perf to
         java.management,
         jdk.internal.jvmstat,
         jdk.management.agent;
+    exports jdk.internal.platform to jdk.management;
     exports jdk.internal.ref to java.desktop;
     exports jdk.internal.reflect to
         java.logging,
@@ -201,8 +202,8 @@ module java.base {
         java.sql.rowset,
         jdk.dynalink,
         jdk.internal.vm.ci,
-        jdk.scripting.nashorn,
         jdk.unsupported;
+    exports jdk.internal.util to jdk.incubator.foreign;
     exports jdk.internal.util.jar to jdk.jartool;
     exports jdk.internal.util.xml to jdk.jfr;
     exports jdk.internal.util.xml.impl to jdk.jfr;
@@ -210,8 +211,12 @@ module java.base {
         jdk.internal.jvmstat,
         jdk.management.agent;
     exports jdk.internal.vm.annotation to
+        jdk.incubator.foreign,
         jdk.internal.vm.ci,
         jdk.unsupported;
+    exports sun.invoke.util to
+        jdk.compiler,
+        jdk.incubator.foreign;
     exports sun.net to
         java.net.http,
         jdk.naming.dns;
@@ -222,7 +227,8 @@ module java.base {
     exports sun.net.util to
         java.desktop,
         java.net.http,
-        jdk.jconsole;
+        jdk.jconsole,
+        jdk.sctp;
     exports sun.net.www to
         java.net.http,
         jdk.jartool;
@@ -230,6 +236,7 @@ module java.base {
     exports sun.nio.ch to
         java.management,
         jdk.crypto.cryptoki,
+        jdk.incubator.foreign,
         jdk.net,
         jdk.sctp;
     exports sun.nio.cs to
@@ -246,7 +253,9 @@ module java.base {
         java.sql.rowset;
     exports sun.security.action to
         java.desktop,
-        java.security.jgss;
+        java.security.jgss,
+        jdk.crypto.ec,
+        jdk.incubator.foreign;
     exports sun.security.internal.interfaces to jdk.crypto.cryptoki;
     exports sun.security.internal.spec to jdk.crypto.cryptoki;
     exports sun.security.jca to
@@ -261,6 +270,7 @@ module java.base {
         java.rmi,
         java.security.jgss,
         jdk.crypto.cryptoki,
+        jdk.crypto.ec,
         jdk.security.auth;
     exports sun.security.provider.certpath to java.naming;
     exports sun.security.rsa to jdk.crypto.cryptoki;
