@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2011, 2011, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2011, 2020, Oracle and/or its affiliates. All rights reserved.
  * ORACLE PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
  *
  *
@@ -26,5 +26,5 @@ package org.graalvm.compiler.asm;
 
 public class AsmOptions {
 
-    public static int InitialCodeBufferSize = 232;
+    public static final int InitialCodeBufferSize = 232;
 }
