@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2014, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2014, 2020, Oracle and/or its affiliates. All rights reserved.
  * ORACLE PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
  *
  *
@@ -24,10 +24,12 @@
 
 package org.graalvm.compiler.core.common;
 
-import static org.graalvm.compiler.core.common.UnsafeAccess.UNSAFE;
+import static org.graalvm.compiler.serviceprovider.GraalUnsafeAccess.getUnsafe;
 
 import java.util.ArrayList;
+import java.util.Arrays;
 import java.util.Collections;
+import java.util.List;
 
 import org.graalvm.compiler.debug.GraalError;
 
@@ -38,6 +40,9 @@ import sun.misc.Unsafe;
  */
 public class Fields {
 
+    private static final Unsafe UNSAFE = getUnsafe();
+    private static final Fields EMPTY_FIELDS = new Fields(Collections.emptyList());
+
     /**
      * Offsets used with {@link Unsafe} to access the fields.
      */
@@ -58,10 +63,10 @@ public class Fields {
     public static Fields forClass(Class<?> clazz, Class<?> endClazz, boolean includeTransient, FieldsScanner.CalcOffset calcOffset) {
         FieldsScanner scanner = new FieldsScanner(calcOffset == null ? new FieldsScanner.DefaultCalcOffset() : calcOffset);
         scanner.scan(clazz, endClazz, includeTransient);
-        return new Fields(scanner.data);
+        return create(scanner.data);
     }
 
-    public Fields(ArrayList<? extends FieldsScanner.FieldInfo> fields) {
+    protected Fields(List<? extends FieldsScanner.FieldInfo> fields) {
         Collections.sort(fields);
         this.offsets = new long[fields.size()];
         this.names = new String[offsets.length];
@@ -77,6 +82,13 @@ public class Fields {
         }
     }
 
+    public static Fields create(ArrayList<? extends FieldsScanner.FieldInfo> fields) {
+        if (fields.size() == 0) {
+            return EMPTY_FIELDS;
+        }
+        return new Fields(fields);
+    }
+
     /**
      * Gets the number of fields represented by this object.
      */
@@ -144,11 +156,42 @@ public class Fields {
                 }
             } else {
                 Object obj = UNSAFE.getObject(from, offset);
+                if (obj != null && type.isArray()) {
+                    if (type.getComponentType().isPrimitive()) {
+                        obj = copyObjectAsArray(obj);
+                    } else {
+                        obj = ((Object[]) obj).clone();
+                    }
+                }
                 UNSAFE.putObject(to, offset, trans == null ? obj : trans.apply(index, obj));
             }
         }
     }
 
+    private static Object copyObjectAsArray(Object obj) {
+        Object objCopy;
+        if (obj instanceof int[]) {
+            objCopy = Arrays.copyOf((int[]) obj, ((int[]) obj).length);
+        } else if (obj instanceof short[]) {
+            objCopy = Arrays.copyOf((short[]) obj, ((short[]) obj).length);
+        } else if (obj instanceof long[]) {
+            objCopy = Arrays.copyOf((long[]) obj, ((long[]) obj).length);
+        } else if (obj instanceof float[]) {
+            objCopy = Arrays.copyOf((float[]) obj, ((float[]) obj).length);
+        } else if (obj instanceof double[]) {
+            objCopy = Arrays.copyOf((double[]) obj, ((double[]) obj).length);
+        } else if (obj instanceof boolean[]) {
+            objCopy = Arrays.copyOf((boolean[]) obj, ((boolean[]) obj).length);
+        } else if (obj instanceof byte[]) {
+            objCopy = Arrays.copyOf((byte[]) obj, ((byte[]) obj).length);
+        } else if (obj instanceof char[]) {
+            objCopy = Arrays.copyOf((char[]) obj, ((char[]) obj).length);
+        } else {
+            throw GraalError.shouldNotReachHere();
+        }
+        return objCopy;
+    }
+
     /**
      * Gets the value of a field for a given object.
      *
@@ -326,7 +369,7 @@ public class Fields {
 
     public void appendFields(StringBuilder sb) {
         for (int i = 0; i < offsets.length; i++) {
-            sb.append(i == 0 ? "" : ", ").append(getName(i)).append('@').append(offsets[i]);
+            sb.append(i == 0 ? "" : ", ").append(getDeclaringClass(i).getSimpleName()).append('.').append(getName(i)).append('@').append(offsets[i]);
         }
     }
 
