@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2013, 2014, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2013, 2020, Oracle and/or its affiliates. All rights reserved.
  * ORACLE PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
  *
  *
@@ -42,5 +42,5 @@ public interface CodeGenProviders {
 
     ConstantReflectionProvider getConstantReflection();
 
-    ArrayOffsetProvider getArrayOffsetProvider();
+    MetaAccessExtensionProvider getMetaAccessExtensionProvider();
 }
