@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2017, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2017, 2020, Oracle and/or its affiliates. All rights reserved.
  * ORACLE PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
  *
  *
@@ -56,8 +56,8 @@ public final class NodeEncodingTest {
     }
 
     @Test
-    public void defaultVersionTheNodeIsntDumpedWithItsID() throws Exception {
-        runTheNodeIsntDumpedWithItsID(false);
+    public void defaultVersionTheNodeIsDumpedWithItsID() throws Exception {
+        runTheNodeIsTreatedPoolEntry(false);
     }
 
     private void runTheNodeIsntDumpedWithItsID(boolean explicitVersion) throws Exception {
