@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2011, 2018, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2011, 2020, Oracle and/or its affiliates. All rights reserved.
  * ORACLE PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
  *
  *
@@ -24,14 +24,15 @@
 
 package org.graalvm.compiler.graph.spi;
 
+import org.graalvm.compiler.core.common.spi.ConstantFieldProvider;
+import org.graalvm.compiler.core.common.spi.MetaAccessExtensionProvider;
+import org.graalvm.compiler.graph.Node;
+import org.graalvm.compiler.options.OptionValues;
+
 import jdk.vm.ci.meta.Assumptions;
 import jdk.vm.ci.meta.ConstantReflectionProvider;
 import jdk.vm.ci.meta.MetaAccessProvider;
 
-import org.graalvm.compiler.core.common.spi.ConstantFieldProvider;
-import org.graalvm.compiler.graph.Node;
-import org.graalvm.compiler.options.OptionValues;
-
 public interface CanonicalizerTool {
 
     Assumptions getAssumptions();
@@ -42,6 +43,8 @@ public interface CanonicalizerTool {
 
     ConstantFieldProvider getConstantFieldProvider();
 
+    MetaAccessExtensionProvider getMetaAccessExtensionProvider();
+
     boolean canonicalizeReads();
 
     /**
