@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2013, 2016, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2013, 2020, Oracle and/or its affiliates. All rights reserved.
  * ORACLE PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
  *
  *
@@ -28,6 +28,7 @@ import static jdk.vm.ci.hotspot.aarch64.AArch64HotSpotRegisterConfig.inlineCache
 
 import org.graalvm.compiler.asm.aarch64.AArch64MacroAssembler;
 import org.graalvm.compiler.hotspot.GraalHotSpotVMConfig;
+import org.graalvm.compiler.hotspot.HotSpotMarkId;
 import org.graalvm.compiler.lir.LIRFrameState;
 import org.graalvm.compiler.lir.LIRInstructionClass;
 import org.graalvm.compiler.lir.Opcode;
@@ -58,14 +59,17 @@ final class AArch64HotSpotDirectStaticCallOp extends DirectCallOp {
     }
 
     @Override
+    @SuppressWarnings("try")
     public void emitCode(CompilationResultBuilder crb, AArch64MacroAssembler masm) {
-        // The mark for an invocation that uses an inline cache must be placed at the
-        // instruction that loads the Klass from the inline cache.
-        // For the first invocation this is set to a bitpattern that is guaranteed to never be a
-        // valid object which causes the called function to call a handler that installs the
-        // correct inline cache value here.
-        crb.recordMark(invokeKind == InvokeKind.Static ? config.MARKID_INVOKESTATIC : config.MARKID_INVOKESPECIAL);
-        masm.movNativeAddress(inlineCacheRegister, config.nonOopBits);
-        super.emitCode(crb, masm);
+        try (CompilationResultBuilder.CallContext callContext = crb.openCallContext(invokeKind.isDirect())) {
+            // The mark for an invocation that uses an inline cache must be placed at the
+            // instruction that loads the Klass from the inline cache.
+            // For the first invocation this is set to a bitpattern that is guaranteed to never be a
+            // valid object which causes the called function to call a handler that installs the
+            // correct inline cache value here.
+            crb.recordMark(invokeKind == InvokeKind.Static ? HotSpotMarkId.INVOKESTATIC : HotSpotMarkId.INVOKESPECIAL);
+            masm.movNativeAddress(inlineCacheRegister, config.nonOopBits);
+            super.emitCode(crb, masm);
+        }
     }
 }
