@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2013, 2018, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2013, 2020, Oracle and/or its affiliates. All rights reserved.
  * ORACLE PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
  *
  *
@@ -53,8 +53,8 @@ import jdk.vm.ci.code.RegisterValue;
  */
 abstract class AArch64HotSpotEpilogueOp extends AArch64BlockEndOp {
 
-    private final GraalHotSpotVMConfig config;
-    private final Register thread;
+    protected final GraalHotSpotVMConfig config;
+    protected final Register thread;
 
     protected AArch64HotSpotEpilogueOp(LIRInstructionClass<? extends AArch64HotSpotEpilogueOp> c, GraalHotSpotVMConfig config, Register thread) {
         super(c);
