@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2013, 2015, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2013, 2020, Oracle and/or its affiliates. All rights reserved.
  * ORACLE PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
  *
  *
@@ -24,19 +24,20 @@
 
 package org.graalvm.compiler.hotspot.amd64;
 
+import static jdk.vm.ci.code.ValueUtil.asRegister;
+import static jdk.vm.ci.code.ValueUtil.isRegister;
+import static jdk.vm.ci.code.ValueUtil.isStackSlot;
 import static org.graalvm.compiler.core.common.GraalOptions.GeneratePIC;
 import static org.graalvm.compiler.lir.LIRInstruction.OperandFlag.HINT;
 import static org.graalvm.compiler.lir.LIRInstruction.OperandFlag.REG;
 import static org.graalvm.compiler.lir.LIRInstruction.OperandFlag.STACK;
-import static jdk.vm.ci.code.ValueUtil.asRegister;
-import static jdk.vm.ci.code.ValueUtil.isRegister;
-import static jdk.vm.ci.code.ValueUtil.isStackSlot;
 
 import org.graalvm.compiler.asm.amd64.AMD64Address;
-import org.graalvm.compiler.core.common.CompressEncoding;
 import org.graalvm.compiler.asm.amd64.AMD64MacroAssembler;
+import org.graalvm.compiler.core.common.CompressEncoding;
 import org.graalvm.compiler.debug.GraalError;
 import org.graalvm.compiler.hotspot.GraalHotSpotVMConfig;
+import org.graalvm.compiler.hotspot.HotSpotMarkId;
 import org.graalvm.compiler.lir.LIRInstructionClass;
 import org.graalvm.compiler.lir.StandardOp.LoadConstantOp;
 import org.graalvm.compiler.lir.amd64.AMD64LIRInstruction;
@@ -113,18 +114,16 @@ public class AMD64HotSpotMove {
         public static final LIRInstructionClass<BaseMove> TYPE = LIRInstructionClass.create(BaseMove.class);
 
         @Def({REG, HINT}) protected AllocatableValue result;
-        private final GraalHotSpotVMConfig config;
 
-        public BaseMove(AllocatableValue result, GraalHotSpotVMConfig config) {
+        public BaseMove(AllocatableValue result) {
             super(TYPE);
             this.result = result;
-            this.config = config;
         }
 
         @Override
         public void emitCode(CompilationResultBuilder crb, AMD64MacroAssembler masm) {
             masm.movq(asRegister(result), masm.getPlaceholder(-1));
-            crb.recordMark(config.MARKID_NARROW_KLASS_BASE_ADDRESS);
+            crb.recordMark(HotSpotMarkId.NARROW_KLASS_BASE_ADDRESS);
         }
 
     }
@@ -187,7 +186,7 @@ public class AMD64HotSpotMove {
         if (pic || encoding.hasBase()) {
             if (pic) {
                 masm.movq(scratch, masm.getPlaceholder(-1));
-                crb.recordMark(config.MARKID_NARROW_KLASS_BASE_ADDRESS);
+                crb.recordMark(HotSpotMarkId.NARROW_KLASS_BASE_ADDRESS);
             } else {
                 assert encoding.getBase() != 0;
                 masm.movq(scratch, encoding.getBase());
