@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2018, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2018, 2020, Oracle and/or its affiliates. All rights reserved.
  * ORACLE PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
  *
  *
@@ -25,16 +25,19 @@
 package org.graalvm.compiler.hotspot.test;
 
 import java.io.IOException;
-import java.util.concurrent.locks.ReentrantLock;
 import java.util.List;
+import java.util.concurrent.locks.ReentrantLock;
 
 import org.graalvm.compiler.test.SubprocessUtil;
 import org.graalvm.compiler.test.SubprocessUtil.Subprocess;
-
 import org.junit.Assume;
 import org.junit.Before;
 import org.junit.Test;
 
+/**
+ * Tests the ReservedStackAccess annotation. This test is derived from
+ * https://github.com/openjdk/jdk/blob/master/test/hotspot/jtreg/runtime/ReservedStack/ReservedStackTest.java
+ */
 public class ReservedStackAccessTest extends HotSpotGraalCompilerTest {
     @Before
     public void check() {
@@ -68,11 +71,15 @@ public class ReservedStackAccessTest extends HotSpotGraalCompilerTest {
 
     @Test
     public void run() throws IOException, InterruptedException {
+        Assume.assumeFalse("GR-19833", runtime().getVMConfig().osName.equals("windows"));
         Assume.assumeTrue(runtime().getVMConfig().enableStackReservedZoneAddress != 0);
         List<String> vmArgs = SubprocessUtil.withoutDebuggerArguments(SubprocessUtil.getVMCommandLine());
         vmArgs.add("-XX:+UseJVMCICompiler");
         vmArgs.add("-Dgraal.Inline=false");
+        vmArgs.add("-XX:MaxInlineLevel=2");
+        vmArgs.add("-XX:CompileCommand=exclude,java/util/concurrent/locks/ReentrantLock,lock");
         vmArgs.add("-XX:CompileCommand=exclude,java/util/concurrent/locks/AbstractOwnableSynchronizer.setExclusiveOwnerThread");
+        vmArgs.add(SubprocessUtil.PACKAGE_OPENING_OPTIONS);
 
         // Avoid SOE in HotSpotJVMCIRuntime.adjustCompilationLevel
         vmArgs.add("-Dgraal.CompileGraalWithC1Only=false");
