@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2011, 2018, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2011, 2020, Oracle and/or its affiliates. All rights reserved.
  * ORACLE PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
  *
  *
@@ -28,6 +28,7 @@ import static jdk.vm.ci.common.InitTimer.timer;
 import static jdk.vm.ci.hotspot.HotSpotJVMCIRuntime.runtime;
 import static org.graalvm.compiler.core.common.GraalOptions.GeneratePIC;
 import static org.graalvm.compiler.core.common.GraalOptions.HotSpotPrintInlining;
+import static org.graalvm.compiler.hotspot.GraalHotSpotVMConfigAccess.JDK;
 
 import java.io.PrintStream;
 import java.util.ArrayList;
@@ -35,6 +36,7 @@ import java.util.EnumMap;
 import java.util.List;
 import java.util.Map;
 import java.util.concurrent.atomic.AtomicReference;
+import java.util.function.Supplier;
 
 import jdk.internal.vm.compiler.collections.EconomicMap;
 import jdk.internal.vm.compiler.collections.EconomicSet;
@@ -44,10 +46,14 @@ import org.graalvm.compiler.api.replacements.SnippetReflectionProvider;
 import org.graalvm.compiler.api.runtime.GraalRuntime;
 import org.graalvm.compiler.core.CompilationWrapper.ExceptionAction;
 import org.graalvm.compiler.core.common.CompilationIdentifier;
+import org.graalvm.compiler.core.common.CompilationListenerProfiler;
+import org.graalvm.compiler.core.common.CompilerProfiler;
 import org.graalvm.compiler.core.common.GraalOptions;
 import org.graalvm.compiler.core.common.spi.ForeignCallsProvider;
 import org.graalvm.compiler.core.target.Backend;
+import org.graalvm.compiler.debug.Assertions;
 import org.graalvm.compiler.debug.DebugContext;
+import org.graalvm.compiler.debug.DebugContext.Builder;
 import org.graalvm.compiler.debug.DebugContext.Description;
 import org.graalvm.compiler.debug.DebugHandlersFactory;
 import org.graalvm.compiler.debug.DebugOptions;
@@ -98,6 +104,12 @@ public final class HotSpotGraalRuntime implements HotSpotGraalRuntimeProvider {
 
     private static final boolean IS_AOT = Boolean.parseBoolean(Services.getSavedProperties().get("com.oracle.graalvm.isaot"));
 
+    /**
+     * A factory for a {@link HotSpotGraalManagementRegistration} injected by
+     * {@code Target_org_graalvm_compiler_hotspot_HotSpotGraalRuntime}.
+     */
+    private static final Supplier<HotSpotGraalManagementRegistration> AOT_INJECTED_MANAGEMENT = null;
+
     private static boolean checkArrayIndexScaleInvariants(MetaAccessProvider metaAccess) {
         assert metaAccess.getArrayIndexScale(JavaKind.Byte) == 1;
         assert metaAccess.getArrayIndexScale(JavaKind.Boolean) == 1;
@@ -113,6 +125,11 @@ public final class HotSpotGraalRuntime implements HotSpotGraalRuntimeProvider {
     private final String runtimeName;
     private final String compilerConfigurationName;
     private final HotSpotBackend hostBackend;
+
+    public GlobalMetrics getMetricValues() {
+        return metricValues;
+    }
+
     private final GlobalMetrics metricValues = new GlobalMetrics();
     private final List<SnippetCounter.Group> snippetCounterGroups;
     private final HotSpotGC garbageCollector;
@@ -129,11 +146,11 @@ public final class HotSpotGraalRuntime implements HotSpotGraalRuntimeProvider {
      */
     private AtomicReference<OptionValues> optionsRef = new AtomicReference<>();
 
-    private final HotSpotGraalCompiler compiler;
-
     private final DiagnosticsOutputDirectory outputDirectory;
     private final Map<ExceptionAction, Integer> compilationProblemsPerAction;
 
+    private final CompilerProfiler compilerProfiler;
+
     /**
      * @param nameQualifier a qualifier to be added to this runtime's {@linkplain #getName() name}
      * @param compilerConfigurationFactory factory for the compiler configuration
@@ -161,13 +178,19 @@ public final class HotSpotGraalRuntime implements HotSpotGraalRuntimeProvider {
         CompilerConfiguration compilerConfiguration = compilerConfigurationFactory.createCompilerConfiguration();
         compilerConfigurationName = compilerConfigurationFactory.getName();
 
-        compiler = new HotSpotGraalCompiler(jvmciRuntime, this, options);
         if (IS_AOT) {
-            management = null;
+            management = AOT_INJECTED_MANAGEMENT == null ? null : AOT_INJECTED_MANAGEMENT.get();
         } else {
             management = GraalServices.loadSingle(HotSpotGraalManagementRegistration.class, false);
-            if (management != null) {
-                management.initialize(this);
+        }
+        if (management != null) {
+            try {
+                management.initialize(this, config);
+            } catch (ThreadDeath td) {
+                throw td;
+            } catch (Throwable error) {
+                TTY.println("Cannot install GraalVM MBean due to " + error.getMessage());
+                management = null;
             }
         }
 
@@ -216,6 +239,8 @@ public final class HotSpotGraalRuntime implements HotSpotGraalRuntimeProvider {
 
         runtimeStartTime = System.nanoTime();
         bootstrapJVMCI = config.getFlag("BootstrapJVMCI", Boolean.class);
+
+        this.compilerProfiler = GraalServices.loadSingle(CompilerProfiler.class, false);
     }
 
     /**
@@ -223,47 +248,70 @@ public final class HotSpotGraalRuntime implements HotSpotGraalRuntimeProvider {
      */
     public enum HotSpotGC {
         // Supported GCs
-        Serial(true, "UseSerialGC"),
-        Parallel(true, "UseParallelGC", "UseParallelOldGC", "UseParNewGC"),
-        CMS(true, "UseConcMarkSweepGC"),
-        G1(true, "UseG1GC"),
+        Serial(true, "UseSerialGC", true),
+        Parallel(true, "UseParallelGC", true, "UseParallelOldGC", JDK < 15, "UseParNewGC", JDK < 10),
+        CMS(true, "UseConcMarkSweepGC", JDK < 14),
+        G1(true, "UseG1GC", true),
 
         // Unsupported GCs
-        Epsilon(false, "UseEpsilonGC"),
-        Z(false, "UseZGC");
+        Epsilon(false, "UseEpsilonGC", JDK >= 11),
+        Z(false, "UseZGC", JDK >= 11);
 
-        HotSpotGC(boolean supported, String... flags) {
+        HotSpotGC(boolean supported,
+                        String flag1, boolean expectFlagPresent1,
+                        String flag2, boolean expectFlagPresent2,
+                        String flag3, boolean expectFlagPresent3) {
             this.supported = supported;
-            this.flags = flags;
+            this.expectFlagsPresent = new boolean[]{expectFlagPresent1, expectFlagPresent2, expectFlagPresent3};
+            this.flags = new String[]{flag1, flag2, flag3};
+        }
+
+        HotSpotGC(boolean supported, String flag, boolean expectFlagPresent) {
+            this.supported = supported;
+            this.expectFlagsPresent = new boolean[]{expectFlagPresent};
+            this.flags = new String[]{flag};
         }
 
         final boolean supported;
+        final boolean[] expectFlagsPresent;
         private final String[] flags;
 
         public boolean isSelected(GraalHotSpotVMConfig config) {
-            for (String flag : flags) {
+            boolean selected = false;
+            for (int i = 0; i < flags.length; i++) {
                 final boolean notPresent = false;
-                if (config.getFlag(flag, Boolean.class, notPresent)) {
-                    return true;
+                if (config.getFlag(flags[i], Boolean.class, notPresent, expectFlagsPresent[i])) {
+                    selected = true;
+                    if (!Assertions.assertionsEnabled()) {
+                        // When asserting, check that isSelected works for all flag names
+                        break;
+                    }
                 }
             }
-            return false;
+            return selected;
         }
-
     }
 
     private HotSpotGC getSelectedGC() throws GraalError {
+        HotSpotGC selected = null;
         for (HotSpotGC gc : HotSpotGC.values()) {
             if (gc.isSelected(config)) {
                 if (!gc.supported) {
                     throw new GraalError(gc.name() + " garbage collector is not supported by Graal");
                 }
-                return gc;
+                selected = gc;
+                if (!Assertions.assertionsEnabled()) {
+                    // When asserting, check that isSelected works for all HotSpotGC values
+                    break;
+                }
             }
         }
-        // As of JDK 9, exactly one GC flag is guaranteed to be selected.
-        // On JDK 8, the default GC is Serial when no GC flag is true.
-        return HotSpotGC.Serial;
+        if (selected == null) {
+            // As of JDK 9, exactly one GC flag is guaranteed to be selected.
+            // On JDK 8, the default GC is Serial when no GC flag is true.
+            selected = HotSpotGC.Serial;
+        }
+        return selected;
     }
 
     private HotSpotBackend registerBackend(HotSpotBackend backend) {
@@ -290,19 +338,31 @@ public final class HotSpotGraalRuntime implements HotSpotGraalRuntimeProvider {
                 HotSpotResolvedObjectType type = ((HotSpotResolvedJavaMethod) compilable).getDeclaringClass();
                 if (type instanceof HotSpotResolvedJavaType) {
                     Class<?> clazz = runtime().getMirror(type);
-                    try {
-                        ClassLoader cl = clazz.getClassLoader();
-                        if (cl != null) {
-                            loaders.add(cl);
+                    if (clazz != null) {
+                        try {
+                            ClassLoader cl = clazz.getClassLoader();
+                            if (cl != null) {
+                                loaders.add(cl);
+                            }
+                        } catch (SecurityException e) {
+                            // This loader can obviously not be used for resolving class names
                         }
-                    } catch (SecurityException e) {
-                        // This loader can obviously not be used for resolving class names
                     }
                 }
             }
         }
+
         Description description = new Description(compilable, compilationId.toString(CompilationIdentifier.Verbosity.ID));
-        return DebugContext.create(compilationOptions, description, metricValues, logStream, factories);
+        Builder builder = new Builder(compilationOptions, factories).//
+                        globalMetrics(metricValues).//
+                        description(description).//
+                        logStream(logStream);
+        if (compilerProfiler != null) {
+            int compileId = ((HotSpotCompilationIdentifier) compilationId).getRequest().getId();
+            builder.compilationListener(new CompilationListenerProfiler(compilerProfiler, compileId));
+        }
+        return builder.build();
+
     }
 
     @Override
@@ -346,6 +406,7 @@ public final class HotSpotGraalRuntime implements HotSpotGraalRuntimeProvider {
         return null;
     }
 
+    @Override
     public HotSpotGC getGarbageCollector() {
         return garbageCollector;
     }
@@ -367,7 +428,16 @@ public final class HotSpotGraalRuntime implements HotSpotGraalRuntimeProvider {
     }
 
     private long runtimeStartTime;
-    private boolean shutdown;
+
+    /**
+     * Called from compiler threads to check whether to bail out of a compilation.
+     */
+    private volatile boolean shutdown;
+
+    /**
+     * Shutdown hooks that should be run on the same thread doing the shutdown.
+     */
+    private List<Runnable> shutdownHooks = new ArrayList<>();
 
     /**
      * Take action related to entering a new execution phase.
@@ -380,8 +450,29 @@ public final class HotSpotGraalRuntime implements HotSpotGraalRuntimeProvider {
         }
     }
 
-    void shutdown() {
+    /**
+     * Adds a {@link Runnable} that will be run when this runtime is {@link #shutdown()}. The
+     * runnable will be run on the same thread doing the shutdown. All the advice for regular
+     * {@linkplain Runtime#addShutdownHook(Thread) shutdown hooks} also applies here but even more
+     * so since the hook runs on the shutdown thread.
+     */
+    public synchronized void addShutdownHook(Runnable hook) {
+        if (!shutdown) {
+            shutdownHooks.add(hook);
+        }
+    }
+
+    synchronized void shutdown() {
         shutdown = true;
+
+        for (Runnable r : shutdownHooks) {
+            try {
+                r.run();
+            } catch (Throwable e) {
+                e.printStackTrace(TTY.out);
+            }
+        }
+
         metricValues.print(optionsRef.get());
 
         phaseTransition("final");
@@ -394,6 +485,16 @@ public final class HotSpotGraalRuntime implements HotSpotGraalRuntimeProvider {
         BenchmarkCounters.shutdown(runtime(), optionsRef.get(), runtimeStartTime);
 
         outputDirectory.close();
+
+        shutdownLibGraal();
+    }
+
+    /**
+     * Substituted by
+     * {@code com.oracle.svm.graal.hotspot.libgraal.Target_org_graalvm_compiler_hotspot_HotSpotGraalRuntime}
+     * to call {@code org.graalvm.nativeimage.VMRuntime.shutdown()}.
+     */
+    private static void shutdownLibGraal() {
     }
 
     void clearMetrics() {
@@ -429,7 +530,7 @@ public final class HotSpotGraalRuntime implements HotSpotGraalRuntimeProvider {
 
     // ------- Management interface ---------
 
-    private final HotSpotGraalManagementRegistration management;
+    private HotSpotGraalManagementRegistration management;
 
     /**
      * @returns the management object for this runtime or {@code null}
@@ -580,6 +681,7 @@ public final class HotSpotGraalRuntime implements HotSpotGraalRuntimeProvider {
         extra.put(DebugOptions.PrintGraphHost, host);
         extra.put(DebugOptions.PrintGraphPort, port);
         OptionValues compileOptions = new OptionValues(getOptions(), extra);
+        HotSpotGraalCompiler compiler = (HotSpotGraalCompiler) runtime().getCompiler();
         compiler.compileMethod(new HotSpotCompilationRequest(hotSpotMethod, -1, 0L), false, compileOptions);
     }
 
