@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2014, 2016, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2014, 2020, Oracle and/or its affiliates. All rights reserved.
  * ORACLE PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
  *
  *
@@ -24,6 +24,11 @@
 
 package org.graalvm.compiler.hotspot.replacements;
 
+import static org.graalvm.compiler.api.directives.GraalDirectives.SLOWPATH_PROBABILITY;
+import static org.graalvm.compiler.api.directives.GraalDirectives.injectBranchProbability;
+import static org.graalvm.compiler.hotspot.meta.HotSpotForeignCallDescriptor.Reexecutability.REEXECUTABLE;
+import static org.graalvm.compiler.hotspot.meta.HotSpotForeignCallDescriptor.Transition.LEAF;
+import static org.graalvm.compiler.hotspot.meta.HotSpotForeignCallsProviderImpl.NO_LOCATIONS;
 import static org.graalvm.compiler.replacements.SnippetTemplate.DEFAULT_REPLACER;
 import static org.graalvm.compiler.replacements.nodes.CStringConstant.cstring;
 
@@ -33,7 +38,9 @@ import org.graalvm.compiler.core.common.spi.ForeignCallDescriptor;
 import org.graalvm.compiler.debug.DebugHandlersFactory;
 import org.graalvm.compiler.graph.Node.ConstantNodeParameter;
 import org.graalvm.compiler.graph.Node.NodeIntrinsic;
+import org.graalvm.compiler.hotspot.meta.HotSpotForeignCallDescriptor;
 import org.graalvm.compiler.hotspot.meta.HotSpotProviders;
+import org.graalvm.compiler.hotspot.nodes.StubForeignCallNode;
 import org.graalvm.compiler.hotspot.nodes.StubStartNode;
 import org.graalvm.compiler.nodes.StructuredGraph;
 import org.graalvm.compiler.nodes.extended.ForeignCallNode;
@@ -54,25 +61,29 @@ public class AssertionSnippets implements Snippets {
      * This call can only be used with true for the "vmError" parameter, so that it can be
      * configured to be a leaf method.
      */
-    public static final ForeignCallDescriptor ASSERTION_VM_MESSAGE_C = new ForeignCallDescriptor("assertionVmMessageC", void.class, boolean.class, Word.class, long.class, long.class, long.class);
+    public static final HotSpotForeignCallDescriptor ASSERTION_VM_MESSAGE_C = new HotSpotForeignCallDescriptor(LEAF, REEXECUTABLE, NO_LOCATIONS, "assertionVmMessageC", void.class, boolean.class,
+                    Word.class, long.class, long.class, long.class);
 
     @Snippet
-    public static void assertion(boolean condition, @ConstantParameter String message) {
-        if (!condition) {
-            vmMessageC(ASSERTION_VM_MESSAGE_C, true, cstring(message), 0L, 0L, 0L);
+    public static void assertion(boolean condition, @ConstantParameter String message, long l1, long l2) {
+        if (injectBranchProbability(SLOWPATH_PROBABILITY, !condition)) {
+            vmMessageC(ASSERTION_VM_MESSAGE_C, true, cstring(message), l1, l2, 0L);
         }
     }
 
     @Snippet
-    public static void stubAssertion(boolean condition, @ConstantParameter String message) {
-        if (!condition) {
-            vmMessageC(ASSERTION_VM_MESSAGE_C, true, cstring(message), 0L, 0L, 0L);
+    public static void stubAssertion(boolean condition, @ConstantParameter String message, long l1, long l2) {
+        if (injectBranchProbability(SLOWPATH_PROBABILITY, !condition)) {
+            vmMessageStub(ASSERTION_VM_MESSAGE_C, true, cstring(message), l1, l2, 0L);
         }
     }
 
     @NodeIntrinsic(ForeignCallNode.class)
     static native void vmMessageC(@ConstantNodeParameter ForeignCallDescriptor stubPrintfC, boolean vmError, Word format, long v1, long v2, long v3);
 
+    @NodeIntrinsic(StubForeignCallNode.class)
+    static native void vmMessageStub(@ConstantNodeParameter ForeignCallDescriptor stubPrintfC, boolean vmError, Word format, long v1, long v2, long v3);
+
     public static class Templates extends AbstractTemplates {
 
         private final SnippetInfo assertion = snippet(AssertionSnippets.class, "assertion");
@@ -87,7 +98,8 @@ public class AssertionSnippets implements Snippets {
             Arguments args = new Arguments(graph.start() instanceof StubStartNode ? stubAssertion : assertion, graph.getGuardsStage(), tool.getLoweringStage());
             args.add("condition", assertionNode.condition());
             args.addConst("message", "failed runtime assertion in snippet/stub: " + assertionNode.message() + " (" + graph.method() + ")");
-
+            args.add("l1", assertionNode.getL1());
+            args.add("l2", assertionNode.getL2());
             template(assertionNode, args).instantiate(providers.getMetaAccess(), assertionNode, DEFAULT_REPLACER, args);
         }
     }
