@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2012, 2015, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2012, 2019, Oracle and/or its affiliates. All rights reserved.
  * ORACLE PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
  *
  *
@@ -38,8 +38,12 @@ public class DefaultSuitesCreator extends SuitesProviderBase {
     private final CompilerConfiguration compilerConfiguration;
 
     public DefaultSuitesCreator(CompilerConfiguration compilerConfiguration, Plugins plugins) {
-        super();
+        this(compilerConfiguration);
         this.defaultGraphBuilderSuite = createGraphBuilderSuite(plugins);
+    }
+
+    public DefaultSuitesCreator(CompilerConfiguration compilerConfiguration) {
+        super();
         this.compilerConfiguration = compilerConfiguration;
     }
 
