@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2011, 2012, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2011, 2020, Oracle and/or its affiliates. All rights reserved.
  * ORACLE PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
  *
  *
@@ -24,18 +24,17 @@
 
 package org.graalvm.compiler.jtt.hotspot;
 
+import java.net.URLClassLoader;
+
 //@formatter:off
 
 /**
+ * These methods can be instrinsified by using bit scan, bit test, and population count instructions.
+ *
  * @test
  * @bug 6823354
- * @summary These methods can be instrinsified by using bit scan, bit test, and population count instructions.
- *
  * @run main/othervm -Xcomp -XX:CompileOnly=Test6823354.lzcomp,Test6823354.tzcomp,.dolzcomp,.dotzcomp Test6823354
  */
-
-import java.net.URLClassLoader;
-
 // Checkstyle: stop
 public class Test6823354 {
     // Arrays of corner case values.
@@ -199,6 +198,7 @@ public class Test6823354 {
         loadandrunclass(classname);
     }
 
+    @SuppressWarnings({"deprecation","unused"})
     static void loadandrunclass(String classname) throws Exception {
         Class<?> cl = Class.forName(classname);
         URLClassLoader apploader = (URLClassLoader) cl.getClassLoader();
