@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2013, 2015, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2013, 2020, Oracle and/or its affiliates. All rights reserved.
  * ORACLE PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
  *
  *
@@ -59,7 +59,7 @@ public final class AMD64CCall extends AMD64LIRInstruction {
 
     private void directCall(AMD64MacroAssembler masm) {
         Register reg = ValueUtil.asRegister(functionPtr);
-        masm.call(reg);
+        masm.indirectCall(reg);
         masm.ensureUniquePC();
     }
 
