@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2013, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2013, 2020, Oracle and/or its affiliates. All rights reserved.
  * ORACLE PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
  *
  *
@@ -75,8 +75,8 @@ public final class CompositeValueClass<T> extends FieldIntrospection<T> {
         CompositeValueFieldsScanner vfs = new CompositeValueFieldsScanner(new FieldsScanner.DefaultCalcOffset());
         vfs.scan(clazz, CompositeValue.class, false);
 
-        values = new Values(vfs.valueAnnotations.get(CompositeValue.Component.class));
-        data = new Fields(vfs.data);
+        values = Values.create(vfs.valueAnnotations.get(Component.class));
+        data = Fields.create(vfs.data);
     }
 
     private static class CompositeValueFieldsScanner extends LIRFieldsScanner {
