@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2015, 2016, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2015, 2019, Oracle and/or its affiliates. All rights reserved.
  * ORACLE PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
  *
  *
@@ -26,6 +26,8 @@ package org.graalvm.compiler.lir.gen;
 
 import org.graalvm.compiler.lir.LIRInstruction;
 import org.graalvm.compiler.lir.StandardOp.SaveRegistersOp;
+import org.graalvm.compiler.lir.StandardOp.ZapRegistersOp;
+
 import jdk.vm.ci.code.Register;
 import jdk.vm.ci.code.RegisterConfig;
 import jdk.vm.ci.code.StackSlot;
@@ -41,25 +43,31 @@ public interface DiagnosticLIRGeneratorTool {
      * Creates a {@link SaveRegistersOp} that fills a given set of registers with known garbage
      * value.
      *
-     * The set of registers actually touched might be {@link SaveRegistersOp#remove reduced} later.
-     *
      * @param zappedRegisters registers to be zapped
      * @param zapValues values used for zapping
      *
      * @see DiagnosticLIRGeneratorTool#createZapRegisters()
      */
-    SaveRegistersOp createZapRegisters(Register[] zappedRegisters, JavaConstant[] zapValues);
+    ZapRegistersOp createZapRegisters(Register[] zappedRegisters, JavaConstant[] zapValues);
 
     /**
-     * Creates a {@link SaveRegistersOp} that fills all
+     * Creates a {@link SaveRegistersOp} that fills a given set of registers with a
+     * {@link LIRGenerator#zapValueForKind known garbage value}.
+     *
+     * @param zappedRegisters registers to be zapped
+     *
+     * @see DiagnosticLIRGeneratorTool#createZapRegisters()
+     */
+    ZapRegistersOp createZapRegisters(Register[] zappedRegisters);
+
+    /**
+     * Creates a {@link ZapRegistersOp} that fills all
      * {@link RegisterConfig#getAllocatableRegisters() allocatable registers} with a
      * {@link LIRGenerator#zapValueForKind known garbage value}.
      *
-     * The set of registers actually touched might be {@link SaveRegistersOp#remove reduced} later.
-     *
      * @see DiagnosticLIRGeneratorTool#createZapRegisters(Register[], JavaConstant[])
      */
-    SaveRegistersOp createZapRegisters();
+    ZapRegistersOp createZapRegisters();
 
     /**
      * Marker interface for {@link LIRInstruction instructions} that should be succeeded with a
