@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2015, 2015, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2015, 2019, Oracle and/or its affiliates. All rights reserved.
  * ORACLE PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
  *
  *
@@ -24,22 +24,18 @@
 
 package org.graalvm.compiler.lir.phases;
 
-import static org.graalvm.compiler.core.common.GraalOptions.TraceRA;
-
 import org.graalvm.compiler.lir.alloc.lsra.LinearScanPhase;
-import org.graalvm.compiler.lir.alloc.trace.TraceRegisterAllocationPhase;
 import org.graalvm.compiler.lir.dfa.LocationMarkerPhase;
+import org.graalvm.compiler.lir.dfa.MarkBasePointersPhase;
 import org.graalvm.compiler.lir.phases.AllocationPhase.AllocationContext;
 import org.graalvm.compiler.lir.stackslotalloc.SimpleStackSlotAllocator;
 import org.graalvm.compiler.options.OptionValues;
 
 public class EconomyAllocationStage extends LIRPhaseSuite<AllocationContext> {
-    public EconomyAllocationStage(OptionValues options) {
-        if (TraceRA.getValue(options)) {
-            appendPhase(new TraceRegisterAllocationPhase());
-        } else {
-            appendPhase(new LinearScanPhase());
-        }
+    public EconomyAllocationStage(@SuppressWarnings("unused") OptionValues options) {
+        appendPhase(new MarkBasePointersPhase());
+
+        appendPhase(new LinearScanPhase());
 
         // build frame map
         appendPhase(new SimpleStackSlotAllocator());
