@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2015, 2016, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2015, 2020, Oracle and/or its affiliates. All rights reserved.
  * ORACLE PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
  *
  *
@@ -88,17 +88,19 @@ public class MathUtil {
         if (isConstantOne(divisor)) {
             return dividend;
         }
-        ValueNode div = graph.addOrUniqueWithInputs(createDiv.apply(dividend, divisor));
+        ValueNode div = createDiv.apply(dividend, divisor);
         if (div instanceof FixedBinaryNode) {
             FixedBinaryNode fixedDiv = (FixedBinaryNode) div;
             if (before.predecessor() instanceof FixedBinaryNode) {
                 FixedBinaryNode binaryPredecessor = (FixedBinaryNode) before.predecessor();
                 if (fixedDiv.dataFlowEquals(binaryPredecessor)) {
-                    fixedDiv.safeDelete();
+                    if (fixedDiv.isAlive()) {
+                        fixedDiv.safeDelete();
+                    }
                     return binaryPredecessor;
                 }
             }
-            graph.addBeforeFixed(before, fixedDiv);
+            graph.addBeforeFixed(before, graph.addOrUniqueWithInputs(fixedDiv));
         }
         return div;
     }
