@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2015, 2015, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2015, 2020, Oracle and/or its affiliates. All rights reserved.
  * ORACLE PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
  *
  *
@@ -27,7 +27,6 @@ package org.graalvm.compiler.microbenchmarks.graal;
 import java.util.HashMap;
 
 import org.openjdk.jmh.annotations.Benchmark;
-import org.openjdk.jmh.annotations.Warmup;
 import org.openjdk.jmh.infra.Blackhole;
 
 import org.graalvm.compiler.graph.Node;
@@ -37,6 +36,7 @@ import org.graalvm.compiler.microbenchmarks.graal.util.MethodSpec;
 import org.graalvm.compiler.microbenchmarks.graal.util.NodesState;
 import org.graalvm.compiler.microbenchmarks.graal.util.NodesState.NodePair;
 import org.graalvm.compiler.nodes.ConstantNode;
+import org.graalvm.compiler.nodes.StructuredGraph;
 import org.graalvm.compiler.nodes.calc.AddNode;
 import org.graalvm.compiler.nodes.util.GraphUtil;
 
@@ -46,8 +46,28 @@ public class NodeBenchmark extends GraalBenchmark {
     public static class StringEquals extends NodesState {
     }
 
+    /**
+     * Variation of {@link StringEquals} that calls {@link StructuredGraph#maybeCompress()} after
+     * every N iterations. The prevents benchmarks that mutate the graph by adding and removing
+     * nodes from causing {@link OutOfMemoryError}s.
+     */
+    @MethodSpec(declaringClass = String.class, name = "equals")
+    public static class StringEqualsWithGraphCompression extends NodesState {
+        private static final int N = 100_000;
+
+        private int latch = N;
+
+        @Override
+        public void afterInvocation() {
+            super.afterInvocation();
+            if (--latch == 0) {
+                graph.maybeCompress();
+                latch = N;
+            }
+        }
+    }
+
     @Benchmark
-    @Warmup(iterations = 20)
     public int getNodeClass(StringEquals s) {
         int sum = 0;
         for (Node n : s.nodes) {
@@ -101,7 +121,7 @@ public class NodeBenchmark extends GraalBenchmark {
     }
 
     @Benchmark
-    public void createAndDeleteAdd(StringEquals s, Blackhole bh) {
+    public void createAndDeleteAdd(StringEqualsWithGraphCompression s, Blackhole bh) {
         AddNode addNode = new AddNode(ConstantNode.forInt(40), ConstantNode.forInt(2));
         s.graph.addOrUniqueWithInputs(addNode);
         GraphUtil.killWithUnusedFloatingInputs(addNode);
@@ -109,7 +129,7 @@ public class NodeBenchmark extends GraalBenchmark {
     }
 
     @Benchmark
-    public void createAndDeleteConstant(StringEquals s, Blackhole bh) {
+    public void createAndDeleteConstant(StringEqualsWithGraphCompression s, Blackhole bh) {
         ConstantNode constantNode = ConstantNode.forInt(42);
         s.graph.addOrUnique(constantNode);
         GraphUtil.killWithUnusedFloatingInputs(constantNode);
@@ -126,7 +146,6 @@ public class NodeBenchmark extends GraalBenchmark {
     }
 
     @Benchmark
-    @Warmup(iterations = 20)
     public void nodeBitmap(StringEquals s, @SuppressWarnings("unused") GraalState g) {
         NodeBitMap bitMap = s.graph.createNodeBitMap();
         for (Node node : s.graph.getNodes()) {
@@ -147,7 +166,6 @@ public class NodeBenchmark extends GraalBenchmark {
 
     // Checkstyle: stop method name check
     @Benchmark
-    @Warmup(iterations = 20)
     public int valueEquals_STRING_EQUALS(StringEquals s) {
         int result = 0;
         for (NodePair np : s.valueEqualsNodePairs) {
@@ -161,7 +179,6 @@ public class NodeBenchmark extends GraalBenchmark {
     }
 
     @Benchmark
-    @Warmup(iterations = 20)
     public int valueEquals_HASHMAP_COMPUTE_IF_ABSENT(HashMapComputeIfAbsent s) {
         int result = 0;
         for (NodePair np : s.valueEqualsNodePairs) {
@@ -175,7 +192,6 @@ public class NodeBenchmark extends GraalBenchmark {
     }
 
     @Benchmark
-    @Warmup(iterations = 20)
     public int valueNumberLeaf_HASHMAP_COMPUTE_IF_ABSENT(HashMapComputeIfAbsent s) {
         int result = 0;
         for (Node n : s.valueNumberableLeafNodes) {
@@ -185,7 +201,6 @@ public class NodeBenchmark extends GraalBenchmark {
     }
 
     @Benchmark
-    @Warmup(iterations = 20)
     public int valueNumberLeaf_STRING_EQUALS(StringEquals s) {
         int result = 0;
         for (Node n : s.valueNumberableLeafNodes) {
