@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2015, 2018, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2015, 2020, Oracle and/or its affiliates. All rights reserved.
  * ORACLE PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
  *
  *
@@ -124,6 +124,14 @@ public class GraphEncoder {
      * The orderId of the first actual node after the {@link StructuredGraph#start() start node}.
      */
     public static final int FIRST_NODE_ORDER_ID = 2;
+    /**
+     * Maximum unsigned integer fitting on 1 byte.
+     */
+    public static final int MAX_INDEX_1_BYTE = 1 << 8 - 1;
+    /**
+     * Maximum unsigned integer fitting on 2 bytes.
+     */
+    public static final int MAX_INDEX_2_BYTES = 1 << 16 - 1;
 
     /**
      * The known offset between the orderId of a {@link AbstractBeginNode} and its
@@ -435,7 +443,14 @@ public class GraphEncoder {
     }
 
     protected void writeOrderId(Node node, NodeOrder nodeOrder) {
-        writer.putUV(node == null ? NULL_ORDER_ID : nodeOrder.orderIds.get(node));
+        int id = node == null ? NULL_ORDER_ID : nodeOrder.orderIds.get(node);
+        if (nodeOrder.nextOrderId <= MAX_INDEX_1_BYTE) {
+            writer.putU1(id);
+        } else if (nodeOrder.nextOrderId <= MAX_INDEX_2_BYTES) {
+            writer.putU2(id);
+        } else {
+            writer.putS4(id);
+        }
     }
 
     protected void writeObjectId(Object object) {
@@ -512,8 +527,8 @@ class GraphComparison {
 
             if (expectedNode instanceof EndNode) {
                 /* Visit the merge node, which is the one and only usage of the EndNode. */
-                assert expectedNode.usages().count() == 1;
-                assert actualNode.usages().count() == 1;
+                assert expectedNode.hasExactlyOneUsage();
+                assert actualNode.hasExactlyOneUsage();
                 verifyNodesEqual(expectedNode.usages(), actualNode.usages(), nodeMapping, workList, false);
             }
 
