@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2011, 2015, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2011, 2018, Oracle and/or its affiliates. All rights reserved.
  * ORACLE PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
  *
  *
@@ -109,30 +109,15 @@ public final class IntegerEqualsNode extends CompareNode implements BinaryCommut
     public static class IntegerEqualsOp extends CompareOp {
         @Override
         protected LogicNode optimizeNormalizeCompare(ConstantReflectionProvider constantReflection, MetaAccessProvider metaAccess, OptionValues options, Integer smallestCompareWidth,
-                        Constant constant, NormalizeCompareNode normalizeNode, boolean mirrored, NodeView view) {
+                        Constant constant, AbstractNormalizeCompareNode normalizeNode, boolean mirrored, NodeView view) {
             PrimitiveConstant primitive = (PrimitiveConstant) constant;
-            ValueNode a = normalizeNode.getX();
-            ValueNode b = normalizeNode.getY();
             long cst = primitive.asLong();
-
             if (cst == 0) {
-                if (normalizeNode.getX().getStackKind() == JavaKind.Double || normalizeNode.getX().getStackKind() == JavaKind.Float) {
-                    return FloatEqualsNode.create(constantReflection, metaAccess, options, smallestCompareWidth, a, b, view);
-                } else {
-                    return IntegerEqualsNode.create(constantReflection, metaAccess, options, smallestCompareWidth, a, b, view);
-                }
+                return normalizeNode.createEqualComparison(constantReflection, metaAccess, options, smallestCompareWidth, view);
             } else if (cst == 1) {
-                if (normalizeNode.getX().getStackKind() == JavaKind.Double || normalizeNode.getX().getStackKind() == JavaKind.Float) {
-                    return FloatLessThanNode.create(b, a, !normalizeNode.isUnorderedLess, view);
-                } else {
-                    return IntegerLessThanNode.create(constantReflection, metaAccess, options, smallestCompareWidth, b, a, view);
-                }
+                return normalizeNode.createLowerComparison(true, constantReflection, metaAccess, options, smallestCompareWidth, view);
             } else if (cst == -1) {
-                if (normalizeNode.getX().getStackKind() == JavaKind.Double || normalizeNode.getX().getStackKind() == JavaKind.Float) {
-                    return FloatLessThanNode.create(a, b, normalizeNode.isUnorderedLess, view);
-                } else {
-                    return IntegerLessThanNode.create(constantReflection, metaAccess, options, smallestCompareWidth, a, b, view);
-                }
+                return normalizeNode.createLowerComparison(false, constantReflection, metaAccess, options, smallestCompareWidth, view);
             } else {
                 return LogicConstantNode.contradiction();
             }
@@ -165,15 +150,19 @@ public final class IntegerEqualsNode extends CompareNode implements BinaryCommut
                 ValueNode v1 = null;
                 ValueNode v2 = null;
                 if (addX.getX() == addY.getX()) {
+                    // (x + y) == (x + z) => y == z
                     v1 = addX.getY();
                     v2 = addY.getY();
                 } else if (addX.getX() == addY.getY()) {
+                    // (x + y) == (z + x) => y == z
                     v1 = addX.getY();
                     v2 = addY.getX();
                 } else if (addX.getY() == addY.getX()) {
+                    // (y + x) == (x + z) => y == z
                     v1 = addX.getX();
                     v2 = addY.getY();
                 } else if (addX.getY() == addY.getY()) {
+                    // (y + x) == (z + x) => y == z
                     v1 = addX.getX();
                     v2 = addY.getX();
                 }
@@ -183,6 +172,64 @@ public final class IntegerEqualsNode extends CompareNode implements BinaryCommut
                 }
             }
 
+            if (forX instanceof SubNode && forY instanceof SubNode) {
+                SubNode subX = (SubNode) forX;
+                SubNode subY = (SubNode) forY;
+                ValueNode v1 = null;
+                ValueNode v2 = null;
+                if (subX.getX() == subY.getX()) {
+                    // (x - y) == (x - z) => y == z
+                    v1 = subX.getY();
+                    v2 = subY.getY();
+                } else if (subX.getY() == subY.getY()) {
+                    // (y - x) == (z - x) => y == z
+                    v1 = subX.getX();
+                    v2 = subY.getX();
+                }
+                if (v1 != null) {
+                    assert v2 != null;
+                    return create(v1, v2, view);
+                }
+            }
+
+            if (forX instanceof AddNode) {
+                AddNode addNode = (AddNode) forX;
+                if (addNode.getX() == forY) {
+                    // (x + y) == x => y == 0
+                    return create(addNode.getY(), ConstantNode.forIntegerStamp(view.stamp(addNode), 0), view);
+                } else if (addNode.getY() == forY) {
+                    // (x + y) == y => x == 0
+                    return create(addNode.getX(), ConstantNode.forIntegerStamp(view.stamp(addNode), 0), view);
+                }
+            }
+
+            if (forY instanceof AddNode) {
+                AddNode addNode = (AddNode) forY;
+                if (addNode.getX() == forX) {
+                    // x == (x + y) => y == 0
+                    return create(addNode.getY(), ConstantNode.forIntegerStamp(view.stamp(addNode), 0), view);
+                } else if (addNode.getY() == forX) {
+                    // y == (x + y) => x == 0
+                    return create(addNode.getX(), ConstantNode.forIntegerStamp(view.stamp(addNode), 0), view);
+                }
+            }
+
+            if (forX instanceof SubNode) {
+                SubNode subNode = (SubNode) forX;
+                if (subNode.getX() == forY) {
+                    // (x - y) == x => y == 0
+                    return create(subNode.getY(), ConstantNode.forIntegerStamp(view.stamp(subNode), 0), view);
+                }
+            }
+
+            if (forY instanceof SubNode) {
+                SubNode subNode = (SubNode) forY;
+                if (forX == subNode.getX()) {
+                    // x == (x - y) => y == 0
+                    return create(subNode.getY(), ConstantNode.forIntegerStamp(view.stamp(subNode), 0), view);
+                }
+            }
+
             return super.canonical(constantReflection, metaAccess, options, smallestCompareWidth, condition, unorderedIsTrue, forX, forY, view);
         }
 
@@ -199,7 +246,7 @@ public final class IntegerEqualsNode extends CompareNode implements BinaryCommut
                     // execution
                     // on specific platforms.
                     return LogicNegationNode.create(
-                                    IntegerEqualsNode.create(constantReflection, metaAccess, options, smallestCompareWidth, nonConstant, ConstantNode.forIntegerKind(nonConstant.getStackKind(), 0),
+                                    IntegerEqualsNode.create(constantReflection, metaAccess, options, smallestCompareWidth, nonConstant, ConstantNode.forIntegerStamp(nonConstantStamp, 0),
                                                     view));
                 } else if (primitiveConstant.asLong() == 0) {
                     if (nonConstant instanceof AndNode) {
@@ -306,4 +353,19 @@ public final class IntegerEqualsNode extends CompareNode implements BinaryCommut
         }
         return TriState.UNKNOWN;
     }
+
+    @Override
+    public TriState implies(boolean thisNegated, LogicNode other) {
+        // x == y => !(x < y)
+        // x == y => !(y < x)
+        if (!thisNegated && other instanceof IntegerLessThanNode) {
+            ValueNode otherX = ((IntegerLessThanNode) other).getX();
+            ValueNode otherY = ((IntegerLessThanNode) other).getY();
+            if ((getX() == otherX && getY() == otherY) || (getX() == otherY && getY() == otherX)) {
+                return TriState.FALSE;
+            }
+        }
+
+        return super.implies(thisNegated, other);
+    }
 }
