@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2009, 2015, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2009, 2020, Oracle and/or its affiliates. All rights reserved.
  * ORACLE PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
  *
  *
@@ -26,9 +26,12 @@ package org.graalvm.compiler.nodes.calc;
 
 import static org.graalvm.compiler.nodeinfo.NodeCycles.CYCLES_2;
 import static org.graalvm.compiler.nodeinfo.NodeSize.SIZE_1;
+import static org.graalvm.compiler.nodes.calc.BinaryArithmeticNode.getArithmeticOpTable;
 
 import org.graalvm.compiler.core.common.type.ArithmeticOpTable;
+import org.graalvm.compiler.core.common.type.ArithmeticOpTable.UnaryOp;
 import org.graalvm.compiler.core.common.type.ArithmeticOpTable.UnaryOp.Neg;
+import org.graalvm.compiler.core.common.type.IntegerStamp;
 import org.graalvm.compiler.core.common.type.FloatStamp;
 import org.graalvm.compiler.core.common.type.Stamp;
 import org.graalvm.compiler.graph.NodeClass;
@@ -49,7 +52,7 @@ public final class NegateNode extends UnaryArithmeticNode<Neg> implements Narrow
     public static final NodeClass<NegateNode> TYPE = NodeClass.create(NegateNode.class);
 
     public NegateNode(ValueNode value) {
-        super(TYPE, ArithmeticOpTable::getNeg, value);
+        super(TYPE, getArithmeticOpTable(value).getNeg(), value);
     }
 
     public static ValueNode create(ValueNode value, NodeView view) {
@@ -60,6 +63,11 @@ public final class NegateNode extends UnaryArithmeticNode<Neg> implements Narrow
         return new NegateNode(value);
     }
 
+    @Override
+    protected UnaryOp<Neg> getOp(ArithmeticOpTable table) {
+        return table.getNeg();
+    }
+
     @Override
     public ValueNode canonical(CanonicalizerTool tool, ValueNode forValue) {
         ValueNode synonym = findSynonym(forValue, getOp(forValue));
@@ -71,6 +79,8 @@ public final class NegateNode extends UnaryArithmeticNode<Neg> implements Narrow
 
     protected static ValueNode findSynonym(ValueNode forValue, NodeView view) {
         ArithmeticOpTable.UnaryOp<Neg> negOp = ArithmeticOpTable.forStamp(forValue.stamp(view)).getNeg();
+
+        // Folds constants
         ValueNode synonym = UnaryArithmeticNode.findSynonym(forValue, negOp);
         if (synonym != null) {
             return synonym;
@@ -82,6 +92,17 @@ public final class NegateNode extends UnaryArithmeticNode<Neg> implements Narrow
             SubNode sub = (SubNode) forValue;
             return SubNode.create(sub.getY(), sub.getX(), view);
         }
+        // e.g. -(x >> 31) => x >>> 31
+        if (forValue instanceof RightShiftNode) {
+            RightShiftNode shift = (RightShiftNode) forValue;
+            Stamp stamp = forValue.stamp(view);
+            if (shift.getY().isConstant() && stamp instanceof IntegerStamp) {
+                int shiftAmount = shift.getY().asJavaConstant().asInt();
+                if (shiftAmount == ((IntegerStamp) stamp).getBits() - 1) {
+                    return UnsignedRightShiftNode.create(shift.getX(), shift.getY(), view);
+                }
+            }
+        }
         return null;
     }
 
