@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2011, 2019, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2011, 2020, Oracle and/or its affiliates. All rights reserved.
  * ORACLE PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
  *
  *
@@ -104,6 +104,13 @@ public class SignedRemNode extends IntegerDivRemNode implements LIRLowerable {
                 }
             }
         }
+        if (self != null && self.hasNoUsages() && self.next() instanceof SignedDivNode) {
+            SignedDivNode div = (SignedDivNode) self.next();
+            if (div.x == self.x && div.y == self.y && div.getZeroCheck() == self.getZeroCheck() && div.stateBefore() == self.stateBefore()) {
+                // left over from canonicalizing ((a - a % b) / b) into (a / b)
+                return null;
+            }
+        }
         if (self != null && self.x == forX && self.y == forY) {
             return self;
         } else {
@@ -117,10 +124,7 @@ public class SignedRemNode extends IntegerDivRemNode implements LIRLowerable {
             return false;
         }
 
-        int compareAgainstZero = 0;
-        int usageCount = self.getUsageCount();
-        for (int i = 0; i < usageCount; i++) {
-            Node usage = self.getUsageAt(i);
+        for (Node usage : self.usages()) {
             if (usage instanceof IntegerEqualsNode) {
                 IntegerEqualsNode equalsNode = (IntegerEqualsNode) usage;
                 ValueNode node = equalsNode.getY();
@@ -131,12 +135,13 @@ public class SignedRemNode extends IntegerDivRemNode implements LIRLowerable {
                     ConstantNode constantNode = (ConstantNode) node;
                     Constant constant = constantNode.asConstant();
                     if (constant instanceof PrimitiveConstant && ((PrimitiveConstant) constant).asLong() == 0) {
-                        compareAgainstZero++;
+                        continue;
                     }
                 }
             }
+            return false;
         }
-        return compareAgainstZero == usageCount;
+        return true;
     }
 
     @Override
