@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2011, 2016, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2011, 2020, Oracle and/or its affiliates. All rights reserved.
  * ORACLE PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
  *
  *
@@ -38,13 +38,14 @@ import org.graalvm.compiler.graph.spi.Canonicalizable;
 import org.graalvm.compiler.graph.spi.CanonicalizerTool;
 import org.graalvm.compiler.nodeinfo.NodeInfo;
 import org.graalvm.compiler.nodeinfo.Verbosity;
+import org.graalvm.compiler.nodes.FrameState;
 import org.graalvm.compiler.nodes.ValueNode;
 import org.graalvm.compiler.nodes.memory.AbstractMemoryCheckpoint;
-import org.graalvm.compiler.nodes.memory.MemoryCheckpoint;
+import org.graalvm.compiler.nodes.memory.SingleMemoryKill;
 import org.graalvm.compiler.nodes.spi.Lowerable;
-import org.graalvm.compiler.nodes.spi.LoweringTool;
 import jdk.internal.vm.compiler.word.LocationIdentity;
 
+import jdk.vm.ci.meta.JavaKind;
 import jdk.vm.ci.meta.MetaAccessProvider;
 
 /**
@@ -56,14 +57,16 @@ import jdk.vm.ci.meta.MetaAccessProvider;
           cyclesRationale = "Node will be lowered to a foreign call.",
           size = SIZE_8)
 // @formatter:on
-public final class BytecodeExceptionNode extends AbstractMemoryCheckpoint implements Lowerable, MemoryCheckpoint.Single, Canonicalizable {
+public final class BytecodeExceptionNode extends AbstractMemoryCheckpoint implements Lowerable, SingleMemoryKill, Canonicalizable {
 
     public enum BytecodeExceptionKind {
         NULL_POINTER(0, NullPointerException.class),
         OUT_OF_BOUNDS(2, ArrayIndexOutOfBoundsException.class),
         CLASS_CAST(2, ClassCastException.class),
         ARRAY_STORE(1, ArrayStoreException.class),
-        DIVISION_BY_ZERO(0, ArithmeticException.class);
+        DIVISION_BY_ZERO(0, ArithmeticException.class),
+        INTEGER_EXACT_OVERFLOW(0, ArithmeticException.class),
+        LONG_EXACT_OVERFLOW(0, ArithmeticException.class);
 
         final int numArguments;
         final Class<? extends Throwable> exceptionClass;
@@ -98,24 +101,30 @@ public final class BytecodeExceptionNode extends AbstractMemoryCheckpoint implem
     }
 
     @Override
-    public LocationIdentity getLocationIdentity() {
+    public LocationIdentity getKilledLocationIdentity() {
         return LocationIdentity.any();
     }
 
     @Override
     public Node canonical(CanonicalizerTool tool) {
-        if (tool.allUsagesAvailable() && getUsageCount() == 0) {
+        if (tool.allUsagesAvailable() && (hasNoUsages() || (hasExactlyOneUsage() && usages().first() == stateAfter))) {
             return null;
         }
         return this;
     }
 
-    @Override
-    public void lower(LoweringTool tool) {
-        tool.getLowerer().lower(this, tool);
-    }
-
     public NodeInputList<ValueNode> getArguments() {
         return arguments;
     }
+
+    /**
+     * Create a new stateDuring for use by a foreign call.
+     */
+    public FrameState createStateDuring() {
+        boolean rethrowException = false;
+        boolean duringCall = true;
+        return stateAfter.duplicateModified(graph(), stateAfter.bci, rethrowException, duringCall,
+                        JavaKind.Object, null, null);
+    }
+
 }
