@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2012, 2015, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2012, 2020, Oracle and/or its affiliates. All rights reserved.
  * ORACLE PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
  *
  *
@@ -24,11 +24,11 @@
 
 package org.graalvm.compiler.nodes.extended;
 
-import org.graalvm.compiler.nodes.memory.MemoryCheckpoint;
+import org.graalvm.compiler.nodes.memory.SingleMemoryKill;
 
 /**
  * Denotes monitor unlocking transition.
  */
-public interface MonitorExit extends MemoryCheckpoint {
+public interface MonitorExit extends SingleMemoryKill {
 
 }
