@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2013, 2015, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2013, 2020, Oracle and/or its affiliates. All rights reserved.
  * ORACLE PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
  *
  *
@@ -31,12 +31,12 @@ import org.graalvm.compiler.core.common.type.StampFactory;
 import org.graalvm.compiler.graph.NodeClass;
 import org.graalvm.compiler.graph.spi.Canonicalizable;
 import org.graalvm.compiler.graph.spi.CanonicalizerTool;
+import org.graalvm.compiler.nodeinfo.InputType;
 import org.graalvm.compiler.nodeinfo.NodeInfo;
 import org.graalvm.compiler.nodes.ConstantNode;
-import org.graalvm.compiler.nodes.FixedWithNextNode;
+import org.graalvm.compiler.nodes.FieldLocationIdentity;
 import org.graalvm.compiler.nodes.ValueNode;
 import org.graalvm.compiler.nodes.spi.Lowerable;
-import org.graalvm.compiler.nodes.spi.LoweringTool;
 import org.graalvm.compiler.nodes.spi.Virtualizable;
 import org.graalvm.compiler.nodes.spi.VirtualizerTool;
 import org.graalvm.compiler.nodes.type.StampTool;
@@ -48,22 +48,21 @@ import jdk.vm.ci.meta.JavaKind;
 import jdk.vm.ci.meta.MetaAccessProvider;
 import jdk.vm.ci.meta.ResolvedJavaType;
 
-@NodeInfo(cycles = CYCLES_2, size = SIZE_2)
-public final class UnboxNode extends FixedWithNextNode implements Virtualizable, Lowerable, Canonicalizable.Unary<ValueNode> {
+@NodeInfo(cycles = CYCLES_2, size = SIZE_2, allowedUsageTypes = {InputType.Memory, InputType.Value})
+public final class UnboxNode extends AbstractBoxingNode implements Virtualizable, Lowerable, Canonicalizable.Unary<ValueNode> {
 
     public static final NodeClass<UnboxNode> TYPE = NodeClass.create(UnboxNode.class);
-    @Input protected ValueNode value;
-    protected final JavaKind boxingKind;
 
-    @Override
-    public ValueNode getValue() {
-        return value;
+    public UnboxNode(ValueNode value, JavaKind boxingKind, FieldLocationIdentity location) {
+        super(TYPE, value, boxingKind, StampFactory.forKind(boxingKind.getStackKind()), location);
     }
 
-    public UnboxNode(ValueNode value, JavaKind boxingKind) {
-        super(TYPE, StampFactory.forKind(boxingKind.getStackKind()));
-        this.value = value;
-        this.boxingKind = boxingKind;
+    public UnboxNode(ValueNode value, JavaKind boxingKind, MetaAccessProvider metaAccess) {
+        super(TYPE, value, boxingKind, StampFactory.forKind(boxingKind.getStackKind()), new FieldLocationIdentity(getValueField(getResultType(metaAccess, boxingKind))));
+    }
+
+    private static ResolvedJavaType getResultType(MetaAccessProvider metaAccess, JavaKind boxingKind) {
+        return metaAccess.lookupJavaType(boxingKind.toBoxedJavaClass());
     }
 
     public static ValueNode create(MetaAccessProvider metaAccess, ConstantReflectionProvider constantReflection, ValueNode value, JavaKind boxingKind) {
@@ -71,16 +70,12 @@ public final class UnboxNode extends FixedWithNextNode implements Virtualizable,
         if (synonym != null) {
             return synonym;
         }
-        return new UnboxNode(value, boxingKind);
-    }
-
-    public JavaKind getBoxingKind() {
-        return boxingKind;
+        return new UnboxNode(value, boxingKind, metaAccess);
     }
 
     @Override
-    public void lower(LoweringTool tool) {
-        tool.getLowerer().lower(this, tool);
+    public ValueNode getValue() {
+        return value;
     }
 
     @Override
@@ -89,7 +84,7 @@ public final class UnboxNode extends FixedWithNextNode implements Virtualizable,
         if (alias instanceof VirtualObjectNode) {
             VirtualObjectNode virtual = (VirtualObjectNode) alias;
             ResolvedJavaType objectType = virtual.type();
-            ResolvedJavaType expectedType = tool.getMetaAccessProvider().lookupJavaType(boxingKind.toBoxedJavaClass());
+            ResolvedJavaType expectedType = tool.getMetaAccess().lookupJavaType(boxingKind.toBoxedJavaClass());
             if (objectType.equals(expectedType)) {
                 tool.replaceWithValue(tool.getEntry(virtual, 0));
             }
@@ -123,4 +118,5 @@ public final class UnboxNode extends FixedWithNextNode implements Virtualizable,
         }
         return null;
     }
+
 }
