@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2013, 2014, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2013, 2020, Oracle and/or its affiliates. All rights reserved.
  * ORACLE PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
  *
  *
@@ -78,18 +78,13 @@ public class DynamicNewInstanceNode extends AbstractNewObjectNode implements Can
                 return this;
             }
             ResolvedJavaType type = tool.getConstantReflection().asJavaType(clazz.asConstant());
-            if (type != null && type.isInitialized() && !throwsInstantiationException(type, tool.getMetaAccess())) {
-                return createNewInstanceNode(type);
+            if (type != null && !throwsInstantiationException(type, tool.getMetaAccess()) && tool.getMetaAccessExtensionProvider().canConstantFoldDynamicAllocation(type)) {
+                return new NewInstanceNode(type, fillContents(), stateBefore());
             }
         }
         return this;
     }
 
-    /** Hook for subclasses to instantiate a subclass of {@link NewInstanceNode}. */
-    protected NewInstanceNode createNewInstanceNode(ResolvedJavaType type) {
-        return new NewInstanceNode(type, fillContents(), stateBefore());
-    }
-
     public static boolean throwsInstantiationException(Class<?> type, Class<?> classClass) {
         return type.isPrimitive() || type.isArray() || type.isInterface() || Modifier.isAbstract(type.getModifiers()) || type == classClass;
     }
