@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2011, 2015, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2011, 2020, Oracle and/or its affiliates. All rights reserved.
  * ORACLE PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
  *
  *
@@ -26,11 +26,12 @@ package org.graalvm.compiler.nodes.spi;
 
 import java.util.List;
 
-import org.graalvm.compiler.core.common.spi.ArrayOffsetProvider;
+import org.graalvm.compiler.core.common.spi.MetaAccessExtensionProvider;
 import org.graalvm.compiler.debug.DebugContext;
 import org.graalvm.compiler.debug.GraalError;
 import org.graalvm.compiler.graph.Node;
 import org.graalvm.compiler.nodes.ValueNode;
+import org.graalvm.compiler.nodes.WithExceptionNode;
 import org.graalvm.compiler.nodes.java.MonitorIdNode;
 import org.graalvm.compiler.nodes.virtual.VirtualObjectNode;
 import org.graalvm.compiler.options.OptionValues;
@@ -48,18 +49,23 @@ import jdk.vm.ci.meta.MetaAccessProvider;
  */
 public interface VirtualizerTool {
 
+    /**
+     * Returns all available providers.
+     */
+    CoreProviders getProviders();
+
     /**
      * @return the {@link MetaAccessProvider} associated with the current compilation.
      */
-    MetaAccessProvider getMetaAccessProvider();
+    MetaAccessProvider getMetaAccess();
 
     /**
      * @return the {@link ConstantReflectionProvider} associated with the current compilation, which
      *         can be used to access {@link JavaConstant}s.
      */
-    ConstantReflectionProvider getConstantReflectionProvider();
+    ConstantReflectionProvider getConstantReflection();
 
-    ArrayOffsetProvider getArrayOffsetProvider();
+    MetaAccessExtensionProvider getMetaAccessExtensionProvider();
 
     /**
      * This method should be used to query the maximum size of virtualized objects before attempting
@@ -98,7 +104,7 @@ public interface VirtualizerTool {
      * @param index the index to be set.
      * @param value the new value for the given index.
      * @param accessKind the kind of the store which might be different than
-     *            {@link VirtualObjectNode#entryKind(int)}.
+     *            {@link VirtualObjectNode#entryKind}.
      * @return true if the operation was permitted
      */
     boolean setVirtualEntry(VirtualObjectNode virtualObject, int index, ValueNode value, JavaKind accessKind, long offset);
@@ -122,7 +128,8 @@ public interface VirtualizerTool {
     // operations on the current node
 
     /**
-     * Deletes the current node and replaces it with the given virtualized object.
+     * Deletes the current node and replaces it with the given virtualized object. If the current
+     * node is a {@link WithExceptionNode}, kills the exception edge.
      *
      * @param virtualObject the virtualized object that should replace the current node.
      */
@@ -136,7 +143,8 @@ public interface VirtualizerTool {
     void replaceWithValue(ValueNode replacement);
 
     /**
-     * Deletes the current node.
+     * Deletes the current node. If the current node is a {@link WithExceptionNode}, kills the
+     * exception edge.
      */
     void delete();
 
@@ -172,6 +180,15 @@ public interface VirtualizerTool {
      */
     boolean ensureMaterialized(VirtualObjectNode virtualObject);
 
+    /**
+     *
+     * Returns whether deoptimization can recover from virtualizing large unsafe accesses to a byte
+     * array.
+     *
+     * @return true if deoptimization can recover, false if not.
+     */
+    boolean canVirtualizeLargeByteArrayUnsafeAccess();
+
     OptionValues getOptions();
 
     DebugContext getDebug();
