@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2014, 2018, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2014, 2020, Oracle and/or its affiliates. All rights reserved.
  * ORACLE PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
  *
  *
@@ -34,10 +34,7 @@ import org.graalvm.compiler.api.replacements.ClassSubstitution;
 import org.graalvm.compiler.api.replacements.MethodSubstitution;
 import org.graalvm.compiler.api.replacements.SnippetReflectionProvider;
 import org.graalvm.compiler.bytecode.BytecodeProvider;
-import org.graalvm.compiler.core.common.LIRKind;
 import org.graalvm.compiler.core.common.spi.ForeignCallDescriptor;
-import org.graalvm.compiler.core.common.spi.ForeignCallLinkage;
-import org.graalvm.compiler.core.common.spi.ForeignCallsProvider;
 import org.graalvm.compiler.core.common.type.Stamp;
 import org.graalvm.compiler.core.common.type.StampFactory;
 import org.graalvm.compiler.debug.DebugContext;
@@ -69,7 +66,7 @@ import org.graalvm.compiler.phases.common.FrameStateAssignmentPhase;
 import org.graalvm.compiler.phases.common.GuardLoweringPhase;
 import org.graalvm.compiler.phases.common.LoweringPhase;
 import org.graalvm.compiler.phases.tiers.HighTierContext;
-import org.graalvm.compiler.test.GraalTest;
+import org.graalvm.compiler.serviceprovider.JavaVersionUtil;
 import jdk.internal.vm.compiler.word.LocationIdentity;
 import org.junit.Assert;
 import org.junit.Assume;
@@ -203,6 +200,12 @@ public class ReplacementsParseTest extends ReplacementsTest {
         }
     }
 
+    static class TestForeignCallDescriptor extends ForeignCallDescriptor {
+        TestForeignCallDescriptor(boolean reexecutable, boolean canDeoptimize, boolean safepoint, String name, Class<?> resultType, Class<?>... argumentTypes) {
+            super(name, resultType, argumentTypes, reexecutable, new LocationIdentity[0], canDeoptimize, safepoint);
+        }
+    }
+
     @ClassSubstitution(TestObject.class)
     static class TestObjectSubstitutions {
 
@@ -308,7 +311,7 @@ public class ReplacementsParseTest extends ReplacementsTest {
             nonVoidIntrinsicWithCallStub(STUB_CALL, zLen);
         }
 
-        static final ForeignCallDescriptor STUB_CALL = new ForeignCallDescriptor("stubCall", void.class, int.class);
+        static final ForeignCallDescriptor STUB_CALL = new ForeignCallDescriptor("stubCall", void.class, new Class<?>[]{int.class}, false, new LocationIdentity[0], false, false);
 
         @NodeIntrinsic(ForeignCallNode.class)
         private static native void nonVoidIntrinsicWithCallStub(@ConstantNodeParameter ForeignCallDescriptor descriptor, int zLen);
@@ -318,7 +321,7 @@ public class ReplacementsParseTest extends ReplacementsTest {
     @Override
     protected void registerInvocationPlugins(InvocationPlugins invocationPlugins) {
         BytecodeProvider replacementBytecodeProvider = getSystemClassLoaderBytecodeProvider();
-        Registration r = new Registration(invocationPlugins, TestObject.class, replacementBytecodeProvider);
+        Registration r = new Registration(invocationPlugins, TestObject.class, getReplacements(), replacementBytecodeProvider);
         NodeIntrinsicPluginFactory.InjectionProvider injections = new DummyInjectionProvider();
         new PluginFactory_ReplacementsParseTest().registerPlugins(invocationPlugins, injections);
         r.registerMethodSubstitution(TestObjectSubstitutions.class, "nextAfter", double.class, double.class);
@@ -375,7 +378,7 @@ public class ReplacementsParseTest extends ReplacementsTest {
 
     @Test
     public void testNextAfter() {
-        Assume.assumeFalse(GraalTest.Java8OrEarlier);
+        Assume.assumeFalse(JavaVersionUtil.JAVA_SPEC <= 8);
         double[] inArray = new double[1024];
         double[] outArray = new double[1024];
         for (int i = 0; i < inArray.length; i++) {
@@ -634,7 +637,7 @@ public class ReplacementsParseTest extends ReplacementsTest {
                 node.remove();
             }
             HighTierContext context = getDefaultHighTierContext();
-            CanonicalizerPhase canonicalizer = new CanonicalizerPhase();
+            CanonicalizerPhase canonicalizer = createCanonicalizerPhase();
             new LoweringPhase(canonicalizer, LoweringTool.StandardLoweringStage.HIGH_TIER).apply(graph, context);
             new FloatingReadPhase().apply(graph);
             canonicalizer.apply(graph, context);
@@ -650,48 +653,10 @@ public class ReplacementsParseTest extends ReplacementsTest {
         @SuppressWarnings("unchecked")
         @Override
         public <T> T getInjectedArgument(Class<T> type) {
-            if (type == ForeignCallsProvider.class) {
-                return (T) new ForeignCallsProvider() {
-                    @Override
-                    public LIRKind getValueKind(JavaKind javaKind) {
-                        return null;
-                    }
-
-                    @Override
-                    public boolean isReexecutable(ForeignCallDescriptor descriptor) {
-                        return false;
-                    }
-
-                    @Override
-                    public LocationIdentity[] getKilledLocations(ForeignCallDescriptor descriptor) {
-                        return new LocationIdentity[0];
-                    }
-
-                    @Override
-                    public boolean canDeoptimize(ForeignCallDescriptor descriptor) {
-                        return false;
-                    }
-
-                    @Override
-                    public boolean isGuaranteedSafepoint(ForeignCallDescriptor descriptor) {
-                        return false;
-                    }
-
-                    @Override
-                    public ForeignCallLinkage lookupForeignCall(ForeignCallDescriptor descriptor) {
-                        return null;
-                    }
-
-                    @Override
-                    public boolean isAvailable(ForeignCallDescriptor descriptor) {
-                        return true;
-                    }
-                };
-            }
             if (type == SnippetReflectionProvider.class) {
                 return (T) getSnippetReflection();
             }
-            return null;
+            throw new InternalError("missing injection " + type);
         }
 
         @Override
