@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2016, 2018, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2016, 2019, Oracle and/or its affiliates. All rights reserved.
  * ORACLE PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
  *
  *
@@ -27,11 +27,15 @@ package org.graalvm.compiler.replacements.test;
 import org.junit.Test;
 
 public class StringIndexOfTest extends StringIndexOfTestBase {
+
+    public StringIndexOfTest(String sourceString, String constantString) {
+        super(sourceString, constantString);
+    }
+
     /*
      * These test definitions could live in the superclass except that the mx junit individual test
      * runner can't find tests in superclasses.
      */
-
     @Override
     @Test
     public void testStringIndexOfConstant() {
