@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2014, 2015, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2014, 2020, Oracle and/or its affiliates. All rights reserved.
  * ORACLE PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
  *
  *
@@ -24,9 +24,14 @@
 
 package org.graalvm.compiler.replacements;
 
+import org.graalvm.compiler.api.replacements.Fold;
+import org.graalvm.compiler.api.replacements.Fold.InjectedParameter;
 import org.graalvm.compiler.debug.Assertions;
 import org.graalvm.compiler.replacements.nodes.AssertionNode;
 
+import jdk.vm.ci.meta.JavaKind;
+import jdk.vm.ci.meta.MetaAccessProvider;
+
 // JaCoCo Exclude
 public final class ReplacementsUtil {
     private ReplacementsUtil() {
@@ -42,7 +47,13 @@ public final class ReplacementsUtil {
      */
     public static void staticAssert(boolean condition, String message) {
         if (REPLACEMENTS_ASSERTIONS_ENABLED) {
-            AssertionNode.assertion(true, condition, message);
+            AssertionNode.staticAssert(condition, message);
+        }
+    }
+
+    public static void staticAssert(boolean condition, String message, Object arg1) {
+        if (REPLACEMENTS_ASSERTIONS_ENABLED) {
+            AssertionNode.staticAssert(condition, message, arg1, "");
         }
     }
 
@@ -50,9 +61,20 @@ public final class ReplacementsUtil {
      * Asserts that condition evaluates to true at runtime. This is intended to be used within
      * snippets or stubs, and will lead to a VM error if it fails.
      */
-    public static void runtimeAssert(boolean condition, String message) {
+    public static void dynamicAssert(boolean condition, String message) {
         if (REPLACEMENTS_ASSERTIONS_ENABLED) {
-            AssertionNode.assertion(false, condition, message);
+            AssertionNode.dynamicAssert(condition, message);
         }
     }
+
+    @Fold
+    public static int arrayIndexScale(@InjectedParameter MetaAccessProvider metaAccessProvider, JavaKind elementKind) {
+        return metaAccessProvider.getArrayIndexScale(elementKind);
+    }
+
+    @Fold
+    public static int getArrayBaseOffset(@InjectedParameter MetaAccessProvider metaAccessProvider, JavaKind elementKind) {
+        return metaAccessProvider.getArrayBaseOffset(elementKind);
+    }
+
 }
