@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2012, 2015, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2012, 2020, Oracle and/or its affiliates. All rights reserved.
  * ORACLE PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
  *
  *
@@ -24,8 +24,14 @@
 
 package org.graalvm.compiler.word;
 
+import static jdk.vm.ci.services.Services.IS_BUILDING_NATIVE_IMAGE;
+
+import org.graalvm.compiler.core.common.Fields;
+import org.graalvm.compiler.core.common.type.AbstractObjectStamp;
 import org.graalvm.compiler.core.common.type.Stamp;
 import org.graalvm.compiler.core.common.type.StampFactory;
+import org.graalvm.compiler.graph.Node;
+import org.graalvm.compiler.nodes.StructuredGraph;
 import org.graalvm.compiler.nodes.ValueNode;
 import org.graalvm.compiler.nodes.type.StampTool;
 import org.graalvm.compiler.word.Word.Operation;
@@ -78,7 +84,9 @@ public class WordTypes {
         this.objectAccessType = metaAccess.lookupJavaType(ObjectAccess.class);
         this.barrieredAccessType = metaAccess.lookupJavaType(BarrieredAccess.class);
 
-        Word.ensureInitialized();
+        if (!IS_BUILDING_NATIVE_IMAGE) {
+            Word.ensureInitialized();
+        }
         this.wordImplType.initialize();
     }
 
@@ -88,7 +96,7 @@ public class WordTypes {
     public boolean isWordOperation(ResolvedJavaMethod targetMethod) {
         final boolean isWordFactory = wordFactoryType.equals(targetMethod.getDeclaringClass());
         if (isWordFactory) {
-            return true;
+            return !targetMethod.isConstructor();
         }
         final boolean isObjectAccess = objectAccessType.equals(targetMethod.getDeclaringClass());
         final boolean isBarrieredAccess = barrieredAccessType.equals(targetMethod.getDeclaringClass());
@@ -159,4 +167,37 @@ public class WordTypes {
     public ResolvedJavaType getWordImplType() {
         return wordImplType;
     }
+
+    /**
+     * Verify that the given graph does not reference word types in their object representation. We
+     * cannot only check the stamps of the nodes but also stamp field of nodes as those can cause
+     * optimizations to create wrong code later on (for example a pi stamp of instanceof stamp).
+     */
+    public boolean ensureGraphContainsNoWordTypeReferences(StructuredGraph graph) {
+        for (Node n : graph.getNodes()) {
+            if (n instanceof ValueNode) {
+                assert !isWord((ValueNode) n) : "Node " + n + " is a word type and not rewritten after graph building, this is invalid.";
+                Fields f = n.getNodeClass().getData();
+                for (int i = 0; i < f.getCount(); i++) {
+                    Object fieldValue = f.get(n, i);
+                    if (fieldValue instanceof AbstractObjectStamp) {
+                        AbstractObjectStamp os = (AbstractObjectStamp) fieldValue;
+                        assert !isWord(os.type()) : "Stamp Field " + f.getName(i) + " of node " + n + " is a word type and not rewritten after graph building, this is invalid";
+                    }
+                    // else if (fieldValue instanceof ResolvedJavaType)
+                    /*
+                     * Naturally, we would also like to verify ResolvedJavaType fields, however, we
+                     * unfortunately cannot have this invariant right now. It is invalidated by
+                     * arrays since the component type can be a word type. We would like to model
+                     * word-type-arrays as "primitive long arrays". However, we cannot since for
+                     * HotSpot we represent metaspace pointers as word types and for arrays of
+                     * metaspace pointers we need to preserve the meta space stamps until the
+                     * backend.
+                     */
+                }
+            }
+        }
+        return true;
+    }
+
 }
