@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2019, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2019, 2020, Oracle and/or its affiliates. All rights reserved.
  * ORACLE PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
  *
  *
@@ -29,7 +29,6 @@ import org.openjdk.jmh.annotations.Param;
 import org.openjdk.jmh.annotations.Scope;
 import org.openjdk.jmh.annotations.State;
 import org.openjdk.jmh.annotations.Threads;
-import org.openjdk.jmh.annotations.Warmup;
 
 /**
  * Benchmarks cost of ArrayList.
@@ -38,13 +37,12 @@ public class ArrayAllocationBenchmark extends BenchmarkBase {
 
     @State(Scope.Benchmark)
     public static class ThreadState {
-        @Param({"128", "256", "512", "1024", "2048", "4096", "8192", "16384", "32768", "65536", "131072"}) int size;
+        @Param({"128", "1024", "65536"}) int size;
         byte[] result;
     }
 
     @Benchmark
-    @Threads(8)
-    @Warmup(iterations = 10)
+    @Threads(Threads.MAX)
     public void arrayAllocate(ThreadState state) {
         state.result = new byte[state.size];
     }
