@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2017, 2018, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2017, 2020, Oracle and/or its affiliates. All rights reserved.
  * ORACLE PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
  *
  *
@@ -29,7 +29,6 @@ import java.util.concurrent.ConcurrentSkipListMap;
 import org.openjdk.jmh.annotations.Benchmark;
 import org.openjdk.jmh.annotations.Scope;
 import org.openjdk.jmh.annotations.State;
-import org.openjdk.jmh.annotations.Warmup;
 
 /**
  * Benchmarks cost of ArrayList.
@@ -44,7 +43,6 @@ public class ConcurrentSkipListBenchmark extends BenchmarkBase {
     }
 
     @Benchmark
-    @Warmup(iterations = 20)
     public void addBoxed(ThreadState state) {
         for (int i = 0; i < N; ++i) {
             state.list.put(i, i);
