@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2015, 2015, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2015, 2020, Oracle and/or its affiliates. All rights reserved.
  * ORACLE PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
  *
  *
@@ -29,7 +29,7 @@ import java.util.Random;
 import org.openjdk.jmh.annotations.Benchmark;
 import org.openjdk.jmh.annotations.Scope;
 import org.openjdk.jmh.annotations.State;
-import org.openjdk.jmh.annotations.Warmup;
+import org.openjdk.jmh.infra.Blackhole;
 
 /**
  * Benchmarks cost of Math intrinsics.
@@ -40,6 +40,7 @@ public class MathFunctionBenchmark extends BenchmarkBase {
     public static class ThreadState {
         double[] data = randomDoubles(100);
         double[] result = new double[100];
+        double k = data[0];
 
         static double[] randomDoubles(int len) {
             double[] data = new double[len];
@@ -52,7 +53,6 @@ public class MathFunctionBenchmark extends BenchmarkBase {
     }
 
     @Benchmark
-    @Warmup(iterations = 5)
     public void mathLog(ThreadState state) {
         double[] data = state.data;
         for (int i = 0; i < data.length; i++) {
@@ -62,7 +62,6 @@ public class MathFunctionBenchmark extends BenchmarkBase {
     }
 
     @Benchmark
-    @Warmup(iterations = 5)
     public void mathLog10(ThreadState state) {
         double[] data = state.data;
         for (int i = 0; i < data.length; i++) {
@@ -72,7 +71,6 @@ public class MathFunctionBenchmark extends BenchmarkBase {
     }
 
     @Benchmark
-    @Warmup(iterations = 5)
     public void mathSin(ThreadState state) {
         double[] data = state.data;
         for (int i = 0; i < data.length; i++) {
@@ -82,7 +80,6 @@ public class MathFunctionBenchmark extends BenchmarkBase {
     }
 
     @Benchmark
-    @Warmup(iterations = 5)
     public void mathCos(ThreadState state) {
         double[] data = state.data;
         for (int i = 0; i < data.length; i++) {
@@ -92,7 +89,6 @@ public class MathFunctionBenchmark extends BenchmarkBase {
     }
 
     @Benchmark
-    @Warmup(iterations = 5)
     public void mathTan(ThreadState state) {
         double[] data = state.data;
         for (int i = 0; i < data.length; i++) {
@@ -100,4 +96,14 @@ public class MathFunctionBenchmark extends BenchmarkBase {
             result[i] = Math.tan(data[i]);
         }
     }
+
+    @Benchmark
+    public void mathSqrt(ThreadState state, Blackhole blackhole) {
+        blackhole.consume(Math.sqrt(state.k));
+    }
+
+    @Benchmark
+    public void strictMathSqrt(ThreadState state, Blackhole blackhole) {
+        blackhole.consume(StrictMath.sqrt(state.k));
+    }
 }
