@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2005, 2019, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2005, 2020, Oracle and/or its affiliates. All rights reserved.
  * ORACLE PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
  *
  *
@@ -27,6 +27,7 @@ package sun.jvm.hotspot;
 import java.io.BufferedOutputStream;
 import java.io.BufferedReader;
 import java.io.ByteArrayOutputStream;
+import java.io.File;
 import java.io.FileInputStream;
 import java.io.FileOutputStream;
 import java.io.IOException;
@@ -74,11 +75,14 @@ import sun.jvm.hotspot.runtime.JavaVFrame;
 import sun.jvm.hotspot.runtime.Threads;
 import sun.jvm.hotspot.runtime.VM;
 import sun.jvm.hotspot.tools.ObjectHistogram;
+import sun.jvm.hotspot.tools.JMap;
 import sun.jvm.hotspot.tools.PMap;
 import sun.jvm.hotspot.tools.PStack;
 import sun.jvm.hotspot.tools.StackTrace;
+import sun.jvm.hotspot.tools.SysPropsDumper;
 import sun.jvm.hotspot.tools.jcore.ClassDump;
 import sun.jvm.hotspot.tools.jcore.ClassFilter;
+import sun.jvm.hotspot.tools.jcore.ClassWriter;
 import sun.jvm.hotspot.types.CIntegerType;
 import sun.jvm.hotspot.types.Field;
 import sun.jvm.hotspot.types.Type;
@@ -100,9 +104,6 @@ import sun.jvm.hotspot.utilities.ReversePtrs;
 import sun.jvm.hotspot.utilities.ReversePtrsAnalysis;
 import sun.jvm.hotspot.utilities.RobustOopDeterminator;
 import sun.jvm.hotspot.utilities.SystemDictionaryHelper;
-import sun.jvm.hotspot.utilities.soql.JSJavaFactory;
-import sun.jvm.hotspot.utilities.soql.JSJavaFactoryImpl;
-import sun.jvm.hotspot.utilities.soql.JSJavaScriptEngine;
 
 public class CommandProcessor {
 
@@ -124,7 +125,7 @@ public class CommandProcessor {
     }
 
     public static class NonBootFilter implements ClassFilter {
-        private HashMap emitted = new HashMap();
+        private HashMap<Symbol, InstanceKlass> emitted = new HashMap<>();
         public boolean canInclude(InstanceKlass kls) {
             if (kls.getClassLoader() == null) return false;
             if (emitted.get(kls.getName()) != null) {
@@ -152,7 +153,7 @@ public class CommandProcessor {
             return t;
         }
 
-        void add(String s, ArrayList t) {
+        void add(String s, ArrayList<String> t) {
             if (s.length() > 0) {
                 t.add(s);
             }
@@ -163,7 +164,7 @@ public class CommandProcessor {
 
             // check for quoting
             int quote = cmd.indexOf('"');
-            ArrayList t = new ArrayList();
+            ArrayList<String> t = new ArrayList<>();
             if (quote != -1) {
                 while (cmd.length() > 0) {
                     if (quote != -1) {
@@ -678,7 +679,7 @@ public class CommandProcessor {
                 } else if (tokens == 0) {
                     out.println("Available commands:");
                     Object[] keys = commands.keySet().toArray();
-                    Arrays.sort(keys, new Comparator() {
+                    Arrays.sort(keys, new Comparator<>() {
                              public int compare(Object o1, Object o2) {
                                  return o1.toString().compareTo(o2.toString());
                              }
@@ -989,8 +990,8 @@ public class CommandProcessor {
                 // be read back.
                 Iterator i = agent.getTypeDataBase().getTypes();
                 // Make sure the types are emitted in an order than can be read back in
-                HashSet emitted = new HashSet();
-                Stack pending = new Stack();
+                HashSet<String> emitted = new HashSet<>();
+                Stack<Type> pending = new Stack<>();
                 while (i.hasNext()) {
                     Type n = (Type)i.next();
                     if (emitted.contains(n.getName())) {
@@ -1201,7 +1202,7 @@ public class CommandProcessor {
                 }
             }
         },
-        new Command("intConstant", "intConstant [ name [ value ] ]", true) {
+        new Command("intConstant", "intConstant [ name [ value ] ]", false) {
             public void doit(Tokens t) {
                 if (t.countTokens() != 1 && t.countTokens() != 0 && t.countTokens() != 2) {
                     usage();
@@ -1224,7 +1225,7 @@ public class CommandProcessor {
                 }
             }
         },
-        new Command("longConstant", "longConstant [ name [ value ] ]", true) {
+        new Command("longConstant", "longConstant [ name [ value ] ]", false) {
             public void doit(Tokens t) {
                 if (t.countTokens() != 1 && t.countTokens() != 0 && t.countTokens() != 2) {
                     usage();
@@ -1247,7 +1248,7 @@ public class CommandProcessor {
                 }
             }
         },
-        new Command("field", "field [ type [ name fieldtype isStatic offset address ] ]", true) {
+        new Command("field", "field [ type [ name fieldtype isStatic offset address ] ]", false) {
             public void doit(Tokens t) {
                 if (t.countTokens() != 1 && t.countTokens() != 0 && t.countTokens() != 6) {
                     usage();
@@ -1319,7 +1320,7 @@ public class CommandProcessor {
                 }
             }
         },
-        new Command("type", "type [ type [ name super isOop isInteger isUnsigned size ] ]", true) {
+        new Command("type", "type [ type [ name super isOop isInteger isUnsigned size ] ]", false) {
             public void doit(Tokens t) {
                 if (t.countTokens() != 1 && t.countTokens() != 0 && t.countTokens() != 6) {
                     usage();
@@ -1385,8 +1386,8 @@ public class CommandProcessor {
                 } else {
                     Iterator i = agent.getTypeDataBase().getTypes();
                     // Make sure the types are emitted in an order than can be read back in
-                    HashSet emitted = new HashSet();
-                    Stack pending = new Stack();
+                    HashSet<String> emitted = new HashSet<>();
+                    Stack<Type> pending = new Stack<>();
                     while (i.hasNext()) {
                         Type n = (Type)i.next();
                         if (emitted.contains(n.getName())) {
@@ -1636,7 +1637,7 @@ public class CommandProcessor {
                 if (t.countTokens() != 0) {
                     usage();
                 } else {
-                    ArrayList nmethods = new ArrayList();
+                    ArrayList<NMethod> nmethods = new ArrayList<>();
                     Threads threads = VM.getVM().getThreads();
                     HTMLGenerator gen = new HTMLGenerator(false);
                     for (int i = 0; i < threads.getNumberOfThreads(); i++) {
@@ -1702,11 +1703,134 @@ public class CommandProcessor {
                 }
             }
         },
+        new Command("dumpclass", "dumpclass {address | name} [directory]", false) {
+            public void doit(Tokens t) {
+                int tokenCount = t.countTokens();
+                if (tokenCount != 1 && tokenCount != 2) {
+                    usage();
+                    return;
+                }
+
+                /* Find the InstanceKlass for specified class name or class address. */
+                InstanceKlass ik = null;
+                String classname = t.nextToken();
+                if (classname.startsWith("0x")) {
+                    // treat it as address
+                    VM vm = VM.getVM();
+                    Address addr = vm.getDebugger().parseAddress(classname);
+                    Metadata metadata = Metadata.instantiateWrapperFor(addr.addOffsetTo(0));
+                    if (metadata instanceof InstanceKlass) {
+                        ik = (InstanceKlass) metadata;
+                    } else {
+                        System.out.println("Specified address is not an InstanceKlass");
+                        return;
+                    }
+                } else {
+                    ik = SystemDictionaryHelper.findInstanceKlass(classname);
+                    if (ik == null) {
+                        System.out.println("class not found: " + classname);
+                        return;
+                    }
+                }
+
+                /* Compute filename for class. */
+                StringBuffer buf = new StringBuffer();
+                if (tokenCount > 1) {
+                    buf.append(t.nextToken());
+                } else {
+                    buf.append('.');
+                }
+                buf.append(File.separatorChar);
+                buf.append(ik.getName().asString().replace('/', File.separatorChar));
+                buf.append(".class");
+                String fileName = buf.toString();
+                File file = new File(fileName);
+
+                /* Dump the class file. */
+                try {
+                    int index = fileName.lastIndexOf(File.separatorChar);
+                    File dir = new File(fileName.substring(0, index));
+                    dir.mkdirs();
+                    try (FileOutputStream fos = new FileOutputStream(file)) {
+                        ClassWriter cw = new ClassWriter(ik, fos);
+                        cw.write();
+                    }
+                } catch (Exception e) {
+                    err.println("Error: " + e);
+                    if (verboseExceptions) {
+                        e.printStackTrace(err);
+                    }
+                }
+            }
+        },
+        new Command("sysprops", "sysprops", false) {
+            public void doit(Tokens t) {
+                if (t.countTokens() != 0) {
+                    usage();
+                    return;
+                }
+                SysPropsDumper sysProps = new SysPropsDumper();
+                sysProps.run();
+            }
+        },
+        new Command("dumpheap", "dumpheap [filename]", false) {
+            public void doit(Tokens t) {
+                if (t.countTokens() > 1) {
+                    usage();
+                } else {
+                    JMap jmap = new JMap();
+                    String filename;
+                    if (t.countTokens() == 1) {
+                        filename = t.nextToken();
+                    } else {
+                        filename = "heap.bin";;
+                    }
+                    try {
+                        jmap.writeHeapHprofBin(filename);
+                    } catch (Exception e) {
+                        err.println("Error: " + e);
+                        if (verboseExceptions) {
+                            e.printStackTrace(err);
+                        }
+                    }
+                }
+            }
+        },
+        new Command("class", "class name", false) {
+            public void doit(Tokens t) {
+                if (t.countTokens() != 1) {
+                    usage();
+                    return;
+                }
+                String classname = t.nextToken();
+                InstanceKlass ik = SystemDictionaryHelper.findInstanceKlass(classname);
+                if (ik == null) {
+                    System.out.println("class not found: " + classname);
+                } else {
+                    System.out.println(ik.getName().asString() + " @" + ik.getAddress());
+                }
+            }
+        },
+        new Command("classes", "classes", false) {
+            public void doit(Tokens t) {
+                if (t.countTokens() != 0) {
+                    usage();
+                    return;
+                }
+                ClassLoaderDataGraph cldg = VM.getVM().getClassLoaderDataGraph();
+                cldg.classesDo(new ClassLoaderDataGraph.ClassVisitor() {
+                        public void visit(Klass k) {
+                            System.out.println(k.getName().asString() + " @" + k.getAddress());
+                        }
+                    }
+                );
+            }
+        },
     };
 
     private boolean verboseExceptions = false;
-    private ArrayList history = new ArrayList();
-    private HashMap commands = new HashMap();
+    private ArrayList<String> history = new ArrayList<>();
+    private HashMap<String, Command> commands = new HashMap<>();
     private boolean doEcho = false;
 
     private Command findCommand(String key) {
@@ -1719,7 +1843,6 @@ public class CommandProcessor {
 
     private DebuggerInterface debugger;
     private HotSpotAgent agent;
-    private JSJavaScriptEngine jsengine;
     private BufferedReader in;
     private PrintStream out;
     private PrintStream err;
@@ -1731,62 +1854,7 @@ public class CommandProcessor {
 
     // called after debuggee attach
     private void postAttach() {
-        // create JavaScript engine and start it
-        try {
-            jsengine = new JSJavaScriptEngine() {
-                        private ObjectReader reader = new ObjectReader();
-                        private JSJavaFactory factory = new JSJavaFactoryImpl();
-                        public ObjectReader getObjectReader() {
-                            return reader;
-                        }
-                        public JSJavaFactory getJSJavaFactory() {
-                            return factory;
-                        }
-                        protected void quit() {
-                            debugger.detach();
-                            quit = true;
-                        }
-                        protected BufferedReader getInputReader() {
-                            return in;
-                        }
-                        protected PrintStream getOutputStream() {
-                            return out;
-                        }
-                        protected PrintStream getErrorStream() {
-                            return err;
-                        }
-                   };
-            try {
-                jsengine.defineFunction(this,
-                     this.getClass().getMethod("registerCommand",
-                                new Class[] {
-                                     String.class, String.class, String.class
-                                }));
-            } catch (NoSuchMethodException exp) {
-                  // should not happen, see below...!!
-                  exp.printStackTrace();
-            }
-            jsengine.start();
-        }
-        catch (Exception ex) {
-            System.out.println("Warning! JS Engine can't start, some commands will not be available.");
-            if (verboseExceptions) {
-                ex.printStackTrace(out);
-            }
-        }
-    }
-
-    public void registerCommand(String cmd, String usage, final String func) {
-        commands.put(cmd, new Command(cmd, usage, false) {
-                              public void doit(Tokens t) {
-                                  final int len = t.countTokens();
-                                  Object[] args = new Object[len];
-                                  for (int i = 0; i < len; i++) {
-                                      args[i] = t.nextToken();
-                                  }
-                                  jsengine.call(func, args);
-                              }
-                          });
+        // nothing for now..
     }
 
     public void setOutput(PrintStream o) {
