@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2002, 2017, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2002, 2020, Oracle and/or its affiliates. All rights reserved.
  * ORACLE PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
  *
  *
@@ -59,15 +59,15 @@ public class WindbgDebuggerLocal extends DebuggerBase implements WindbgDebugger
 
   // Symbol lookup support
   // This is a map of library names to DLLs
-  private Map nameToDllMap;
+  private Map<String, DLL> nameToDllMap;
 
   // C/C++ debugging support
-  private List/*<LoadObject>*/ loadObjects;
+  private List<LoadObject> loadObjects;
   private CDebugger cdbg;
 
   // thread access
-  private Map threadIntegerRegisterSet;
-  private List threadList;
+  private Map<Long, long[]> threadIntegerRegisterSet;
+  private List<ThreadProxy> threadList;
 
   // windbg native interface pointers
 
@@ -137,7 +137,7 @@ public class WindbgDebuggerLocal extends DebuggerBase implements WindbgDebugger
   }
 
   /** From the Debugger interface via JVMDebugger */
-  public List getProcessList() throws DebuggerException {
+  public List<ProcessInfo> getProcessList() throws DebuggerException {
     return null;
   }
 
@@ -158,7 +158,7 @@ public class WindbgDebuggerLocal extends DebuggerBase implements WindbgDebugger
     isCore = true;
   }
 
-  public List getLoadObjectList() {
+  public List<LoadObject> getLoadObjectList() {
     requireAttach();
     return loadObjects;
   }
@@ -341,7 +341,7 @@ public class WindbgDebuggerLocal extends DebuggerBase implements WindbgDebugger
 
   private synchronized void setThreadIntegerRegisterSet(long threadId,
                                                long[] regs) {
-    threadIntegerRegisterSet.put(new Long(threadId), regs);
+    threadIntegerRegisterSet.put(threadId, regs);
   }
 
   private synchronized void addThread(long sysId) {
@@ -351,10 +351,10 @@ public class WindbgDebuggerLocal extends DebuggerBase implements WindbgDebugger
   public synchronized long[] getThreadIntegerRegisterSet(long threadId)
     throws DebuggerException {
     requireAttach();
-    return (long[]) threadIntegerRegisterSet.get(new Long(threadId));
+    return (long[]) threadIntegerRegisterSet.get(threadId);
   }
 
-  public synchronized List getThreadList() throws DebuggerException {
+  public synchronized List<ThreadProxy> getThreadList() throws DebuggerException {
     requireAttach();
     return threadList;
   }
@@ -438,10 +438,10 @@ public class WindbgDebuggerLocal extends DebuggerBase implements WindbgDebugger
 
   private void attachInit() {
     checkAttached();
-    loadObjects = new ArrayList();
-    nameToDllMap = new HashMap();
-    threadIntegerRegisterSet = new HashMap();
-    threadList = new ArrayList();
+    loadObjects = new ArrayList<>();
+    nameToDllMap = new HashMap<>();
+    threadIntegerRegisterSet = new HashMap<>();
+    threadList = new ArrayList<>();
   }
 
   private void resetNativePointers() {
@@ -510,7 +510,7 @@ public class WindbgDebuggerLocal extends DebuggerBase implements WindbgDebugger
     static {
 
      /*
-      * sawindbg.dll depends on dbgeng.dll which itself depends on
+      * saproc.dll depends on dbgeng.dll which itself depends on
       * dbghelp.dll. We have to make sure that the dbgeng.dll and
       * dbghelp.dll that we load are compatible with each other. We
       * load both of those libraries from the same directory based
@@ -538,18 +538,18 @@ public class WindbgDebuggerLocal extends DebuggerBase implements WindbgDebugger
 
     String dbgengPath   = null;
     String dbghelpPath  = null;
-    String sawindbgPath = null;
-    List   searchList   = new ArrayList();
+    String saprocPath = null;
+    List<String> searchList = new ArrayList<>();
 
     boolean loadLibraryDEBUG =
         System.getProperty("sun.jvm.hotspot.loadLibrary.DEBUG") != null;
 
     {
-      // First place to search is co-located with sawindbg.dll in
+      // First place to search is co-located with saproc.dll in
       // $JAVA_HOME/jre/bin (java.home property is set to $JAVA_HOME/jre):
       searchList.add(System.getProperty("java.home") + File.separator + "bin");
-      sawindbgPath = (String) searchList.get(0) + File.separator +
-          "sawindbg.dll";
+      saprocPath = (String) searchList.get(0) + File.separator +
+          "saproc.dll";
 
       // second place to search is specified by an environment variable:
       String DTFWHome = System.getenv("DEBUGGINGTOOLSFORWINDOWS");
@@ -646,11 +646,11 @@ public class WindbgDebuggerLocal extends DebuggerBase implements WindbgDebugger
     }
     System.load(dbgengPath);
 
-    // Now, load sawindbg.dll
+    // Now, load saproc.dll
     if (loadLibraryDEBUG) {
-      System.err.println("DEBUG: loading '" + sawindbgPath + "'.");
+      System.err.println("DEBUG: loading '" + saprocPath + "'.");
     }
-    System.load(sawindbgPath);
+    System.load(saprocPath);
 
     // where do I find '.exe', '.dll' files?
     imagePath = System.getProperty("sun.jvm.hotspot.debugger.windbg.imagePath");
