@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2008, 2013, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2008, 2019, Oracle and/or its affiliates. All rights reserved.
  * ORACLE PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
  *
  *
@@ -33,22 +33,22 @@ import sun.nio.cs.SingleByte;
 import sun.nio.cs.HistoricallyNamedCharset;
 import static sun.nio.cs.CharsetMapping.*;
 
-public class ISO_8859_11 extends Charset implements HistoricallyNamedCharset
+public class ISO_8859_15 extends Charset implements HistoricallyNamedCharset
 {
-    public ISO_8859_11() {
-        super("x-iso-8859-11", StandardCharsets.aliases_ISO_8859_11());
+    public ISO_8859_15() {
+        super("ISO-8859-15", StandardCharsets.aliases_ISO_8859_15());
     }
 
     public String historicalName() {
-        return "x-iso-8859-11";
+        return "ISO8859_15";
     }
 
     public boolean contains(Charset cs) {
-        return ((cs.name().equals("US-ASCII")) || (cs instanceof ISO_8859_11));
+        return ((cs.name().equals("US-ASCII")) || (cs instanceof ISO_8859_15));
     }
 
     public CharsetDecoder newDecoder() {
-        return new SingleByte.Decoder(this, b2c, true);
+        return new SingleByte.Decoder(this, b2c, true, false);
     }
 
     public CharsetEncoder newEncoder() {
@@ -60,18 +60,18 @@ public class ISO_8859_11 extends Charset implements HistoricallyNamedCharset
         "\u0088\u0089\u008A\u008B\u008C\u008D\u008E\u008F" +      // 0x88 - 0x8f
         "\u0090\u0091\u0092\u0093\u0094\u0095\u0096\u0097" +      // 0x90 - 0x97
         "\u0098\u0099\u009A\u009B\u009C\u009D\u009E\u009F" +      // 0x98 - 0x9f
-        "\u00A0\u0E01\u0E02\u0E03\u0E04\u0E05\u0E06\u0E07" +      // 0xa0 - 0xa7
-        "\u0E08\u0E09\u0E0A\u0E0B\u0E0C\u0E0D\u0E0E\u0E0F" +      // 0xa8 - 0xaf
-        "\u0E10\u0E11\u0E12\u0E13\u0E14\u0E15\u0E16\u0E17" +      // 0xb0 - 0xb7
-        "\u0E18\u0E19\u0E1A\u0E1B\u0E1C\u0E1D\u0E1E\u0E1F" +      // 0xb8 - 0xbf
-        "\u0E20\u0E21\u0E22\u0E23\u0E24\u0E25\u0E26\u0E27" +      // 0xc0 - 0xc7
-        "\u0E28\u0E29\u0E2A\u0E2B\u0E2C\u0E2D\u0E2E\u0E2F" +      // 0xc8 - 0xcf
-        "\u0E30\u0E31\u0E32\u0E33\u0E34\u0E35\u0E36\u0E37" +      // 0xd0 - 0xd7
-        "\u0E38\u0E39\u0E3A\uFFFD\uFFFD\uFFFD\uFFFD\u0E3F" +      // 0xd8 - 0xdf
-        "\u0E40\u0E41\u0E42\u0E43\u0E44\u0E45\u0E46\u0E47" +      // 0xe0 - 0xe7
-        "\u0E48\u0E49\u0E4A\u0E4B\u0E4C\u0E4D\u0E4E\u0E4F" +      // 0xe8 - 0xef
-        "\u0E50\u0E51\u0E52\u0E53\u0E54\u0E55\u0E56\u0E57" +      // 0xf0 - 0xf7
-        "\u0E58\u0E59\u0E5A\u0E5B\uFFFD\uFFFD\uFFFD\uFFFD" +      // 0xf8 - 0xff
+        "\u00A0\u00A1\u00A2\u00A3\u20AC\u00A5\u0160\u00A7" +      // 0xa0 - 0xa7
+        "\u0161\u00A9\u00AA\u00AB\u00AC\u00AD\u00AE\u00AF" +      // 0xa8 - 0xaf
+        "\u00B0\u00B1\u00B2\u00B3\u017D\u00B5\u00B6\u00B7" +      // 0xb0 - 0xb7
+        "\u017E\u00B9\u00BA\u00BB\u0152\u0153\u0178\u00BF" +      // 0xb8 - 0xbf
+        "\u00C0\u00C1\u00C2\u00C3\u00C4\u00C5\u00C6\u00C7" +      // 0xc0 - 0xc7
+        "\u00C8\u00C9\u00CA\u00CB\u00CC\u00CD\u00CE\u00CF" +      // 0xc8 - 0xcf
+        "\u00D0\u00D1\u00D2\u00D3\u00D4\u00D5\u00D6\u00D7" +      // 0xd0 - 0xd7
+        "\u00D8\u00D9\u00DA\u00DB\u00DC\u00DD\u00DE\u00DF" +      // 0xd8 - 0xdf
+        "\u00E0\u00E1\u00E2\u00E3\u00E4\u00E5\u00E6\u00E7" +      // 0xe0 - 0xe7
+        "\u00E8\u00E9\u00EA\u00EB\u00EC\u00ED\u00EE\u00EF" +      // 0xe8 - 0xef
+        "\u00F0\u00F1\u00F2\u00F3\u00F4\u00F5\u00F6\u00F7" +      // 0xf0 - 0xf7
+        "\u00F8\u00F9\u00FA\u00FB\u00FC\u00FD\u00FE\u00FF" +      // 0xf8 - 0xff
         "\u0000\u0001\u0002\u0003\u0004\u0005\u0006\u0007" +      // 0x00 - 0x07
         "\b\t\n\u000B\f\r\u000E\u000F" +      // 0x08 - 0x0f
         "\u0010\u0011\u0012\u0013\u0014\u0015\u0016\u0017" +      // 0x10 - 0x17
