@@ -25,9 +25,6 @@
 
 package sun.nio.fs;
 
-import java.security.AccessController;
-import java.security.PrivilegedAction;
-
 /**
  * Unix system and library calls.
  */
@@ -36,7 +33,7 @@ class UnixNativeDispatcher {
     protected UnixNativeDispatcher() { }
 
     // returns a NativeBuffer containing the given path
-    private static NativeBuffer copyToNativeBuffer(UnixPath path) {
+    static NativeBuffer copyToNativeBuffer(UnixPath path) {
         byte[] cstr = path.getByteArrayForSysCalls();
         int size = cstr.length + 1;
         NativeBuffer buffer = NativeBuffers.getNativeBufferFromCache(size);
@@ -101,25 +98,14 @@ class UnixNativeDispatcher {
     private static native void close0(int fd);
 
     /**
-     * FILE* fopen(const char *filename, const char* mode);
+     * void rewind(FILE* stream);
      */
-    static long fopen(UnixPath filename, String mode) throws UnixException {
-        NativeBuffer pathBuffer = copyToNativeBuffer(filename);
-        NativeBuffer modeBuffer = NativeBuffers.asNativeBuffer(Util.toBytes(mode));
-        try {
-            return fopen0(pathBuffer.address(), modeBuffer.address());
-        } finally {
-            modeBuffer.release();
-            pathBuffer.release();
-        }
-    }
-    private static native long fopen0(long pathAddress, long modeAddress)
-        throws UnixException;
+    static native void rewind(long stream) throws UnixException;
 
     /**
-     * fclose(FILE* stream)
+     * ssize_t getline(char **lineptr, size_t *n, FILE *stream);
      */
-    static native void fclose(long stream) throws UnixException;
+    static native int getlinelen(long stream) throws UnixException;
 
     /**
      * link(const char* existing, const char* new)
@@ -421,6 +407,11 @@ class UnixNativeDispatcher {
      */
     static native void futimes(int fd, long times0, long times1) throws UnixException;
 
+    /**
+     * futimens(int fildes, const struct timespec times[2])
+     */
+    static native void futimens(int fd, long times0, long times1) throws UnixException;
+
     /**
      * lutimes(const char* path, const struct timeval times[2])
      */
@@ -567,25 +558,6 @@ class UnixNativeDispatcher {
     private static native void statvfs0(long pathAddress, UnixFileStoreAttributes attrs)
         throws UnixException;
 
-    /**
-     * long int pathconf(const char *path, int name);
-     */
-    static long pathconf(UnixPath path, int name) throws UnixException {
-        NativeBuffer buffer = copyToNativeBuffer(path);
-        try {
-            return pathconf0(buffer.address(), name);
-        } finally {
-            buffer.release();
-        }
-    }
-    private static native long pathconf0(long pathAddress, int name)
-        throws UnixException;
-
-    /**
-     * long fpathconf(int fildes, int name);
-     */
-    static native long fpathconf(int filedes, int name) throws UnixException;
-
     /**
      * char* strerror(int errnum)
      */
@@ -596,7 +568,8 @@ class UnixNativeDispatcher {
      */
     private static final int SUPPORTS_OPENAT        = 1 << 1;  // syscalls
     private static final int SUPPORTS_FUTIMES       = 1 << 2;
-    private static final int SUPPORTS_LUTIMES       = 1 << 4;
+    private static final int SUPPORTS_FUTIMENS      = 1 << 4;
+    private static final int SUPPORTS_LUTIMES       = 1 << 8;
     private static final int SUPPORTS_BIRTHTIME     = 1 << 16; // other features
     private static final int capabilities;
 
@@ -614,6 +587,13 @@ class UnixNativeDispatcher {
         return (capabilities & SUPPORTS_FUTIMES) != 0;
     }
 
+    /**
+     * Supports futimens
+     */
+    static boolean futimensSupported() {
+        return (capabilities & SUPPORTS_FUTIMENS) != 0;
+    }
+
     /**
      * Supports lutimes
      */
@@ -630,11 +610,7 @@ class UnixNativeDispatcher {
 
     private static native int init();
     static {
-        AccessController.doPrivileged(new PrivilegedAction<>() {
-            public Void run() {
-                System.loadLibrary("nio");
-                return null;
-        }});
+        jdk.internal.loader.BootLoader.loadLibrary("nio");
         capabilities = init();
     }
 }
