@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2003, 2023, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2003, 2020, Oracle and/or its affiliates. All rights reserved.
  * ORACLE PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
  *
  *
@@ -26,8 +26,6 @@
 package sun.security.rsa;
 
 import java.io.IOException;
-import java.io.InvalidObjectException;
-import java.io.ObjectInputStream;
 import java.math.BigInteger;
 
 import java.security.*;
@@ -44,7 +42,7 @@ import sun.security.rsa.RSAUtil.KeyType;
  * RSA private key implementation for "RSA", "RSASSA-PSS" algorithms in CRT form.
  * For non-CRT private keys, see RSAPrivateKeyImpl. We need separate classes
  * to ensure correct behavior in instanceof checks, etc.
- * <p>
+ *
  * Note: RSA keys must be at least 512 bits long
  *
  * @see RSAPrivateKeyImpl
@@ -56,6 +54,7 @@ import sun.security.rsa.RSAUtil.KeyType;
 public final class RSAPrivateCrtKeyImpl
         extends PKCS8Key implements RSAPrivateCrtKey {
 
+    @java.io.Serial
     private static final long serialVersionUID = -1326088454257084918L;
 
     private BigInteger n;       // modulus
@@ -75,52 +74,30 @@ public final class RSAPrivateCrtKeyImpl
     private transient AlgorithmParameterSpec keyParams;
 
     /**
-     * Generate a new RSAPrivate(Crt)Key from the specified type,
-     * format and encoding. Returns a CRT key if possible and a non-CRT
-     * key otherwise.
-     * Also used by SunPKCS11 provider.
+     * Generate a new key from its encoding. Returns a CRT key if possible
+     * and a non-CRT key otherwise. Used by RSAKeyFactory.
      */
-    public static RSAPrivateKey newKey(KeyType type, String format,
-            byte[] encoded) throws InvalidKeyException {
+    public static RSAPrivateKey newKey(byte[] encoded)
+            throws InvalidKeyException {
         if (encoded == null || encoded.length == 0) {
             throw new InvalidKeyException("Missing key encoding");
         }
-        switch (format) {
-            case "PKCS#8":
-                RSAPrivateCrtKeyImpl key = new RSAPrivateCrtKeyImpl(encoded);
-                RSAKeyFactory.checkKeyAlgo(key, type.keyAlgo);
-                // check all CRT-specific components are available, if any one
-                // missing, return a non-CRT key instead
-                if ((key.getPublicExponent().signum() == 0) ||
-                        (key.getPrimeExponentP().signum() == 0) ||
-                        (key.getPrimeExponentQ().signum() == 0) ||
-                        (key.getPrimeP().signum() == 0) ||
-                        (key.getPrimeQ().signum() == 0) ||
-                        (key.getCrtCoefficient().signum() == 0)) {
-                    return new RSAPrivateKeyImpl(key.type, key.keyParams,
-                            key.getModulus(), key.getPrivateExponent());
-                } else {
-                    return key;
-                }
-            case "PKCS#1":
-                try {
-                    BigInteger[] comps = parseASN1(encoded);
-                    if ((comps[1].signum() == 0) || (comps[3].signum() == 0) ||
-                            (comps[4].signum() == 0) || (comps[5].signum() == 0) ||
-                            (comps[6].signum() == 0) || (comps[7].signum() == 0)) {
-                        return new RSAPrivateKeyImpl(type, null, comps[0],
-                                comps[2]);
-                    } else {
-                        return new RSAPrivateCrtKeyImpl(type, null, comps[0],
-                                comps[1], comps[2], comps[3], comps[4], comps[5],
-                                comps[6], comps[7]);
-                    }
-                } catch (IOException ioe) {
-                    throw new InvalidKeyException("Invalid PKCS#1 encoding", ioe);
-                }
-            default:
-                throw new InvalidKeyException("Unsupported RSA Private(Crt)Key "
-                        + "format: " + format);
+        RSAPrivateCrtKeyImpl key = new RSAPrivateCrtKeyImpl(encoded);
+        // check all CRT-specific components are available, if any one
+        // missing, return a non-CRT key instead
+        if ((key.getPublicExponent().signum() == 0) ||
+            (key.getPrimeExponentP().signum() == 0) ||
+            (key.getPrimeExponentQ().signum() == 0) ||
+            (key.getPrimeP().signum() == 0) ||
+            (key.getPrimeQ().signum() == 0) ||
+            (key.getCrtCoefficient().signum() == 0)) {
+            return new RSAPrivateKeyImpl(
+                key.type, key.keyParams,
+                key.getModulus(),
+                key.getPrivateExponent()
+            );
+        } else {
+            return key;
         }
     }
 
@@ -149,12 +126,9 @@ public final class RSAPrivateCrtKeyImpl
     /**
      * Construct a key from its encoding. Called from newKey above.
      */
-    private RSAPrivateCrtKeyImpl(byte[] encoded) throws InvalidKeyException {
-        if (encoded == null || encoded.length == 0) {
-            throw new InvalidKeyException("Missing key encoding");
-        }
-
-        decode(encoded);
+    RSAPrivateCrtKeyImpl(byte[] encoded) throws InvalidKeyException {
+        super(encoded);
+        parseKeyBits();
         RSAKeyFactory.checkRSAProviderKeyLengths(n.bitLength(), e);
         try {
             // check the validity of oid and params
@@ -284,67 +258,39 @@ public final class RSAPrivateCrtKeyImpl
                + "\n  modulus: " + n + "\n  private exponent: " + d;
     }
 
-    // utility method for parsing DER encoding of RSA private keys in PKCS#1
-    // format as defined in RFC 8017 Appendix A.1.2, i.e. SEQ of version, n,
-    // e, d, p, q, pe, qe, and coeff, and return the parsed components.
-    private static BigInteger[] parseASN1(byte[] raw) throws IOException {
-        DerValue derValue = new DerValue(raw);
-        if (derValue.tag != DerValue.tag_Sequence) {
-            throw new IOException("Not a SEQUENCE");
-        }
-        int version = derValue.data.getInteger();
-        if (version != 0) {
-            throw new IOException("Version must be 0");
-        }
-
-        BigInteger[] result = new BigInteger[8]; // n, e, d, p, q, pe, qe, coeff
-        /*
-         * Some implementations do not correctly encode ASN.1 INTEGER values
-         * in 2's complement format, resulting in a negative integer when
-         * decoded. Correct the error by converting it to a positive integer.
-         *
-         * See CR 6255949
-         */
-        for (int i = 0; i < result.length; i++) {
-            result[i] = derValue.data.getPositiveBigInteger();
-        }
-        if (derValue.data.available() != 0) {
-            throw new IOException("Extra data available");
-        }
-        return result;
-    }
-
-    /**
-     * Parse the key. Called by PKCS8Key.
-     */
-    protected void parseKeyBits() throws InvalidKeyException {
+    private void parseKeyBits() throws InvalidKeyException {
         try {
-            BigInteger[] comps = parseASN1(key);
-            n = comps[0];
-            e = comps[1];
-            d = comps[2];
-            p = comps[3];
-            q = comps[4];
-            pe = comps[5];
-            qe = comps[6];
-            coeff = comps[7];
+            DerInputStream in = new DerInputStream(key);
+            DerValue derValue = in.getDerValue();
+            if (derValue.tag != DerValue.tag_Sequence) {
+                throw new IOException("Not a SEQUENCE");
+            }
+            DerInputStream data = derValue.data;
+            int version = data.getInteger();
+            if (version != 0) {
+                throw new IOException("Version must be 0");
+            }
+
+            /*
+             * Some implementations do not correctly encode ASN.1 INTEGER values
+             * in 2's complement format, resulting in a negative integer when
+             * decoded. Correct the error by converting it to a positive integer.
+             *
+             * See CR 6255949
+             */
+            n = data.getPositiveBigInteger();
+            e = data.getPositiveBigInteger();
+            d = data.getPositiveBigInteger();
+            p = data.getPositiveBigInteger();
+            q = data.getPositiveBigInteger();
+            pe = data.getPositiveBigInteger();
+            qe = data.getPositiveBigInteger();
+            coeff = data.getPositiveBigInteger();
+            if (derValue.data.available() != 0) {
+                throw new IOException("Extra data available");
+            }
         } catch (IOException e) {
             throw new InvalidKeyException("Invalid RSA private key", e);
         }
     }
-
-    /**
-     * Restores the state of this object from the stream.
-     * <p>
-     * Deserialization of this object is not supported.
-     *
-     * @param  stream the {@code ObjectInputStream} from which data is read
-     * @throws IOException if an I/O error occurs
-     * @throws ClassNotFoundException if a serialized class cannot be loaded
-     */
-    private void readObject(ObjectInputStream stream)
-            throws IOException, ClassNotFoundException {
-        throw new InvalidObjectException(
-                "RSAPrivateCrtKeyImpl keys are not directly deserializable");
-    }
 }
