@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2012, 2023, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2012, 2020, Oracle and/or its affiliates. All rights reserved.
  * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
  *
  * This code is free software; you can redistribute it and/or modify it
@@ -72,7 +72,7 @@ public class TimeZoneNames_nl extends TimeZoneNamesBundle {
     protected final Object[][] getContents() {
         final String[] GMT = new String[] {
                "Greenwich Mean Time",
-               "",
+               "GMT",
                "",
                "",
                "",
@@ -182,14 +182,6 @@ public class TimeZoneNames_nl extends TimeZoneNamesBundle {
                "",
                "",
             };
-        final String[] Casey = new String[] {
-               "Casey tijd",
-               "",
-               "",
-               "",
-               "",
-               "",
-            };
         final String[] Chile = new String[] {
                "Chileense standaardtijd",
                "",
@@ -296,11 +288,11 @@ public class TimeZoneNames_nl extends TimeZoneNamesBundle {
             };
         final String[] Alaska = new String[] {
                "Alaska-standaardtijd",
-               "",
+               "AKST",
                "Alaska-zomertijd",
-               "",
+               "AKDT",
                "Alaska-tijd",
-               "",
+               "AKT",
             };
         final String[] Amazon = new String[] {
                "Amazone-standaardtijd",
@@ -535,12 +527,12 @@ public class TimeZoneNames_nl extends TimeZoneNamesBundle {
                "",
             };
         final String[] Norfolk = new String[] {
+               "Norfolkeilandse standaardtijd",
+               "",
+               "Norfolkeilandse zomertijd",
+               "",
                "Norfolkeilandse tijd",
                "",
-               "",
-               "",
-               "",
-               "",
             };
         final String[] Noronha = new String[] {
                "Fernando de Noronha-standaardtijd",
@@ -608,11 +600,11 @@ public class TimeZoneNames_nl extends TimeZoneNamesBundle {
             };
         final String[] Atlantic = new String[] {
                "Atlantic-standaardtijd",
-               "",
+               "AST",
                "Atlantic-zomertijd",
-               "",
+               "ADT",
                "Atlantic-tijd",
-               "",
+               "AT",
             };
         final String[] Brasilia = new String[] {
                "Braziliaanse standaardtijd",
@@ -727,7 +719,7 @@ public class TimeZoneNames_nl extends TimeZoneNamesBundle {
                "",
             };
         final String[] Galapagos = new String[] {
-               "Galapagoseilandse standaardtijd",
+               "Galapagoseilandse tijd",
                "",
                "",
                "",
@@ -798,14 +790,6 @@ public class TimeZoneNames_nl extends TimeZoneNamesBundle {
                "Mauritiaanse tijd",
                "",
             };
-        final String[] Pyongyang = new String[] {
-               "Pyongyang-tijd",
-               "",
-               "",
-               "",
-               "",
-               "",
-            };
         final String[] Singapore = new String[] {
                "Singaporese standaardtijd",
                "",
@@ -822,6 +806,14 @@ public class TimeZoneNames_nl extends TimeZoneNamesBundle {
                "",
                "",
             };
+        final String[] Volgograd = new String[] {
+               "Wolgograd-standaardtijd",
+               "",
+               "Wolgograd-zomertijd",
+               "",
+               "Wolgograd-tijd",
+               "",
+            };
         final String[] Azerbaijan = new String[] {
                "Azerbeidzjaanse standaardtijd",
                "",
@@ -846,14 +838,6 @@ public class TimeZoneNames_nl extends TimeZoneNamesBundle {
                "Kaapverdische tijd",
                "",
             };
-        final String[] Choibalsan = new String[] {
-               "Tsjojbalsan-standaardtijd",
-               "",
-               "Tsjojbalsan-zomertijd",
-               "",
-               "Tsjojbalsan-tijd",
-               "",
-            };
         final String[] East_Timor = new String[] {
                "Oost-Timorese tijd",
                "",
@@ -1072,27 +1056,27 @@ public class TimeZoneNames_nl extends TimeZoneNamesBundle {
             };
         final String[] America_Central = new String[] {
                "Central-standaardtijd",
-               "",
+               "CST",
                "Central-zomertijd",
-               "",
+               "CDT",
                "Central-tijd",
-               "",
+               "CT",
             };
         final String[] America_Eastern = new String[] {
                "Eastern-standaardtijd",
-               "",
+               "EST",
                "Eastern-zomertijd",
-               "",
+               "EDT",
                "Eastern-tijd",
-               "",
+               "ET",
             };
         final String[] America_Pacific = new String[] {
                "Pacific-standaardtijd",
-               "",
+               "PST",
                "Pacific-zomertijd",
-               "",
+               "PDT",
                "Pacific-tijd",
-               "",
+               "PT",
             };
         final String[] French_Southern = new String[] {
                "Franse zuidelijke en Antarctische tijd",
@@ -1112,11 +1096,11 @@ public class TimeZoneNames_nl extends TimeZoneNamesBundle {
             };
         final String[] Hawaii_Aleutian = new String[] {
                "Hawaii-Aleoetische standaardtijd",
-               "",
+               "HAST",
                "Hawaii-Aleoetische zomertijd",
-               "",
+               "HADT",
                "Hawaii-Aleoetische tijd",
-               "",
+               "HAT",
             };
         final String[] Phoenix_Islands = new String[] {
                "Phoenixeilandse tijd",
@@ -1136,11 +1120,11 @@ public class TimeZoneNames_nl extends TimeZoneNamesBundle {
             };
         final String[] America_Mountain = new String[] {
                "Mountain-standaardtijd",
-               "",
+               "MST",
                "Mountain-zomertijd",
-               "",
+               "MDT",
                "Mountain-tijd",
-               "",
+               "MT",
             };
         final String[] Marshall_Islands = new String[] {
                "Marshalleilandse tijd",
@@ -1166,14 +1150,6 @@ public class TimeZoneNames_nl extends TimeZoneNamesBundle {
                "",
                "",
             };
-        final String[] Argentina_Western = new String[] {
-               "West-Argentijnse standaardtijd",
-               "",
-               "West-Argentijnse zomertijd",
-               "",
-               "West-Argentijnse tijd",
-               "",
-            };
         final String[] Australia_Central = new String[] {
                "Midden-Australische standaardtijd",
                "",
@@ -1285,7 +1261,6 @@ public class TimeZoneNames_nl extends TimeZoneNamesBundle {
             { "America/St_Johns", Newfoundland },
             { "Europe/Paris", Europe_Central },
             { "GMT", GMT },
-            { "Africa/Casablanca", Europe_Western },
             { "Asia/Jerusalem", Israel },
             { "Asia/Tokyo", Japan },
             { "Europe/Bucharest", Europe_Eastern },
@@ -1303,20 +1278,8 @@ public class TimeZoneNames_nl extends TimeZoneNamesBundle {
             { "CTT", China },
             { "EAT", Africa_Eastern },
             { "ECT", Europe_Central },
-            { "EST", America_Eastern },
-            { "HST",
-                new String[] {
-                    "Hawaii-Aleoetische standaardtijd",
-                    "HST",
-                    "Hawaii-Aleoetische zomertijd",
-                    "HDT",
-                    "Hawaii-Aleoetische tijd",
-                    "HST",
-                }
-            },
             { "JST", Japan },
             { "MIT", Apia },
-            { "MST", America_Mountain },
             { "NET", Armenia },
             { "NST", New_Zealand },
             { "PLT", Pakistan },
@@ -1439,6 +1402,7 @@ public class TimeZoneNames_nl extends TimeZoneNamesBundle {
             { "Asia/Katmandu", Nepal },
             { "Asia/Khandyga", Yakutsk },
             { "Asia/Makassar", Indonesia_Central },
+            { "Asia/Qostanay", Kazakhstan_Eastern },
             { "Asia/Sakhalin", Sakhalin },
             { "Asia/Tashkent", Uzbekistan },
             { "Asia/Ust-Nera", Vladivostok },
@@ -1447,7 +1411,7 @@ public class TimeZoneNames_nl extends TimeZoneNamesBundle {
             { "Europe/Dublin",
                 new String[] {
                     "Greenwich Mean Time",
-                    "",
+                    "GMT",
                     "Ierse standaardtijd",
                     "",
                     "",
@@ -1459,7 +1423,7 @@ public class TimeZoneNames_nl extends TimeZoneNamesBundle {
             { "Europe/London",
                 new String[] {
                     "Greenwich Mean Time",
-                    "",
+                    "GMT",
                     "Britse zomertijd",
                     "",
                     "",
@@ -1521,8 +1485,8 @@ public class TimeZoneNames_nl extends TimeZoneNamesBundle {
             { "Asia/Hong_Kong", Hong_Kong },
             { "Asia/Kamchatka", Kamchatka },
             { "Asia/Pontianak", Indonesia_Western },
-            { "Asia/Pyongyang", Pyongyang },
-            { "Asia/Qyzylorda", Kazakhstan_Eastern },
+            { "Asia/Pyongyang", Korea },
+            { "Asia/Qyzylorda", Kazakhstan_Western },
             { "Asia/Samarkand", Uzbekistan },
             { "Asia/Singapore", Singapore },
             { "Asia/Vientiane", Indochina },
@@ -1530,7 +1494,7 @@ public class TimeZoneNames_nl extends TimeZoneNamesBundle {
             { "Europe/Belfast",
                 new String[] {
                     "Greenwich Mean Time",
-                    "",
+                    "GMT",
                     "Britse zomertijd",
                     "",
                     "",
@@ -1554,14 +1518,13 @@ public class TimeZoneNames_nl extends TimeZoneNamesBundle {
             { "Pacific/Wallis", Wallis },
             { "Africa/Blantyre", Africa_Central },
             { "Africa/Djibouti", Africa_Eastern },
-            { "Africa/El_Aaiun", Europe_Western },
             { "Africa/Freetown", GMT },
             { "Africa/Gaborone", Africa_Central },
             { "Africa/Khartoum", Africa_Central },
             { "Africa/Kinshasa", Africa_Western },
             { "Africa/Monrovia", GMT },
             { "Africa/Ndjamena", Africa_Western },
-            { "Africa/Sao_Tome", Africa_Western },
+            { "Africa/Sao_Tome", GMT },
             { "Africa/Timbuktu", GMT },
             { "Africa/Windhoek", Africa_Central },
             { "America/Antigua", Atlantic },
@@ -1581,12 +1544,12 @@ public class TimeZoneNames_nl extends TimeZoneNamesBundle {
             { "America/Moncton", Atlantic },
             { "America/Nipigon", America_Eastern },
             { "America/Noronha", Noronha },
-            { "America/Ojinaga", America_Central },
+            { "America/Ojinaga", America_Mountain },
             { "America/Tijuana", America_Pacific },
             { "America/Toronto", America_Eastern },
             { "America/Tortola", Atlantic },
             { "America/Yakutat", Alaska },
-            { "Asia/Choibalsan", Choibalsan },
+            { "Asia/Choibalsan", Mongolia },
             { "Asia/Phnom_Penh", Indochina },
             { "Atlantic/Azores", Azores },
             { "Atlantic/Canary", Europe_Western },
@@ -1631,7 +1594,7 @@ public class TimeZoneNames_nl extends TimeZoneNamesBundle {
             { "America/St_Kitts", Atlantic },
             { "America/St_Lucia", Atlantic },
             { "America/Winnipeg", America_Central },
-            { "Antarctica/Casey", Casey },
+            { "Antarctica/Casey", Australia_Western },
             { "Antarctica/Davis", Davis },
             { "Antarctica/Syowa", Syowa },
             { "Antarctica/Troll", GMT },
@@ -1652,7 +1615,7 @@ public class TimeZoneNames_nl extends TimeZoneNamesBundle {
             { "Europe/Mariehamn", Europe_Eastern },
             { "Europe/Podgorica", Europe_Central },
             { "Europe/Stockholm", Europe_Central },
-            { "Europe/Volgograd", Moscow },
+            { "Europe/Volgograd", Volgograd },
             { "Indian/Christmas", Christmas },
             { "Indian/Kerguelen", French_Southern },
             { "Indian/Mauritius", Mauritius },
@@ -1667,7 +1630,7 @@ public class TimeZoneNames_nl extends TimeZoneNamesBundle {
             { "America/Araguaina", Brasilia },
             { "America/Boa_Vista", Amazon },
             { "America/Catamarca", Argentina },
-            { "America/Chihuahua", America_Central },
+            { "America/Chihuahua", Mexico_Pacific },
             { "America/Fortaleza", Brasilia },
             { "America/Glace_Bay", Atlantic },
             { "America/Goose_Bay", Atlantic },
@@ -1784,7 +1747,7 @@ public class TimeZoneNames_nl extends TimeZoneNamesBundle {
             { "timezone.excity.Asia/Oral", "Oral" },
             { "America/Argentina/La_Rioja", Argentina },
             { "America/Argentina/San_Juan", Argentina },
-            { "America/Argentina/San_Luis", Argentina_Western },
+            { "America/Argentina/San_Luis", Argentina },
             { "America/Indiana/Petersburg", America_Eastern },
             { "timezone.excity.Asia/Amman", "Amman" },
             { "timezone.excity.Asia/Aqtau", "Aqtau" },
@@ -1888,6 +1851,7 @@ public class TimeZoneNames_nl extends TimeZoneNamesBundle {
             { "timezone.excity.Asia/Jayapura", "Jayapura" },
             { "timezone.excity.Asia/Khandyga", "Khandyga" },
             { "timezone.excity.Asia/Makassar", "Makassar" },
+            { "timezone.excity.Asia/Qostanay", "Qostanay" },
             { "timezone.excity.Asia/Sakhalin", "Sachalin" },
             { "timezone.excity.Asia/Shanghai", "Sjanghai" },
             { "timezone.excity.Asia/Tashkent", "Tasjkent" },
@@ -1937,6 +1901,7 @@ public class TimeZoneNames_nl extends TimeZoneNamesBundle {
             { "timezone.excity.America/La_Paz", "La Paz" },
             { "timezone.excity.America/Maceio", "Macei\u00f3" },
             { "timezone.excity.America/Manaus", "Manaus" },
+            { "timezone.excity.America/Merida", "M\u00e9rida" },
             { "timezone.excity.America/Nassau", "Nassau" },
             { "timezone.excity.America/Panama", "Panama" },
             { "timezone.excity.America/Recife", "Recife" },
@@ -2038,6 +2003,9 @@ public class TimeZoneNames_nl extends TimeZoneNamesBundle {
             { "timezone.excity.America/Resolute", "Resolute" },
             { "timezone.excity.America/Santarem", "Santarem" },
             { "timezone.excity.America/Santiago", "Santiago" },
+            { "timezone.excity.America/St_Johns", "Saint John\u2019s" },
+            { "timezone.excity.America/St_Kitts", "Saint Kitts" },
+            { "timezone.excity.America/St_Lucia", "Saint Lucia" },
             { "timezone.excity.America/Winnipeg", "Winnipeg" },
             { "timezone.excity.Antarctica/Casey", "Casey" },
             { "timezone.excity.Antarctica/Davis", "Davis" },
@@ -2091,6 +2059,7 @@ public class TimeZoneNames_nl extends TimeZoneNamesBundle {
             { "timezone.excity.America/Menominee", "Menominee" },
             { "timezone.excity.America/Monterrey", "Monterrey" },
             { "timezone.excity.America/Sao_Paulo", "S\u00e3o Paulo" },
+            { "timezone.excity.America/St_Thomas", "Saint Thomas" },
             { "timezone.excity.America/Vancouver", "Vancouver" },
             { "timezone.excity.Antarctica/Mawson", "Mawson" },
             { "timezone.excity.Antarctica/Palmer", "Palmer" },
@@ -2125,6 +2094,7 @@ public class TimeZoneNames_nl extends TimeZoneNamesBundle {
             { "timezone.excity.America/Montserrat", "Montserrat" },
             { "timezone.excity.America/Paramaribo", "Paramaribo" },
             { "timezone.excity.America/Rio_Branco", "Rio Branco" },
+            { "timezone.excity.America/St_Vincent", "Saint Vincent" },
             { "timezone.excity.America/Whitehorse", "Whitehorse" },
             { "timezone.excity.Antarctica/McMurdo", "McMurdo" },
             { "timezone.excity.Antarctica/Rothera", "Rothera" },
@@ -2176,6 +2146,7 @@ public class TimeZoneNames_nl extends TimeZoneNamesBundle {
             { "timezone.excity.America/St_Barthelemy", "Saint-Barth\u00e9lemy" },
             { "timezone.excity.America/Swift_Current", "Swift Current" },
             { "timezone.excity.Australia/Broken_Hill", "Broken Hill" },
+            { "timezone.excity.America/Bahia_Banderas", "Bah\u00eda de Banderas" },
             { "timezone.excity.America/Port-au-Prince", "Port-au-Prince" },
             { "timezone.excity.Atlantic/South_Georgia", "Zuid-Georgia" },
             { "timezone.excity.America/Argentina/Salta", "Salta" },
